#include "BenutzerHelper.h"

#include <iostream>

#include <mysql++.h>

#include "BenutzerController.h"
#include "DataHelper.h"
#include "QueryHelper.h"
#include "TechnicalAttributeManager.h"
#include "VereinsDatenBerechtigung.h"
#include "VereinsDatenBerechtigungController.h"


void BenutzerHelper::aktualisiereDBBenutzer(DatabaseContext *context, Benutzer *benutzer, bool onlyPassword) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString passwort = (benutzer->hasPassword() ? benutzer->getPassword() : "keines");

	QString passwordForLog = (benutzer->hasPassword() ? "(password given)" : "(no password given)");
	context->log("Benutzer", "BenutzerHelper", 7, "Updating db user with name " + dbBenutzerName + " " + passwordForLog);

	mysqlpp::Connection *connection = context->getConnection();
	bool isInDatabase;
	int userFindNumRows = -1;
	if (!onlyPassword) {
		mysqlpp::Query userFindQuery = connection->query();
		QString userFindDebugQueryString = QString();

		try {
			QueryHelper::addToQuery(&userFindQuery, &userFindDebugQueryString, "SELECT user from mysql.user WHERE user = '" + dbBenutzerName + "'");
			userFindNumRows = QueryHelper::executeQuery(context, &userFindQuery);
			context->logQuery("BenutzerHelper", userFindNumRows, "USER", "", userFindDebugQueryString, "", true);
		} catch (mysqlpp::Exception e) {
			QString infoString = "Database error: " + QString(e.what());
			context->logQuery("BenutzerHelper", 1, "USER", infoString, userFindDebugQueryString, "mysql.user", true);

			QString shortText = QObject::trUtf8("Interner Datenbankfehler");
			QString longText = QObject::trUtf8("Es ist ein interner Datenbankfehler aufgetreten: \n\n") + e.what() + QObject::trUtf8("\n\nBitte kontaktieren Sie den Administrator.");
			throw PersistenceException(shortText, longText);
		}
		isInDatabase = (userFindNumRows > 0);
	} else {
		isInDatabase = true;
	}
	context->log("Benutzer", "BenutzerHelper", 4, "User currently is " + QString(isInDatabase ? "" : " not ") + " in database, found " + QString::number(userFindNumRows) + " entries. "
											   + "Setting onlyPassword is " + QString(onlyPassword ? "true" : "false"));

	if (dbBenutzerName != "root") {
		if (!isInDatabase) {
			QString debugQueryString = "";
			QString flushDebugString = "";
			try {
				mysqlpp::Query query = connection->query();


				QueryHelper::addToQuery(&query,
										"CREATE USER " + dbBenutzerName	+ " IDENTIFIED BY '" + passwort + "'",
										&debugQueryString,
										"CREATE USER " + dbBenutzerName	+ " IDENTIFIED BY '" + "password replaced" + "'");

				int numRows = QueryHelper::executeQuery(context, &query);
				context->logQuery("BenutzerHelper", numRows, "USER", "", debugQueryString, "", true);
				context->log("Benutzer", "BenutzerHelper", 4, "Created db user " + dbBenutzerName + ", will now set privileges...");

				setPrivileges(context, benutzer);

				mysqlpp::Query flushQuery = connection->query();
				QueryHelper::addToQuery(&flushQuery, &flushDebugString, "FLUSH PRIVILEGES");
				QueryHelper::executeQuery(context, &flushQuery);
				context->logQuery("BenutzerHelper", 0, "USER", "", flushDebugString, "", true);
			} catch (mysqlpp::Exception e) {
				QString infoString = "Database error: " + QString(e.what());
				context->logQuery("BenutzerHelper", 1, "USER", infoString, debugQueryString, "mysql.user", true);
				context->logQuery("BenutzerHelper", 0, "USER", infoString, flushDebugString, "mysql.user", true);

				QString shortText = QObject::trUtf8("Interner Datenbankfehler");
				QString longText = QObject::trUtf8("Es ist ein interner Datenbankfehler aufgetreten: \n\n") + e.what() + QObject::trUtf8("\n\nBitte kontaktieren Sie den Administrator.");
				throw PersistenceException(shortText, longText);
			}
		} else {
			QString debugQueryString = QString();
			QString flushDebugString = "";
			try {
				mysqlpp::Connection *connection = context->getConnection();
				mysqlpp::Query query = connection->query();

				QueryHelper::addToQuery(&query,
										"SET PASSWORD FOR " + dbBenutzerName + " = PASSWORD('" + passwort + "');",
										&debugQueryString,
										"SET PASSWORD FOR " + dbBenutzerName + " = PASSWORD('" + "password replaced" + "');");

				int numRows = QueryHelper::executeQuery(context, &query);
				context->logQuery("BenutzerHelper", numRows, "USER", "", debugQueryString, "", true);
				context->log("Benutzer", "BenutzerHelper", 4, "User " + dbBenutzerName + " was already in database, we have set his password.");

//				mysqlpp::Query flushQuery = connection->query();
//				QueryHelper::addToQuery(&flushQuery, &flushDebugString, "FLUSH PRIVILEGES");
//				QueryHelper::executeQuery(context, &flushQuery);
				context->logQuery("BenutzerHelper", 0, "USER", "", flushDebugString, "", true);
			} catch (mysqlpp::Exception e) {
				QString infoString = "Database error: " + QString(e.what());
				context->logQuery("BenutzerHelper", 1, "USER", infoString, debugQueryString, "mysql.user", true);
				context->logQuery("BenutzerHelper", 0, "USER", infoString, flushDebugString, "mysql.user", true);

				QString shortText = QObject::trUtf8("Interner Datenbankfehler");
				QString longText = QObject::trUtf8("Es ist ein interner Datenbankfehler aufgetreten: \n\n") + e.what() + QObject::trUtf8("\n\nBitte kontaktieren Sie den Administrator.");
				throw PersistenceException(shortText, longText);
			}
		}
	}
}

void BenutzerHelper::aktualisiereDBBenutzerBerechtigungen(DatabaseContext *context, Benutzer *benutzer) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	context->log("Benutzer", "BenutzerHelper", 7, "Updating only privileges of db user with name " + dbBenutzerName);

	setPrivileges(context, benutzer);

	QString flushDebugString = "";
	mysqlpp::Connection *connection = context->getConnection();
	mysqlpp::Query flushQuery = connection->query();
	QueryHelper::addToQuery(&flushQuery, &flushDebugString, "FLUSH PRIVILEGES");
	QueryHelper::executeQuery(context, &flushQuery);
	context->logQuery("BenutzerHelper", 0, "USER", "", flushDebugString, "", true);
	context->log("Benutzer", "BenutzerHelper", 7, "Updating only privileges of db user with name " + dbBenutzerName + " done.");
}

void BenutzerHelper::deleteBenutzer(DatabaseContext *context, Benutzer *benutzer) {
	BenutzerController benutzerController = BenutzerController();
	VereinsDatenBerechtigungController vdbController = VereinsDatenBerechtigungController();

	std::vector<VereinsDatenBerechtigung*> berechtigungen = vdbController.getByBenutzer(context, benutzer);
	vdbController.remove(context, berechtigungen);
	DataHelper::deleteVectorContents(berechtigungen);

	QString benutzername = (benutzer->hasBenutzername() ? benutzer->getBenutzername() : "(ohne Benutzername)");
	QString debugQueryString = QString();
	QString flushDebugString = "";

	try {
		mysqlpp::Connection *connection = context->getConnection();
		mysqlpp::Query query = connection->query();
		QString dbBenutzerName = getDBBenutzerName(context, benutzer);
		QString queryString = QString("DROP USER '") + dbBenutzerName + QString("';");
		QueryHelper::addToQuery(&query, &debugQueryString, queryString);
		int numRows = QueryHelper::executeQuery(context, &query);
		context->logQuery("BenutzerHelper", numRows, "USER", "", debugQueryString, "", true);
		context->log("Benutzer", "BenutzerHelper", 7, "Deleted user jdb_" + benutzername + " from database.");

		mysqlpp::Query flushQuery = connection->query();
		QueryHelper::addToQuery(&query, &flushDebugString, "FLUSH PRIVILEGES");
		QueryHelper::executeQuery(context, &query);
		context->logQuery("BenutzerHelper", 0, "USER", "", flushDebugString, "", true);
	} catch (mysqlpp::Exception e) {
		QString infoString = "Database error: " + QString(e.what());
		context->logQuery("BenutzerHelper", 1, "USER", infoString, debugQueryString, "mysql.user", true);
		context->logQuery("BenutzerHelper", 0, "USER", infoString, flushDebugString, "mysql.user", true);

		QString shortText = QObject::trUtf8("Interner Datenbankfehler");
		QString longText = QObject::trUtf8("Es ist ein interner Datenbankfehler aufgetreten: \n\n") + e.what() + QObject::trUtf8("\n\nBitte kontaktieren Sie den Administrator.");
		throw PersistenceException(shortText, longText);
	}

	benutzerController.remove(context, benutzer);
}

void BenutzerHelper::setPrivileges(DatabaseContext *context, Benutzer *benutzer) {
	QString benutzerName = benutzer->getBenutzername();
	bool webBenutzer = benutzer->hasWebBenutzer() && benutzer->getWebBenutzer();
	bool guiBenutzer = benutzer->hasGuiBenutzer() && benutzer->getGuiBenutzer();

	if (benutzerName != "root") {
		revokeAll(context, benutzer);
		if (webBenutzer && !guiBenutzer) {
			context->log("Benutzer", "BenutzerHelper", 5, "Setting privileges for webBenutzer /!guiBenutzer...");
			setPrivilegesForWebOnlyUser(context, benutzer);
		} else if (guiBenutzer) {
			context->log("Benutzer", "BenutzerHelper", 5, "Setting privileges for guiBenutzer...");
			setPrivilegesForStandardUser(context, benutzer);
	    } else {
			context->log("Benutzer", "BenutzerHelper", 5, "Setting no privileges as neither gui nor webBenutzer...");
			setPrivilegesForNothingUser(context, benutzer);
		}
	} else {
		setPrivilegesForAdmin(context, benutzer);
	}
}

void BenutzerHelper::setPrivilegesForNothingUser(DatabaseContext *context, Benutzer *benutzer) {
	// Outcommented means: Deliberately no permissions for the respective table
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges for nothing user...");
	grantReadOnlyTableAccess(context, benutzer, "Benutzer");
//	grantFullTableAccess(context, benutzer, "Bild");
//	grantFullTableAccess(context, benutzer, "BildDaten");
	grantReadOnlyTableAccess(context, benutzer, "CodeType");
	grantReadOnlyTableAccess(context, benutzer, "CodeValue");
	grantReadOnlyTableAccess(context, benutzer, "DataModelInfo");
//	grantFullTableAccess(context, benutzer, "Durchgang");
//	grantFullTableAccess(context, benutzer, "EinzelWertung");
	grantInsertOnlyTableAccess(context, benutzer, "EventLog");
	grantReadOnlyTableAccess(context, benutzer, "EventLogCategory");
	grantReadOnlyTableAccess(context, benutzer, "EventLogSettings");
//	grantFullTableAccess(context, benutzer, "ExternerJuror");
//	grantFullTableAccess(context, benutzer, "FotographischerTitel");
//	grantFullTableAccess(context, benutzer, "Funktionstraeger");
	grantReadOnlyTableAccess(context, benutzer, "GlobaleEinstellung");
//	grantFullTableAccess(context, benutzer, "JuriertBei");
//	grantFullTableAccess(context, benutzer, "Jurierung");
//	grantFullTableAccess(context, benutzer, "Mitglied");
	grantInsertOnlyTableAccess(context, benutzer, "QueryLog");
	grantReadOnlyTableAccess(context, benutzer, "QueryLogSettings");
	grantInsertUpdateTableAccess(context, benutzer, "Session");
//	grantReadOnlyTableAccess(context, benutzer, "Sparte");
//	grantFullTableAccess(context, benutzer, "TeilnehmerKombinationsWertung");
//	grantFullTableAccess(context, benutzer, "TeilnehmerWertung");
//	grantFullTableAccess(context, benutzer, "Verein");
	grantReadOnlyTableAccess(context, benutzer, "VereinsDatenBerechtigung");
//	grantFullTableAccess(context, benutzer, "VereinsKombinationsWertung");
//	grantFullTableAccess(context, benutzer, "VereinsWertung");
//	grantFullTableAccess(context, benutzer, "Wertung");
//	grantReadOnlyTableAccess(context, benutzer, "Wettbewerb");
//	grantFullTableAccess(context, benutzer, "WettbewerbsTeilnahme");
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges finished.");
}

void BenutzerHelper::setPrivilegesForWebOnlyUser(DatabaseContext *context, Benutzer *benutzer) {
	// Outcommented means: Deliberately no permissions for the respective table
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges for web only user...");
	grantReadOnlyTableAccess(context, benutzer, "Benutzer");
	grantFullTableAccess(context, benutzer, "Bild");
	grantFullTableAccess(context, benutzer, "BildDaten");
	grantReadOnlyTableAccess(context, benutzer, "CodeType");
	grantReadOnlyTableAccess(context, benutzer, "CodeValue");
	grantReadOnlyTableAccess(context, benutzer, "DataModelInfo");
//	grantFullTableAccess(context, benutzer, "Durchgang");
//	grantFullTableAccess(context, benutzer, "EinzelWertung");
	grantInsertOnlyTableAccess(context, benutzer, "EventLog");
	grantReadOnlyTableAccess(context, benutzer, "EventLogCategory");
	grantReadOnlyTableAccess(context, benutzer, "EventLogSettings");
//	grantFullTableAccess(context, benutzer, "ExternerJuror");
//	grantFullTableAccess(context, benutzer, "FotographischerTitel");
//	grantFullTableAccess(context, benutzer, "Funktionstraeger");
	grantReadOnlyTableAccess(context, benutzer, "GlobaleEinstellung");
//	grantFullTableAccess(context, benutzer, "JuriertBei");
//	grantFullTableAccess(context, benutzer, "Jurierung");
	grantFullTableAccess(context, benutzer, "Mitglied");
	grantInsertOnlyTableAccess(context, benutzer, "QueryLog");
	grantReadOnlyTableAccess(context, benutzer, "QueryLogSettings");
	grantInsertUpdateTableAccess(context, benutzer, "Session");
	grantReadOnlyTableAccess(context, benutzer, "Sparte");
	grantReadOnlyTableAccess(context, benutzer, "StatistikData");
//	grantFullTableAccess(context, benutzer, "TeilnehmerKombinationsWertung");
//	grantFullTableAccess(context, benutzer, "TeilnehmerWertung");
//	grantFullTableAccess(context, benutzer, "Verein");
	grantReadOnlyTableAccess(context, benutzer, "VereinsDatenBerechtigung");
//	grantFullTableAccess(context, benutzer, "VereinsKombinationsWertung");
//	grantFullTableAccess(context, benutzer, "VereinsWertung");
//	grantFullTableAccess(context, benutzer, "Wertung");
	grantReadOnlyTableAccess(context, benutzer, "Wettbewerb");
	grantFullTableAccess(context, benutzer, "WettbewerbsTeilnahme");
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges finished.");
}

void BenutzerHelper::setPrivilegesForStandardUser(DatabaseContext *context, Benutzer *benutzer) {
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges for standard user...");
	grantReadOnlyTableAccess(context, benutzer, "Benutzer");
	grantFullTableAccess(context, benutzer, "Bild");
	grantFullTableAccess(context, benutzer, "BildDaten");
	grantReadOnlyTableAccess(context, benutzer, "CodeType");
	grantReadOnlyTableAccess(context, benutzer, "CodeValue");
	grantReadOnlyTableAccess(context, benutzer, "DataModelInfo");
	grantFullTableAccess(context, benutzer, "Durchgang");
	grantFullTableAccess(context, benutzer, "EinzelWertung");
	grantInsertOnlyTableAccess(context, benutzer, "EventLog");
	grantReadOnlyTableAccess(context, benutzer, "EventLogCategory");
	grantReadOnlyTableAccess(context, benutzer, "EventLogSettings");
	grantFullTableAccess(context, benutzer, "ExternerJuror");
	grantFullTableAccess(context, benutzer, "FotographischerTitel");
	grantFullTableAccess(context, benutzer, "Funktionstraeger");
	grantReadOnlyTableAccess(context, benutzer, "GlobaleEinstellung");
	grantFullTableAccess(context, benutzer, "JuriertBei");
	grantFullTableAccess(context, benutzer, "Jurierung");
	grantFullTableAccess(context, benutzer, "Mitglied");
	grantInsertOnlyTableAccess(context, benutzer, "QueryLog");
	grantReadOnlyTableAccess(context, benutzer, "QueryLogSettings");
	grantInsertUpdateTableAccess(context, benutzer, "Session");
	grantFullTableAccess(context, benutzer, "Sparte");
	grantFullTableAccess(context, benutzer, "StatistikData");
	grantFullTableAccess(context, benutzer, "TeilnehmerKombinationsWertung");
	grantFullTableAccess(context, benutzer, "TeilnehmerWertung");
	grantFullTableAccess(context, benutzer, "Verein");
	grantReadOnlyTableAccess(context, benutzer, "VereinsDatenBerechtigung");
	grantFullTableAccess(context, benutzer, "VereinsKombinationsWertung");
	grantFullTableAccess(context, benutzer, "VereinsWertung");
	grantFullTableAccess(context, benutzer, "Wertung");
	grantFullTableAccess(context, benutzer, "Wettbewerb");
	grantFullTableAccess(context, benutzer, "WettbewerbsTeilnahme");
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges finished.");
}

void BenutzerHelper::setPrivilegesForAdmin(DatabaseContext *context, Benutzer *benutzer) {
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges for root...");
	grantFullTableAccess(context, benutzer, "Benutzer");
	grantFullTableAccess(context, benutzer, "Bild");
	grantFullTableAccess(context, benutzer, "BildDaten");
	grantFullTableAccess(context, benutzer, "CodeType");
	grantFullTableAccess(context, benutzer, "CodeValue");
	grantFullTableAccess(context, benutzer, "DataModelInfo");
	grantFullTableAccess(context, benutzer, "Durchgang");
	grantFullTableAccess(context, benutzer, "EinzelWertung");
	grantReadInsertTableAccess(context, benutzer, "EventLog");
	grantFullTableAccess(context, benutzer, "EventLogCategory");
	grantFullTableAccess(context, benutzer, "EventLogSettings");
	grantFullTableAccess(context, benutzer, "ExternerJuror");
	grantFullTableAccess(context, benutzer, "FotographischerTitel");
	grantFullTableAccess(context, benutzer, "Funktionstraeger");
	grantFullTableAccess(context, benutzer, "GlobaleEinstellung");
	grantFullTableAccess(context, benutzer, "JuriertBei");
	grantFullTableAccess(context, benutzer, "Jurierung");
	grantFullTableAccess(context, benutzer, "Mitglied");
	grantReadInsertTableAccess(context, benutzer, "QueryLog");
	grantFullTableAccess(context, benutzer, "QueryLogSettings");
	grantFullTableAccess(context, benutzer, "Session");
	grantFullTableAccess(context, benutzer, "Sparte");
	grantFullTableAccess(context, benutzer, "StatistikData");
	grantFullTableAccess(context, benutzer, "TeilnehmerKombinationsWertung");
	grantFullTableAccess(context, benutzer, "TeilnehmerWertung");
	grantFullTableAccess(context, benutzer, "Verein");
	grantFullTableAccess(context, benutzer, "VereinsDatenBerechtigung");
	grantFullTableAccess(context, benutzer, "VereinsKombinationsWertung");
	grantFullTableAccess(context, benutzer, "VereinsWertung");
	grantFullTableAccess(context, benutzer, "Wertung");
	grantFullTableAccess(context, benutzer, "Wettbewerb");
	grantFullTableAccess(context, benutzer, "WettbewerbsTeilnahme");
	context->log("Benutzer", "BenutzerHelper", 4, "Setting privileges finished.");
}

void BenutzerHelper::revokeAllPrivileges(DatabaseContext *context, Benutzer *benutzer) {
	context->log("Benutzer", "BenutzerHelper", 4, "Revoking all privileges for standard user...");
	revokeTableAccess(context, benutzer, "Benutzer");
	revokeTableAccess(context, benutzer, "Bild");
	revokeTableAccess(context, benutzer, "BildDaten");
	revokeTableAccess(context, benutzer, "CodeType");
	revokeTableAccess(context, benutzer, "CodeValue");
	revokeTableAccess(context, benutzer, "DataModelInfo");
	revokeTableAccess(context, benutzer, "Durchgang");
	revokeTableAccess(context, benutzer, "EinzelWertung");
	revokeTableAccess(context, benutzer, "EventLog");
	revokeTableAccess(context, benutzer, "EventLogCategory");
	revokeTableAccess(context, benutzer, "EventLogSettings");
	revokeTableAccess(context, benutzer, "ExternerJuror");
	revokeTableAccess(context, benutzer, "FotographischerTitel");
	revokeTableAccess(context, benutzer, "Funktionstraeger");
	revokeTableAccess(context, benutzer, "GlobaleEinstellung");
	revokeTableAccess(context, benutzer, "JuriertBei");
	revokeTableAccess(context, benutzer, "Jurierung");
	revokeTableAccess(context, benutzer, "Mitglied");
	revokeTableAccess(context, benutzer, "QueryLog");
	revokeTableAccess(context, benutzer, "QueryLogSettings");
	revokeTableAccess(context, benutzer, "Session");
	revokeTableAccess(context, benutzer, "Sparte");
	revokeTableAccess(context, benutzer, "StatistikData");
	revokeTableAccess(context, benutzer, "TeilnehmerKombinationsWertung");
	revokeTableAccess(context, benutzer, "TeilnehmerWertung");
	revokeTableAccess(context, benutzer, "Verein");
	revokeTableAccess(context, benutzer, "VereinsDatenBerechtigung");
	revokeTableAccess(context, benutzer, "VereinsKombinationsWertung");
	revokeTableAccess(context, benutzer, "VereinsWertung");
	revokeTableAccess(context, benutzer, "Wertung");
	revokeTableAccess(context, benutzer, "Wettbewerb");
	revokeTableAccess(context, benutzer, "WettbewerbsTeilnahme");
	context->log("Benutzer", "BenutzerHelper", 4, "Revoking privileges finished.");
}

QString BenutzerHelper::getDBBenutzerName(DatabaseContext *context, Benutzer *benutzer) {
	// caution when altering - redundant code within DatabaseContext
	QString dbName = context->getDatabaseName();
	QString benutzerName = benutzer->getBenutzername();
	if (benutzerName != "root") {
		return dbName + "_" + benutzerName;
	} else {
		// there is only one root
		return benutzerName;
	}
}

void BenutzerHelper::grantFullTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "GRANT INSERT, UPDATE, SELECT, DELETE ON "
						+ databaseName + "." + entityDBName + " TO " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Granting full table access for table " + entityDBName + " to user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::grantReadOnlyTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "GRANT SELECT ON "
						+ databaseName + "." + entityDBName + " TO " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Granting read only table access for table " + entityDBName + " to user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::grantInsertOnlyTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "GRANT INSERT ON "
						+ databaseName + "." + entityDBName + " TO " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Granting insert only table access for table " + entityDBName + " to user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::grantInsertUpdateTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "GRANT INSERT, UPDATE ON "
						+ databaseName + "." + entityDBName + " TO " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Granting insert and update table access for table " + entityDBName + " to user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::grantReadInsertTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "GRANT SELECT, INSERT ON "
						+ databaseName + "." + entityDBName + " TO " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Granting read and insert table access for table " + entityDBName + " to user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::revokeAll(DatabaseContext *context, Benutzer *benutzer) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "REVOKE ALL PRIVILEGES, GRANT OPTION FROM " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Revoking all privileges from user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}

void BenutzerHelper::revokeTableAccess(DatabaseContext *context, Benutzer *benutzer, QString entityDBName) {
	QString dbBenutzerName = getDBBenutzerName(context, benutzer);
	QString databaseName = context->getDatabaseName();
	QString queryString = "REVOKE all ON "
						+ databaseName + "." + entityDBName + " FROM " + dbBenutzerName;
	context->log("Benutzer", "BenutzerHelper", 3, "==> Revoking all privileges for table " + entityDBName + " from user " + dbBenutzerName);
	QueryHelper::executeQuery(context, queryString);
}
