#ifndef ARCHIVEHELPER_H
#define ARCHIVEHELPER_H

#include <QDir>
#include <QObject>
#include <QString>
#include <QStringList>

#include "DatabaseContext.h"
#include "ExportCallStrategy.h"
#include "ImportResult.h"

#include <archive.h>

class ArchiveHelper {

public:
	enum ARCHIVE_TYPE {
		ZIP = 1
	};

	static QVariant exportData(QWidget *parent, DatabaseContext *context, ExportCallStrategy *exportCallStrategy);
	static QVariant exportData(QWidget *parent, DatabaseContext *context, ExportCallStrategy *exportCallStrategy, QStringList fileFormats);

	static QDir createArchiveTempDirectoryForFile(DatabaseContext *context, QString fileName, QString prefix = QObject::trUtf8("tmp"));
	static QDir createArchiveTempDirectory(DatabaseContext *context, QDir baseDir, QString prefix = QObject::trUtf8("tmp"));
	static void writeArchive(DatabaseContext *context, QDir archiveTempDir, QDir outputDir, QString outputFileNamePrefix,
							  ArchiveHelper::ARCHIVE_TYPE archiveType, bool deleteTempDirectory);

	static bool isArchive(QString filePathName);
	static QString prepareForArchiveImport(DatabaseContext *context, QDir archiveTempDir, QString filePathName);


//	static ImportResult* importArchive(DatabaseContext *context, QDir archiveTempDir, QString filePathName, bool deleteTempDirectory);
	static void uncompressArchive(DatabaseContext *context, QDir archiveTempDir, QString filePathName);
	static void deleteTempDir(DatabaseContext *context, QDir archiveTempDir);

	static QString convertFileNameForOutput(QString fileName);

private:
	static QString getArchiveName(DatabaseContext *context, QString prefix, ArchiveHelper::ARCHIVE_TYPE archiveType);
	static ArchiveHelper::ARCHIVE_TYPE getArchiveType(DatabaseContext *context, QString fileName);
	static void setArchiveMode(DatabaseContext *context, struct archive *a, ArchiveHelper::ARCHIVE_TYPE archiveType);

	static void deleteOldArchive(DatabaseContext *context, QDir dir, QString fileName);

	static struct archive* prepareArchive(DatabaseContext *context, std::string outputFilePath, ArchiveHelper::ARCHIVE_TYPE archiveType);
	static struct archive_entry* prepareArchiveEntry(DatabaseContext *context, struct archive *a, std::string inputFile, std::string absoluteInputFile);

	static void processReturnCode(DatabaseContext *context, QString title, struct archive *a, int okLevel, int failLevel);
	static void processReturnCode(DatabaseContext *context, QString title, int code, struct archive *a, int okLevel, int failLevel);


};
#endif
