#include "DataSet.h"

#include <iostream>

#include "AttributeData.h"

DataSet::DataSet() {
	entityNameToAttributes = std::map<QString, std::vector<QString> >();
	entityDBNameToData = std::map<QString, std::vector<DataTuple*>* >();
	entityNames = std::set<QString>();
	completeEntityDBNames = std::set<QString>();
}

DataSet::DataSet(std::map<QString, ImpExpSchema*> entityDBNameToSchema) {
	this->entityDBNameToSchema = entityDBNameToSchema;
}

void DataSet::setEntityDBNameToSchemaMap(std::map<QString, ImpExpSchema*> entityDBNameToSchema) {
	this->entityDBNameToSchema = entityDBNameToSchema;
}

std::map<QString, ImpExpSchema*> DataSet::getEntityDBNameToSchemaMap() {
	return entityDBNameToSchema;
}

ImpExpSchema* DataSet::getSchema(QString entityDBName) {
	return entityDBNameToSchema[entityDBName];
}

void DataSet::setEntityNameToAttributesMap(std::map<QString, std::vector<QString> > entityNameToAttributes) {
	hasEntityNameToAttributes = true;
	this->entityNameToAttributes = entityNameToAttributes;
	entityNames = std::set<QString>();

	for (std::map<QString, std::vector<QString> >::iterator it = entityNameToAttributes.begin();
		it != entityNameToAttributes.end(); it++) {
		QString entityDBName = it->first;
		entityNames.insert(entityDBName);
	}
}

std::map<QString, std::vector<QString> > DataSet::getEntityNameToAttributesMap() {
	return entityNameToAttributes;
}

bool DataSet::hasEntityNameToAttributesMap() {
	return hasEntityNameToAttributes;
}

std::vector<QString> DataSet::getAttributes(QString entityDBName) {
	return entityNameToAttributes[entityDBName];
}

void DataSet::addDataTuple(QString entityDBName, DataTuple* dataTuple) {
	if (entityDBNameToData.find(entityDBName) == entityDBNameToData.end()) {
		entityDBNameToData[entityDBName] = new std::vector<DataTuple*>();
	}

	std::vector<DataTuple*>* tuplesForEntity = entityDBNameToData[entityDBName];
	tuplesForEntity->push_back(dataTuple);
	entityNames.insert(entityDBName);
}

void DataSet::saveAttributeData() {
	for (std::map<QString, std::vector<DataTuple*>* >::iterator it = entityDBNameToData.begin();	it != entityDBNameToData.end(); it++) {
		std::vector<DataTuple*>* tupleVector = it->second;
		for (std::vector<DataTuple*>::const_iterator it2 = tupleVector->begin(); it2 != tupleVector->end(); it2++) {
			DataTuple *tuple = *it2;
			tuple->saveAttributeData();
		}
	}
}

void DataSet::addAllTuplesOfEntity(QString entityDBName) {
	this->completeEntityDBNames.insert(entityDBName);
}

std::set<QString> DataSet::getEntityDBNamesWithAllTuples() {
	return this->completeEntityDBNames;
}

bool DataSet::isEntityWithAllTuples(QString entityDBName) {
	return (this->completeEntityDBNames.find(entityDBName) != this->completeEntityDBNames.end());
}

void DataSet::removeDataTuple(DataTuple *tuple) {
	bool lastTupleForEntityErased = false;
	for (std::map<QString, std::vector<DataTuple*>* >::iterator it = entityDBNameToData.begin(); it != entityDBNameToData.end(); it++) {
		std::vector<DataTuple*>* tupleVector = it->second;
		for (int n = 0; n < tupleVector->size(); n++) {
			if (tuple == tupleVector->at(n)) {
				tupleVector->erase(tupleVector->begin() + n);
				n--;
			}
		}

/*
		for (std::vector<DataTuple*>::iterator it2 = tupleVector->begin(); it2 != tupleVector->end(); it2++) {
			DataTuple *foundTuple = *it2;
			if (foundTuple == tuple) {
				tupleVector->erase(it2);
			}
		}
*/
		if (tupleVector->size() == 0) {
			entityDBNameToData.erase(it);
			delete tupleVector;
			lastTupleForEntityErased = true;
		}
	}

	QString entityDBName = tuple->getEntityDBName();
	if (lastTupleForEntityErased) {
//		std::cout << "No tuples for entity " << qPrintable(entityDBName) << " left, so deleting meta information..." << std::endl;
		// no tuples of this entities left, so clean up all information for that entity
		if (hasEntityNameToAttributes) {
//			std::cout << "EntityNameToAttributes before erase has size " << entityNameToAttributes.size() << std::endl;
			entityNameToAttributes.erase(entityDBName);
//			std::cout << "==> EntityNameToAttributes afterwards has size " << entityNameToAttributes.size() << std::endl;
		}
//		std::cout << "entityNames before erase has size " << entityNames.size() << std::endl;
		entityNames.erase(entityDBName);
//		std::cout << "==> entityNames afterwards has size " << entityNames.size() << std::endl;

//		std::cout << "entityDBNameToSchema before erase has size " << entityDBNameToSchema.size() << std::endl;
		entityDBNameToSchema.erase(entityDBName);
//		std::cout << "==> entityDBNameToSchema afterwards has size " << entityDBNameToSchema.size() << std::endl;

//		std::cout << "... cleaning up done ..." << std::endl;
	}
}

std::set<QString> DataSet::getAllEntities() {
	return entityNames;
}

std::vector<DataTuple*> DataSet::getAllTuples() {
	std::vector<DataTuple*> tuples = std::vector<DataTuple*>();

	for (std::map<QString, std::vector<DataTuple*>* >::iterator it = entityDBNameToData.begin();	it != entityDBNameToData.end(); it++) {
		std::vector<DataTuple*>* tupleVector = it->second;
		for (std::vector<DataTuple*>::const_iterator it2 = tupleVector->begin(); it2 != tupleVector->end(); it2++) {
			tuples.push_back(*it2);
		}
	}
	return tuples;
}

std::vector<DataTuple*> DataSet::getTuples(QString entityDBName) {
	std::vector<DataTuple*> tuples = std::vector<DataTuple*>();

	if (entityDBNameToData.find(entityDBName) != entityDBNameToData.end()) {
		std::vector<DataTuple*>* foundTuples = entityDBNameToData[entityDBName];
		for (std::vector<DataTuple*>::const_iterator it = foundTuples->begin(); it != foundTuples->end(); it++) {
			tuples.push_back(*it);
		}
	}
	return tuples;
}

std::vector<DataTuple*> DataSet::getTuplesByContent(DatabaseContext *context, QString entityDBName, 
													std::map<QString, QVariant> pathKeyToValue,
													std::map<QString, QVariant> pathKeyToForeignKey,
												    std::map<QString, DataTuple*> pathKeyToTuple) {

	std::vector<DataTuple*> foundTuples = std::vector<DataTuple*>();

	std::vector<DataTuple*> tuplesForEntity = getTuples(entityDBName);
	for (std::vector<DataTuple*>::const_iterator it = tuplesForEntity.begin(); it != tuplesForEntity.end(); it++) {
		DataTuple* tuple = *it;
		bool equal = true;
		for (std::map<QString, QVariant>::const_iterator it2 = pathKeyToValue.begin();
			  equal && it2 != pathKeyToValue.end(); it2++) {
			QString pathKey = it2->first;
			QVariant expectedValue = it2->second;
			if (tuple->hasAttributeData(pathKey)) {
				AttributeData *attributeData = tuple->getAttributeData(pathKey);
				if (attributeData->isValue() && attributeData->getValue() == expectedValue) {
					// equal - nothing to do here
				} else {
					equal = false;
				}
			} else {
//				context->log("ImportExport", "DataSet", 7, "Not equal because the is no value path key " + pathKey);
				equal = false;
			}
		}

		for (std::map<QString, QVariant>::const_iterator it2 = pathKeyToForeignKey.begin();
			  equal && it2 != pathKeyToForeignKey.end(); it2++) {
			QString pathKey = it2->first;
			QVariant expectedValue = it2->second;
			if (tuple->hasAttributeData(pathKey)) {
				AttributeData *attributeData = tuple->getAttributeData(pathKey);
				if (attributeData->isForeignKeyToDB() && attributeData->getForeignKeyToDB() == expectedValue) {
					// equal - nothing to do here
				} else {
					equal = false;
				}
			} else {
//				context->log("ImportExport", "DataSet", 7, "Not equal because the is no fk path key " + pathKey);
				equal = false;
			}
		}

		for (std::map<QString, DataTuple*>::const_iterator it2 = pathKeyToTuple.begin();
			  equal && it2 != pathKeyToTuple.end(); it2++) {
			QString pathKey = it2->first;
			DataTuple* expectedValue = it2->second;
			if (tuple->hasAttributeData(pathKey)) {
				AttributeData *attributeData = tuple->getAttributeData(pathKey);
				// Note: We compare pointers here.  The reason is that for each tuple
				// in the import file, exactly one DataTuple instance exists.
				if (!attributeData->isForeignKeyToDataTuple() 
					|| attributeData->getForeignKeyToDataTuple() != expectedValue) {
					// maybe TODO: debug output
					equal = false;
				}
			} else {
//				context->log("ImportExport", "DataSet", 7, "Not equal because the is no tuple path key " + pathKey);
				equal = false;
			}
		}

		// expected values were found in all attributes, so we have found our tuple
		if (equal) {
			foundTuples.push_back(tuple);
		}
	}

	if (foundTuples.size() > 1) {
		QString valueString = "";
		for (std::map<QString, QVariant>::const_iterator it = pathKeyToValue.begin(); it != pathKeyToValue.end(); it++) {
			valueString += (it->first) + "=" + (it->second.toString()) + ", ";
		}
		QString fkString = "";
		for (std::map<QString, QVariant>::const_iterator it = pathKeyToForeignKey.begin(); it != pathKeyToForeignKey.end(); it++) {
			fkString += (it->first) + "=" + (it->second.toString()) + ", ";
		}
		QString tupleString = "";
		for (std::map<QString, DataTuple*>::const_iterator it = pathKeyToTuple.begin(); it != pathKeyToTuple.end(); it++) {
			QString tupleAdress = QString::number((qlonglong)it->second, 16);
			tupleString += (it->first) + "=" + tupleAdress + ", ";
		}
		context->log("ImportExport", "DataSet", 7, "Called getTuplesByContent with parameters entityDBName = " + entityDBName + ", pathKeyToValue = " + valueString
												  + ", pathKeyToForeignKey = " + fkString + ", pathKeyToTuple = " + tupleString + " and found more than one tuple:");

		for (int n = 0; n < foundTuples.size(); n++) {
			DataTuple *tuple = foundTuples[n];
			tuple->printData(context, 7);
		}
	}

	return foundTuples;
}

void DataSet::printStructure(DatabaseContext *context) {
	context->log("ImportExport", "DataSet.cpp", 4, "Started printStructure(DatabaseContext*):");
	for (std::map<QString, ImpExpSchema*>::const_iterator it = entityDBNameToSchema.begin();
		  it != entityDBNameToSchema.end(); it++) {
		ImpExpSchema *schema = it->second;
		schema->printStructure(context);
	}
	context->log("ImportExport", "DataSet.cpp", 4, "Finished printStructure(DatabaseContext*):");
}

void DataSet::printData(DatabaseContext *context) {
	context->log("ImportExport", "DataSet.cpp", 4, "Started printData(DatabaseContext*):");

	for (std::map<QString, std::vector<DataTuple*>* >::iterator it = entityDBNameToData.begin();	it != entityDBNameToData.end(); it++) {
		std::vector<DataTuple*>* tupleVector = it->second;
		for (std::vector<DataTuple*>::const_iterator it2 = tupleVector->begin(); it2 != tupleVector->end(); it2++) {
			DataTuple *tuple = *it2;
			tuple->printData(context);
		}
	}
	context->log("ImportExport", "DataSet.cpp", 4, "Finished printData(DatabaseContext*):");
}
