#ifndef DATASET_H
#define DATASET_H

#include "DatabaseContext.h"
#include "DataTuple.h"
#include "ImpExpSchema.h"

#include <QString>
#include <QVariant>

#include <map>
#include <set>
#include <vector>

class DataSet {

public:
	DataSet();
	DataSet(std::map<QString, ImpExpSchema*> entityDBNameToSchema);

	void setEntityDBNameToSchemaMap(std::map<QString, ImpExpSchema*> entityDBNameToSchema);
	std::map<QString, ImpExpSchema*> getEntityDBNameToSchemaMap();
	ImpExpSchema* getSchema(QString entityDBName);

	void setEntityNameToAttributesMap(std::map<QString, std::vector<QString> > entityNameToAttributes);
	bool hasEntityNameToAttributesMap();
	std::map<QString, std::vector<QString> > getEntityNameToAttributesMap();
	std::vector<QString> getAttributes(QString entityDBName);

	void addDataTuple(QString entityDBName, DataTuple* dataTuple);
	void saveAttributeData();

	void addAllTuplesOfEntity(QString entityDBName);
	std::set<QString> getEntityDBNamesWithAllTuples();
	bool isEntityWithAllTuples(QString entityDBName);

	void removeDataTuple(DataTuple *tuple);

	/** Returns a set with the db names of all entities in this data set.
	 *  I.e. all entities which are type of at least one tuple in this
	 *  DataSet must be present in this set.
	 *  @return a set with the db names of all entities in this data set
	 */
	std::set<QString> getAllEntities();
	std::vector<DataTuple*> getAllTuples();
	std::vector<DataTuple*> getTuples(QString entity);

	std::vector<DataTuple*> getTuplesByContent(DatabaseContext *context,
											   QString entityDBName, std::map<QString, QVariant> pathKeyToValue,
											   std::map<QString, QVariant> pathKeyToForeignKey,
											   std::map<QString, DataTuple*> pathKeyToTuple);

	void printStructure(DatabaseContext *context);
	void printData(DatabaseContext *context);

private:
	bool hasEntityNameToAttributes;
	std::set<QString> entityNames;
	std::map<QString, std::vector<QString> > entityNameToAttributes;
	std::map<QString, std::vector<DataTuple*>* > entityDBNameToData;
	std::set<QString> completeEntityDBNames;
	std::map<QString, ImpExpSchema*> entityDBNameToSchema;

};
#endif


/* Evtl. here: 	std::set<QString> getEntityDBNamesToExport(DataSet *dataSet); 

std::set<QString> CSVDataExporter::getEntityDBNamesToExport(DataSet *dataSet) {
	std::set<QString> entityDBNames = std::set<QString>();

	std::vector<DataTuple*> tuples = dataSet->getAllTuples();
	for (std::vector<DataTuple*>::const_iterator it = tuples.begin(); it != tuples.end(); it++) {
		DataTuple* dataTuple = *it;
		QString entityDBName = dataTuple->getEntityDBName();
		entityDBNames.insert(entityDBName);
	}
	return entityDBNames;
}

*/
