#ifndef DATATUPLE_H
#define DATATUPLE_H

#include <QObject>
#include <QString>
#include <QVariant>

#include <map>
#include <set>

#include "DatabaseContext.h"
#include "Exception.h"
#include "ForeignKeyDestination.h"

class AttributeData;

class DataTuple : public QObject {

public:
	DataTuple(QString entityDBName, int lineNumber, int tupleId);
	DataTuple(QString entityDBName, std::map<QString, QVariant> dbNameToValue, QVariant primaryKey);
	int getLineNumber();
	int getTupleId();

	QString getEntityDBName();
	void setValue(QString dbname, QVariant value);

	void setForeignKeyToDB(QString pathKey, QVariant foreignKey);
	void setForeignKeyToDataTuple(QString pathKey, DataTuple* dataTuple);
	void setNull(QString pathKey);

	void deleteAttributeData(QString pathKey);

	bool hasAttributeData(QString pathKey);
	AttributeData *getAttributeData(QString pathKey);

	std::set<QString> getAllPathKeys();

	void saveAttributeData();
	std::map<QString, QString> getOriginalPathKeyToData();

	void setPrimaryKey(QVariant primaryKey);

	/** Returns the primary key of this DataTuple.
	 *  @return primary key of this DataTuple
	 */
	QVariant getPrimaryKey();

	void setPKMatchPassed(bool pkMatchPassed);
	bool isPKMatchPassed();

	void setDuplicateCheckPassed(bool duplicateCheckPassed);
	bool isDuplicateCheckPassed();

	std::map<QString, QVariant> getAllAttributes();

	void printData(DatabaseContext *context);
	void printData(DatabaseContext *context, int level);

private:
	std::map<QString, AttributeData*> pathKeyToAttributeData;
	std::map<QString, AttributeData*> originalAttributeData;

	QString entityDBName;
	int lineNumber;
	int tupleId;
	QVariant primaryKey;

	bool pkMatchPassed;
	bool duplicateCheckPassed;
};
#endif
