#ifndef FOREIGNKEYEDGE_H
#define FOREIGNKEYEDGE_H

#include <QString>
#include <QVariant>

#include <map>
#include <set>

#include "Attribute.h"
#include "Entity.h"
#include "ForeignKeyNode.h"

/** Instances of this class represent edges in a ForeignKeyGraph.
 *  Edges correspond to attributes in entities and import/export
 *  keys.  Each edge is annotated with a pathKey, which represents
 *  the position in the ForeignKeyGraph.
 *  For more information, see ... (TODO)
 */
class ForeignKeyEdge {

public:
	/** Constructs a new ForeignKeyEdge from the given inNode to
     *  the given outNode for the given attribute and annotates it
	 *  with the given pathKey.
	 *  @param inNode the in node
     *  @param attribute the attribute
     *  @param outNode the out node
     *  @param pathKey the path key
	 */
	ForeignKeyEdge(ForeignKeyNode *inNode, const Attribute *attribute, ForeignKeyNode *outNode, QString pathKey);

	/** Returns the db name of the attribute corresponding to this
	 *  ForeignKeyEdge.
	 *  @return the db name of the attribute corresponding to this
	 *  ForeignKeyEdge.
	 */
	QString getAttributeDBName();

	/** Returns the in node of this ForeignKeyEdge.
	 *  @return the in node of this ForeignKeyEdge.
	 */
	ForeignKeyNode *getInNode();

	/** Returns the out node of this ForeignKeyEdge.
	 *  @return the out node of this ForeignKeyEdge.
	 */
	ForeignKeyNode *getOutNode();

	/** Returns the db name of the entity corresponding to the out node
	 *  of this ForeignKeyEdge.
	 *  @return the db name of the entity corresponding to the out node
	 *  of this ForeignKeyEdge.
	 */
	QString getOutNodeEntityDBName();

	/** Returns the path key of this ForeignKeyEdge.
	 *  @return the path key of this ForeignKeyEdge.
	 */
	QString getPathKey();


private:
	const Attribute *attribute;
	ForeignKeyNode *inNode;
	ForeignKeyNode *outNode;
	QString pathKey;


};
#endif
