#include "ForeignKeyGraph.h"

#include "Entity.h"

ForeignKeyGraph::ForeignKeyGraph(const Entity *entity) {
	this->rootNode = new ForeignKeyNode(entity, QString());
	this->entityDBNameToNode = std::map<QString, ForeignKeyNode*>();
	// see other constructor for why we don't add rootNode to this map
}

ForeignKeyGraph::ForeignKeyGraph(const Attribute *attribute) {
	const Entity *entity = attribute->getEntity();
	this->rootNode = new ForeignKeyNode(entity, QString());
	this->entityDBNameToNode = std::map<QString, ForeignKeyNode*>();

//  We don't count rootNode as node here.
//  Reason: We must enable self-references, e.g. from Verein via attribute 
//  Dachverband to Verein.  In other words, in such a situation we follow
//  a foreign key from Verein to Verein, identifying the latter instance using
//  its usual Import/Export-Key.  We only have a problem, if the Import/Export-Key
//  would include a reference to Verein, because then we would have to expect
//  inifinite loops due to code that expects a acyclic graph, not a cycle.
//  Thus: This line is commented out on purpose.

//	this->entityDBNameToNode[entity->getDBName()] = this->rootNode;
}

ForeignKeyGraph::~ForeignKeyGraph() {
	for (std::map<QString, ForeignKeyNode*>::const_iterator it = entityDBNameToNode.begin(); 
		it != entityDBNameToNode.end(); it++) {
		delete it->second;
	}
}

ForeignKeyNode *ForeignKeyGraph::getRootNode() {
	return rootNode;
}

bool ForeignKeyGraph::hasNodeForEntity(QString entityDBName) {
	return (entityDBNameToNode.find(entityDBName) != entityDBNameToNode.end());
}
/*
ForeignKeyNode *ForeignKeyGraph::getNodeForEntity(QString entityDBName) {
	return entityDBNameToNode[entityDBName];
}
*/

void ForeignKeyGraph::addNodeToGraph(ForeignKeyNode *node) {
	QString entityDBName = node->getEntity()->getDBName();
	entityDBNameToNode[entityDBName] = node;
}

std::vector<QStringList> ForeignKeyGraph::getAttributePaths() {
	std::vector<QStringList> paths = std::vector<QStringList>();
	rootNode->addAttributePaths(paths, QStringList());
	return paths;
}

void ForeignKeyGraph::printStructure(DatabaseContext *context) {
	context->log("ImportExport", "ForeignKeyGraph.cpp", 4, "  Graph containing");

	rootNode->printStructure(context);
	for (std::map<QString, ForeignKeyNode*>::const_iterator it = entityDBNameToNode.begin();
		it != entityDBNameToNode.end(); it++) {
		ForeignKeyNode *node = it->second;
		node->printStructure(context);
	}
}
