#include "RegistryHelper.h"

#include "DataHelper.h"

#include <QSettings>
#include <QStringList>

void RegistryHelper::registerOldConnection(QString userName, QString url) {
	std::vector<DatenbankVerbindung*> newConnections = std::vector<DatenbankVerbindung*>();

	bool found = false;
	std::vector<DatenbankVerbindung*> existingOldConnections = getOldConnections();
	for (std::vector<DatenbankVerbindung*>::const_iterator it = existingOldConnections.begin(); it != existingOldConnections.end(); it++) {
		DatenbankVerbindung *candidateConnection = *it;
		if (candidateConnection->hasBenutzername() && candidateConnection->hasConnectionURL()) {
			QString candidateUserName = candidateConnection->getBenutzername();
			QString candidateURL = candidateConnection->getConnectionURL();
			if (candidateUserName == userName && candidateURL == url) {
				found = true;
			} else {
				newConnections.push_back(candidateConnection->copy());
			}
		}
	}

	DatenbankVerbindung *connection = new DatenbankVerbindung();
	connection->setBenutzername(userName);
	connection->setConnectionURL(url);

	newConnections.push_back(connection);

	int startIndex;
	if (newConnections.size() <= 3) {
		startIndex = 0;
	} else {
		startIndex = newConnections.size() - 3;
	}

	QString settingString = "";
	for (int s = startIndex; s < newConnections.size(); s++) {
		DatenbankVerbindung *currConnection = newConnections[s];
		if (currConnection->hasBenutzername() && currConnection->hasConnectionURL()) {
			settingString += (currConnection->getBenutzername() + ";" + currConnection->getConnectionURL());
			if (s < newConnections.size() - 1) {
				settingString += ";";
			}
		}
	}

	#ifdef JUROR_WINDOWS_VERSION
	QSettings settings(getJurorRegistryKey(), QSettings::NativeFormat);
	settings.setValue("OldConnections", settingString);
	#endif

	DataHelper::deleteVectorContents(newConnections);
}

std::vector<DatenbankVerbindung*> RegistryHelper::getOldConnections() {
	std::vector<DatenbankVerbindung*> connections = std::vector<DatenbankVerbindung*>();

	#ifdef JUROR_WINDOWS_VERSION
	QSettings settings(getJurorRegistryKey(), QSettings::NativeFormat);
	QVariant connectionsVariant = settings.value("OldConnections", QVariant());
	if (connectionsVariant.isValid()) {
		QString connectionsString = connectionsVariant.toString();
	    QStringList connectionsList = connectionsString.split(";");
		for (int n = 0; n < connectionsList.size() - 1; n += 2) {
			QString userName = connectionsList.at(n);
			QString url = connectionsList.at(n + 1);
			DatenbankVerbindung *connection = new DatenbankVerbindung();
			connection->setBenutzername(userName);
			connection->setConnectionURL(url);
			connections.push_back(connection);
		}
	}


	#endif

	return connections;
}

QString RegistryHelper::getJurorRegistryKey() {
	return "HKEY_LOCAL_MACHINE\\Software\\Juror";
}
