void ArchiveHelper::writeArchive(QString outNameString, QDir archiveDir) {
	std::cout << "archive_failed constant is " << ARCHIVE_FAILED << std::endl;

	const char *outname = outNameString.toAscii().data();
    struct archive *a;
    struct archive_entry *entry;
	struct stat statStruct;
    char buff[8192];
	int retCode;

    a = archive_write_new();
    archive_write_set_compression_none(a);
    //int formatReturnValue = archive_write_set_format_zip(a); // Note 1
    int formatReturnValue = archive_write_set_format_zip(a); // Note 1
	std::cout << "formatReturnValue = " << formatReturnValue << "; archive_errno = " << archive_errno(a) << "; errMsg = " << archive_error_string(a) << std::endl;
    archive_write_open_filename(a, outname);

	QStringList inputFiles = archiveDir.entryList(QStringList(), QDir::Files);
	for (int n = 0; n < inputFiles.size(); n++) {
		QString inputFile = inputFiles.at(n);

		std::cout << "Inspecting file " << qPrintable(inputFile) << std::endl;

		//const char* inputFileAsChar = inputFile.toAscii().data();
		const char* inputFileAsChar = "test57.txt";
		QString absoluteInputFilePath = archiveDir.absoluteFilePath(inputFile);
		const char* absoluteInputFilePathAsChar = absoluteInputFilePath.toAscii().data();
		std::cout << "absolute path: " << absoluteInputFilePathAsChar << std::endl;
		QFile *file = new QFile(absoluteInputFilePath);

		std::cout << "inputFileAsChar = " << inputFileAsChar << "; size = " << statStruct.st_size << std::endl;

		entry = archive_entry_new();
		archive_entry_set_pathname(entry, inputFileAsChar);
		processReturnCode("archive_entry_set_pathname", a);

		archive_entry_set_filetype(entry, AE_IFREG);
		processReturnCode("archive_entry_set_filetype", a);

		stat(absoluteInputFilePathAsChar, &statStruct);
		archive_entry_set_size(entry, statStruct.st_size); // Note 3
		processReturnCode("archive_entry_set_size", a);

		archive_entry_set_perm(entry, 0644);
		processReturnCode("archive_entry_set_perm", a);

		int writeHeaderReturnValue = archive_write_header(a, entry);
		std::cout << "==> writeHeaderReturnValue is " << writeHeaderReturnValue << std::endl;
		std::cout << "==> errCode: " << archive_errno(a) << ", errMsg: " << archive_error_string(a) << std::endl;

		if (file->open(QIODevice::ReadOnly)) {
			QDataStream stream(file);
			while (!stream.atEnd()) {
				int length = stream.readRawData((char*)&buff, sizeof(buff));
				std::cout << "Writing " << length << " bytes of data." << std::endl;
				retCode = archive_write_data(a, buff, length);
				processReturnCode("archive_write_data", retCode, a);
			}
		} else {
			// TODO error handling
		}

		delete file;

		archive_entry_free(entry);
		break;
	}

    archive_write_close(a); // Note 4
    archive_write_finish(a); // Note 5
}

void ArchiveHelper::processReturnCode(QString title, int code, struct archive *a) {
	std::cout << "Return value for " << qPrintable(title) << " is " << code << std::endl;
	std::cout << "==> errCode: " << archive_errno(a) << ", errMsg: " << archive_error_string(a) << std::endl;
}

void ArchiveHelper::processReturnCode(QString title, struct archive *a) {
	std::cout << "Executed " << qPrintable(title) << std::endl;
	std::cout << "==> errCode: " << archive_errno(a) << ", errMsg: " << archive_error_string(a) << std::endl;
}
