#include "DataExportColumn.h"

DataExportColumn::DataExportColumn(QString caption, ExportPath *path) : ExportColumn(caption) {
	this->path = path;
	this->rowToData = std::map<int, QString>();
}

void DataExportColumn::setValue(int row, QString data) {
	rowToData[row] = data;
}

bool DataExportColumn::hasValue(int row) {
	return rowToData.find(row) != rowToData.end();
}

int DataExportColumn::getFirstIndexOfValue(QString data) {
	for (std::map<int, QString>::const_iterator it = rowToData.begin(); it != rowToData.end(); it++) {
		int index = it->first;
		QString currData = it->second;
		if (data == currData) {
			return index;
		}
	}
	return -1;
}

std::vector<QString> DataExportColumn::getData() {
	std::vector<QString> retValue = std::vector<QString>();
	for (std::map<int, QString>::const_iterator it = rowToData.begin(); it != rowToData.end(); it++) {
		retValue.push_back(it->second);
	}
	return retValue;
}

int DataExportColumn::setCaptionCoordinates(DatabaseContext *context, int row, int column) {
	setCaptionRow(row);
	setCaptionStartColumn(column);
	return getCaptionWidth();
}

void DataExportColumn::calculateCaptions(DatabaseContext *context, CSVFileData *csvFileData) {
	for (unsigned int n = captionStartColumn; n < captionStartColumn + captions.size(); n++) {
		csvFileData->setValue(captionRow, n, captions[n - captionStartColumn]);
	}
}

void DataExportColumn::calculateContents(DatabaseContext *context, CSVFileData *csvFileData, int startRow) {
	for (std::map<int, QString>::const_iterator it = rowToData.begin(); it != rowToData.end(); it++) {
		int row = it->first + startRow;
		int column = getCaptionStartColumn();
		QString value = it->second;
		csvFileData->setValue(row, column, value);
		context->log("ImportExport", "DataExportColumn", 1, "calculateContents: Setting token " + value + " for row/column " + QString::number(row) + "/" + QString::number(column));
	}
}

void DataExportColumn::processConverters(DatabaseContext *context) {
	ExportConverter *converter = path->getConverter();
	if (converter != NULL) {
		converter->convert(context, this);
	}
}

int DataExportColumn::getMaxCaptionRow() {
	return getCaptionRow();
}

int DataExportColumn::getMaxDataRow() {
	int maxDataRow = -1;
	for (std::map<int, QString>::const_iterator it = rowToData.begin(); it != rowToData.end(); it++) {
		int row = it->first;
		maxDataRow = (row > maxDataRow ? row : maxDataRow);
	}
	return maxDataRow;
}
