#include "ExportBlock.h"

#include <QObject>

#include "BlockExportColumn.h"

#include "Exception.h"

ExportBlock::ExportBlock() {
	columns = std::vector<ExportColumn*>();
	columnKeyToColumn = std::map<QString, ExportColumn*>();
	parent = NULL;
}

void ExportBlock::addColumn(QString key, ExportColumn *column) {
	columns.push_back(column);
	columnKeyToColumn[key] = column;
	column->setParent(this);
	column->setKey(key);
}

ExportColumn *ExportBlock::getColumn(QString key) {
	return columnKeyToColumn[key];
}

bool ExportBlock::hasColumn(QString key) {
	return columnKeyToColumn.find(key) != columnKeyToColumn.end();
}

std::vector<ExportColumn*> ExportBlock::getAllColumns() {
	return columns;
}

void ExportBlock::exchangeColumn(QString key, ExportColumn *newColumn) {
	if (columnKeyToColumn.find(key) != columnKeyToColumn.end()) {
		int n = 0;
		for (std::vector<ExportColumn*>::iterator it = columns.begin(); it != columns.end(); it++) {
			ExportColumn *currColumn = *it;
			if (currColumn->getKey() == key) {
				columns.erase(it);
				columns.insert(columns.begin() + n, newColumn);
				break;
			} else {
				n++;
			}
		}
		delete columnKeyToColumn[key];
		columnKeyToColumn[key] = newColumn;
	} else {
		throw Exception("ImportExport", "ExportBlock", QObject::trUtf8("Interner Fehler"), QObject::trUtf8("Spalte %1 beim internen Spaltentausch nicht gefunden.").arg(key));
	}
}


int ExportBlock::setCaptionCoordinates(DatabaseContext *context, int row, int column) {
	int currRow = row;
	int currColumn = column;
	int numberOfColumns = 0;

	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		int currNumberOfColumns = column->setCaptionCoordinates(context, currRow, currColumn);
		numberOfColumns += currNumberOfColumns;
		currColumn += currNumberOfColumns;
	}
	return numberOfColumns;
}

void ExportBlock::calculateCaptions(DatabaseContext *context, CSVFileData *csvFileData) {
	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		column->calculateCaptions(context, csvFileData);
	}
}

void ExportBlock::calculateContents(DatabaseContext *context, CSVFileData *csvFileData, int startRow) {
	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		column->calculateContents(context, csvFileData, startRow);
	}
}

void ExportBlock::processConverters(DatabaseContext *context) {
	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		column->processConverters(context);
	}
}

int ExportBlock::getMaxCaptionRow() {
	int maxCaptionRow = -1;
	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		int currMaxCaptionRow = column->getMaxCaptionRow();
		maxCaptionRow = (maxCaptionRow > currMaxCaptionRow ? maxCaptionRow : currMaxCaptionRow);
	}
	return maxCaptionRow;
}

int ExportBlock::getMaxDataRow() {
	int maxDataRow = -1;
	std::vector<ExportColumn*> columns = getAllColumns();
	for(std::vector<ExportColumn*>::const_iterator it = columns.begin(); it != columns.end(); it++) {
		ExportColumn *column = *it;
		int currMaxDataRow = column->getMaxDataRow();
		maxDataRow = (maxDataRow > currMaxDataRow ? maxDataRow : currMaxDataRow);
	}
	return maxDataRow;
}

BlockExportColumn *ExportBlock::getParent() {
	return parent;
}

void ExportBlock::setParent(BlockExportColumn *parent) {
	this->parent = parent;
}

ExportBlock *ExportBlock::getSuperParent() {
	if (this->parent == NULL) {
		return NULL;
	} else {
		return this->parent->getSuperParent();
	}
}
