#include "ExportHelper.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataHelper.h"
#include "DataSet.h"

#include "Benutzer.h"
#include "BenutzerController.h"
#include "Bild.h"
#include "BildController.h"
#include "BildDaten.h"
#include "BildDatenController.h"
#include "CodeType.h"
#include "CodeTypeController.h"
#include "CodeValue.h"
#include "CodeValueController.h"
#include "DataModelInfo.h"
#include "DataModelInfoController.h"
#include "Durchgang.h"
#include "DurchgangController.h"
#include "EinzelWertung.h"
#include "EinzelWertungController.h"
#include "EventLog.h"
#include "EventLogController.h"
#include "EventLogCategory.h"
#include "EventLogCategoryController.h"
#include "EventLogSettings.h"
#include "EventLogSettingsController.h"
#include "ExternerJuror.h"
#include "ExternerJurorController.h"
#include "FotographischerTitel.h"
#include "FotographischerTitelController.h"
#include "Funktionstraeger.h"
#include "FunktionstraegerController.h"
#include "GlobaleEinstellung.h"
#include "GlobaleEinstellungController.h"
#include "JuriertBei.h"
#include "JuriertBeiController.h"
#include "Jurierung.h"
#include "JurierungController.h"
#include "Mitglied.h"
#include "MitgliedController.h"
#include "QueryLog.h"
#include "QueryLogController.h"
#include "QueryLogSettings.h"
#include "QueryLogSettingsController.h"
#include "Session.h"
#include "SessionController.h"
#include "Sparte.h"
#include "SparteController.h"
#include "TeilnehmerKombinationsWertung.h"
#include "TeilnehmerKombinationsWertungController.h"
#include "TeilnehmerWertung.h"
#include "TeilnehmerWertungController.h"
#include "Verein.h"
#include "VereinController.h"
#include "VereinsDatenBerechtigung.h"
#include "VereinsDatenBerechtigungController.h"
#include "VereinsKombinationsWertung.h"
#include "VereinsKombinationsWertungController.h"
#include "VereinsWertung.h"
#include "VereinsWertungController.h"
#include "Wertung.h"
#include "WertungController.h"
#include "Wettbewerb.h"
#include "WettbewerbController.h"
#include "WettbewerbsTeilnahme.h"
#include "WettbewerbsTeilnahmeController.h"

void ExportHelper::fuegeVereinsDatenZuDataSetHinzu(DatabaseContext *databaseContext, DataExporter *exporter, DataSet *dataSet, Verein *verein) {
	QString idString = (verein->hasId() ? QString::number(verein->getId()) : "(keine)");
	QString nameString = (verein->hasName() ? verein->getName() : "(ohne Name)");

	// fetch Mitglieder
	FotographischerTitelController fotoController = FotographischerTitelController();
	FunktionstraegerController funktController = FunktionstraegerController();
	MitgliedController mitgliedController = MitgliedController();
	std::vector<Mitglied*> mitglieder = mitgliedController.getByVerein(databaseContext, verein);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(mitglieder.size()) + " Mitglieder von Verein " + nameString + " mit der ID " + idString);

	// copy to a vector of Persistents
	std::vector<Persistent*> persistents = std::vector<Persistent*>();
	for (std::vector<Mitglied*>::const_iterator it = mitglieder.begin(); it != mitglieder.end(); it++) {
		Mitglied *mitglied = *it;
		persistents.push_back(mitglied);

		std::vector<FotographischerTitel*> fotoTitel = fotoController.getByMitglied(databaseContext, mitglied);
		for (std::vector<FotographischerTitel*>::const_iterator it2 = fotoTitel.begin(); it2 != fotoTitel.end(); it2++) {
			persistents.push_back(*it2);
		}
	}

	persistents.push_back(verein->copy());
	std::vector<Funktionstraeger*> funktionstraeger = funktController.getByVerein(databaseContext, verein);
	for (std::vector<Funktionstraeger*>::const_iterator it = funktionstraeger.begin(); it != funktionstraeger.end(); it++) {
		persistents.push_back(*it);
	}

	exporter->addToDataSetForExport(databaseContext, dataSet, persistents);
}

void ExportHelper::fuegeVereinsMitgliederZuDataSetHinzu(DatabaseContext *databaseContext, DataExporter *exporter, DataSet *dataSet, Verein *verein) {
	QString idString = (verein->hasId() ? QString::number(verein->getId()) : "(keine)");
	QString nameString = (verein->hasName() ? verein->getName() : "(ohne Name)");

	// fetch Mitglieder
	MitgliedController controller = MitgliedController();
	std::vector<Mitglied*> mitglieder = controller.getByVerein(databaseContext, verein);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(mitglieder.size()) + " Mitglieder von Verein " + nameString + " mit der ID " + idString);

	// copy to a vector of Persistents
	std::vector<Persistent*> persistents = std::vector<Persistent*>();
	for (std::vector<Mitglied*>::const_iterator it = mitglieder.begin(); it != mitglieder.end(); it++) {
		persistents.push_back(*it);
	}

	exporter->addToDataSetForExport(databaseContext, dataSet, persistents);
}

void ExportHelper::exportiereVerein(DatabaseContext *databaseContext, Verein *verein, QString fileName) {
	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	fuegeVereinsDatenZuDataSetHinzu(databaseContext, exporter, dataSet, verein);

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportiereVereinsHierarchie(DatabaseContext *databaseContext, Verein *verein, QString fileName) {
	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();

	VereinController vereinController = VereinController();

	std::vector<Verein*> uebrigeVereine = std::vector<Verein*>();
	uebrigeVereine.push_back(verein);
	while (uebrigeVereine.size() > 0) {
		std::vector<Verein*> neueVereine = std::vector<Verein*>();
		for (std::vector<Verein*>::const_iterator it = uebrigeVereine.begin(); it != uebrigeVereine.end(); it++) {
			Verein *currVerein = *it;
			fuegeVereinsDatenZuDataSetHinzu(databaseContext, exporter, dataSet, currVerein);

			std::vector<Verein*> unterVereine = vereinController.getByDachverband(databaseContext, currVerein);
			for (std::vector<Verein*>::const_iterator it2 = unterVereine.begin(); it2 != unterVereine.end(); it2++) {
				neueVereine.push_back(*it2);
			}
		}
		uebrigeVereine = std::vector<Verein*>();
		for (std::vector<Verein*>::const_iterator it = neueVereine.begin(); it != neueVereine.end(); it++) {
			uebrigeVereine.push_back(*it);
		}
	}

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportiereVereinsMitglieder(DatabaseContext *databaseContext, Verein *verein, QString fileName) {
	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	fuegeVereinsMitgliederZuDataSetHinzu(databaseContext, exporter, dataSet, verein);

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportiereVereinsHierarchieMitglieder(DatabaseContext *databaseContext, Verein *verein, QString fileName) {
	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();

	VereinController vereinController = VereinController();

	std::vector<Verein*> uebrigeVereine = std::vector<Verein*>();
	uebrigeVereine.push_back(verein);
	while (uebrigeVereine.size() > 0) {
		std::vector<Verein*> neueVereine = std::vector<Verein*>();
		for (std::vector<Verein*>::const_iterator it = uebrigeVereine.begin(); it != uebrigeVereine.end(); it++) {
			Verein *currVerein = *it;
			fuegeVereinsMitgliederZuDataSetHinzu(databaseContext, exporter, dataSet, currVerein);

			std::vector<Verein*> unterVereine = vereinController.getByDachverband(databaseContext, currVerein);
			for (std::vector<Verein*>::const_iterator it2 = unterVereine.begin(); it2 != unterVereine.end(); it2++) {
				neueVereine.push_back(*it2);
			}
		}
		uebrigeVereine = std::vector<Verein*>();
		for (std::vector<Verein*>::const_iterator it = neueVereine.begin(); it != neueVereine.end(); it++) {
			uebrigeVereine.push_back(*it);
		}
	}

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportWettbewerb(DatabaseContext *databaseContext, Wettbewerb *wettbewerb, QString fileName) {
	std::vector<Persistent*> persistents = getPersistentsZuWettbewerb(databaseContext, wettbewerb);

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(databaseContext, dataSet, persistents);

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportWettbewerbMitMitgliedsDaten(DatabaseContext *databaseContext,
													 Wettbewerb *wettbewerb, QString fileName) {

	QString idString = (wettbewerb->hasId() ? QString::number(wettbewerb->getId()) : "(keine)");
	QString nameString = (wettbewerb->hasName() ? wettbewerb->getName() : "(keiner)");

	std::vector<Persistent*> persistents = getPersistentsZuWettbewerb(databaseContext, wettbewerb);

	WettbewerbsTeilnahmeController teilnahmeController = WettbewerbsTeilnahmeController();
	MitgliedController mitgliedsController = MitgliedController();
	VereinController vereinController = VereinController();

	// Die ids aller Vereine
	std::set<qlonglong> mitgliedsIds = std::set<qlonglong>();
	std::set<qlonglong> vereinsIds = std::set<qlonglong>();
	std::set<qlonglong> externerJurorIds = std::set<qlonglong>();
	std::vector<WettbewerbsTeilnahme*> teilnahmeVector = teilnahmeController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(teilnahmeVector.size()) + " Teilnehmer des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<WettbewerbsTeilnahme*>::const_iterator it = teilnahmeVector.begin();
													  it != teilnahmeVector.end(); it++) {
		// Die Teilnahme-Instanzen nicht hinzufügen, wir haben es schon vorher gemacht.
		WettbewerbsTeilnahme *teilnahme = *it;
		qlonglong mitgliedsId = teilnahme->getMitglied();
		mitgliedsIds.insert(mitgliedsId);
	}

	JuriertBeiController juriertBeiController = JuriertBeiController();
	std::vector<JuriertBei*> juriertBeiVector = juriertBeiController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(juriertBeiVector.size()) + " Juroren des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<JuriertBei*>::const_iterator it = juriertBeiVector.begin(); it != juriertBeiVector.end(); it++) {
		// Die JuriertBei-Instanzen nicht hinzufügen, das machen wir in
		// getPersistentsZuWettbewerb.  Wohl aber müssen wir uns hier um
		// die dranhängenden Mitglieder und externe Juroren kümmern.
		// Die werden nur zum Jurierungsrechner hin, aber nicht davon weg
		// transferiert.
		JuriertBei *juriertBei = *it;
		if (juriertBei->hasMitglied()) {
			mitgliedsIds.insert(juriertBei->getMitglied());
		}
		if (juriertBei->hasExternerJuror()) {
			externerJurorIds.insert(juriertBei->getExternerJuror());
		}
	}

	for (std::set<qlonglong>::const_iterator it = mitgliedsIds.begin(); it != mitgliedsIds.end(); it++) {
		qlonglong mitgliedsId = *it;
		Mitglied *mitglied = mitgliedsController.getByPrimaryKey(databaseContext, mitgliedsId);
		if (mitglied->hasVerein()) {
			vereinsIds.insert(mitglied->getVerein());
		}
		databaseContext->log("ImportExport", "ExportHelper", 4, "Filtere persönliche Mitgliedsdaten vor Export aus");
		Mitglied *reduziertesMitglied = filtereMitgliedsAttribute(databaseContext, mitglied);
		delete mitglied;

		persistents.push_back(reduziertesMitglied);
	}

	ExternerJurorController externerJurorController = ExternerJurorController();
	for (std::set<qlonglong>::const_iterator it = externerJurorIds.begin(); it != externerJurorIds.end(); it++) {
		qlonglong externerJurorId = *it;
		ExternerJuror *externerJuror = externerJurorController.getByPrimaryKey(databaseContext, externerJurorId);
		persistents.push_back(externerJuror);
	}

	for (std::set<qlonglong>::const_iterator it = vereinsIds.begin(); it != vereinsIds.end(); it++) {
		qlonglong vereinsId = *it;
		Verein *verein = vereinController.getByPrimaryKey(databaseContext, vereinsId);
		persistents.push_back(verein);
	}

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(databaseContext, dataSet, persistents);

	// export
	exporter->exportData(databaseContext, fileName, dataSet);

	delete exporter;
	delete dataSet;
}


std::vector<Persistent*> ExportHelper::getPersistentsZuWettbewerb(DatabaseContext *databaseContext, Wettbewerb *wettbewerb) {
	QString idString = (wettbewerb->hasId() ? QString::number(wettbewerb->getId()) : "(keine)");
	QString nameString = (wettbewerb->hasName() ? wettbewerb->getName() : "(keiner)");

	// Construct a vector of Persistents.  All instances to be exported will
	// be added to that vector.
	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	// First add the Wettbewerb itself
	persistents.push_back(wettbewerb);

	// Then add all JuriertBei instances
	JuriertBeiController juriertBeiController = JuriertBeiController();
	std::vector<JuriertBei*> juriertBeiVector = juriertBeiController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(juriertBeiVector.size()) + " JuriertBei-Instanzen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<JuriertBei*>::const_iterator it = juriertBeiVector.begin(); it != juriertBeiVector.end(); it++) {
		persistents.push_back(*it);
	}

	// Then add all WettbewerbsTeilnahme instances
	WettbewerbsTeilnahmeController teilnahmeController = WettbewerbsTeilnahmeController();
	std::vector<WettbewerbsTeilnahme*> teilnahmeVector = teilnahmeController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(teilnahmeVector.size()) 
												  + " WettbewerbsTeilnahme-Instanzen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<WettbewerbsTeilnahme*>::const_iterator it = teilnahmeVector.begin(); it != teilnahmeVector.end(); it++) {
		persistents.push_back(*it);
	}

	// Then add all Sparte instances
	SparteController sparteController = SparteController();
	std::vector<Sparte*> sparteVector = sparteController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(sparteVector.size()) + " Sparten des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<Sparte*>::const_iterator it = sparteVector.begin(); it != sparteVector.end(); it++) {
		persistents.push_back(*it);
	}

	std::set<qlonglong> ungueltigeBilderIds = std::set<qlonglong>();

	int bildAnzahl = 0;
	// Then add all Bild instances.  We assume that they are all linked to an
	// appropriate WettbewerbsTeilnahme instance.  Furthermore, for each
	// Bild add the corresponding BildDaten instance.
	BildController bildController = BildController();
	BildDatenController bildDatenController = BildDatenController();
	for (std::vector<WettbewerbsTeilnahme*>::const_iterator it = teilnahmeVector.begin(); it != teilnahmeVector.end(); it++) {
		WettbewerbsTeilnahme *teilnahme = *it;
		std::vector<Bild*> bildVector = bildController.getByWettbewerbsTeilnahme(databaseContext, teilnahme);
		bildAnzahl += bildVector.size();
		for (std::vector<Bild*>::const_iterator it2 = bildVector.begin(); it2 != bildVector.end(); it2++) {
			Bild *bild = *it2;
			// BZ 303 - Ungueltige Bilder ignorieren
			if (bild->hasGueltig() && bild->getGueltig()) {
				persistents.push_back(bild);
				qlonglong bildDatenId = bild->getBildDaten();
				BildDaten *bildDaten = bildDatenController.getByPrimaryKey(databaseContext, bildDatenId);
				// Den BLOB brauchen wir in diesem Objekt nicht mehr, also koennen wir uns den Speicher sparen
				// (bei 1000 Bildern waeren ansonsten schnell 1GB Bilder im Hauptspeicher)
				// deleteDaten alleine loescht nicht den ByteArray.
				bildDaten->setDaten(QByteArray());
				bildDaten->deleteDaten();
				persistents.push_back(bildDaten);
			} else {
				ungueltigeBilderIds.insert(bild->getId());
			}
		}
	}

	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(bildAnzahl) + " Bilder des Wettbewerbs " + nameString + " mit der ID " + idString);

	int durchgangAnzahl = 0;
	int wertungsAnzahl = 0;
	int einzelWertungsAnzahl = 0;

	// And finally add the Jurierung, Durchgang, Wertung and EinzelWertung instances
	JurierungController jurierungController = JurierungController();
	DurchgangController durchgangController = DurchgangController();
	WertungController wertungController = WertungController();
	EinzelWertungController einzelWertungController = EinzelWertungController();
	std::vector<Jurierung*> jurierungVector = jurierungController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(jurierungVector.size()) + "Jurierungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<Jurierung*>::const_iterator it = jurierungVector.begin(); it != jurierungVector.end(); it++) {
		Jurierung *jurierung = *it;
		persistents.push_back(jurierung);

		std::vector<Durchgang*> durchgangVector = durchgangController.getByJurierung(databaseContext, jurierung);
		durchgangAnzahl += durchgangVector.size();
		for (std::vector<Durchgang*>::const_iterator it2 = durchgangVector.begin(); it2 != durchgangVector.end(); it2++) {
			Durchgang *durchgang = *it2;
			persistents.push_back(durchgang);

			std::vector<Wertung*> wertungVector = wertungController.getByDurchgang(databaseContext, durchgang);
			wertungsAnzahl += wertungVector.size();
			for (std::vector<Wertung*>::const_iterator it3 = wertungVector.begin(); it3 != wertungVector.end(); it3++) {
				Wertung *wertung = *it3;

				// BZ 303 - Wertungen ungueltiger Bilder ignorieren
				if (wertung->hasBild() && ungueltigeBilderIds.find(wertung->getBild()) == ungueltigeBilderIds.end()) {
					persistents.push_back(wertung);
					std::vector<EinzelWertung*> einzelwertungVector = einzelWertungController.getByWertung(databaseContext, wertung);
					einzelWertungsAnzahl += einzelwertungVector.size();
					for (std::vector<EinzelWertung*>::const_iterator it4 = einzelwertungVector.begin(); 
							it4 != einzelwertungVector.end(); it4++) {
						persistents.push_back(*it4);
					}
				}
			}
		}
	}

	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(durchgangAnzahl) + " Durchgänge des Wettbewerbs " + nameString + " mit der ID " + idString);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(wertungsAnzahl) + " Wertungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(einzelWertungsAnzahl) + " EinzelWertungen des Wettbewerbs " + nameString + " mit der ID " + idString);

	TeilnehmerWertungController teilnehmerWertungController = TeilnehmerWertungController();
	std::vector<TeilnehmerWertung*> teilnehmerWertungen = teilnehmerWertungController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(teilnehmerWertungen.size()) + " Teilnehmer-Wertungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<TeilnehmerWertung*>::const_iterator it = teilnehmerWertungen.begin(); it != teilnehmerWertungen.end(); it++) {
		TeilnehmerWertung *teilnehmerWertung = *it;
		persistents.push_back(teilnehmerWertung);
	}

	TeilnehmerKombinationsWertungController teilnehmerKombinationsWertungController = TeilnehmerKombinationsWertungController();
	std::vector<TeilnehmerKombinationsWertung*> teilnehmerKombinationsWertungen = teilnehmerKombinationsWertungController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(teilnehmerKombinationsWertungen.size())
							+ " Teilnehmer-Kombinations-Wertungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<TeilnehmerKombinationsWertung*>::const_iterator it = teilnehmerKombinationsWertungen.begin(); it != teilnehmerKombinationsWertungen.end(); it++) {
		TeilnehmerKombinationsWertung *teilnehmerKombinationsWertung = *it;
		persistents.push_back(teilnehmerKombinationsWertung);
	}

	VereinsWertungController vereinsWertungController = VereinsWertungController();
	std::vector<VereinsWertung*> vereinsWertungen = vereinsWertungController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(vereinsWertungen.size()) + " Vereins-Wertungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<VereinsWertung*>::const_iterator it = vereinsWertungen.begin(); it != vereinsWertungen.end(); it++) {
		VereinsWertung *vereinsWertung = *it;
		persistents.push_back(vereinsWertung);
	}

	VereinsKombinationsWertungController vereinsKombinationsWertungController = VereinsKombinationsWertungController();
	std::vector<VereinsKombinationsWertung*> vereinsKombinationsWertungen = vereinsKombinationsWertungController.getByWettbewerb(databaseContext, wettbewerb);
	databaseContext->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(vereinsKombinationsWertungen.size())
						  + " Vereins-Kombinationswertungen des Wettbewerbs " + nameString + " mit der ID " + idString);
	for (std::vector<VereinsKombinationsWertung*>::const_iterator it = vereinsKombinationsWertungen.begin(); it != vereinsKombinationsWertungen.end(); it++) {
		VereinsKombinationsWertung *vereinsKombinationsWertung = *it;
		persistents.push_back(vereinsKombinationsWertung);
	}


	// TODO: Have a look on the log tables


	return persistents;
}



Mitglied* ExportHelper::filtereMitgliedsAttribute(DatabaseContext *context, Mitglied *mitglied) {
	Mitglied *reduziertesMitglied = new Mitglied();

	if (mitglied->hasId()) {
		reduziertesMitglied->setId(mitglied->getId());
	} else {
		reduziertesMitglied->deleteId();
	}
	if (mitglied->hasVerein()) {
		reduziertesMitglied->setVerein(mitglied->getVerein());
	} else {
		reduziertesMitglied->deleteVerein();
	}
	if (mitglied->hasMitgliedsArt()) {
		reduziertesMitglied->setMitgliedsArt(mitglied->getMitgliedsArt());
	} else {
		reduziertesMitglied->deleteMitgliedsArt();
	}
	if (mitglied->hasMitgliedsNummerImVerein()) {
		reduziertesMitglied->setMitgliedsNummerImVerein(mitglied->getMitgliedsNummerImVerein());
	} else {
		reduziertesMitglied->deleteMitgliedsNummerImVerein();
	}
	if (mitglied->hasMitgliedsNummerKomplett()) {
		reduziertesMitglied->setMitgliedsNummerKomplett(mitglied->getMitgliedsNummerKomplett());
	} else {
		reduziertesMitglied->deleteMitgliedsNummerKomplett();
	}
	if (mitglied->hasJugend()) {
		reduziertesMitglied->setJugend(mitglied->getJugend());
	} else {
		reduziertesMitglied->deleteJugend();
	}
	if (mitglied->hasDatumEintritt()) {
		reduziertesMitglied->setDatumEintritt(mitglied->getDatumEintritt());
	} else {
		reduziertesMitglied->deleteDatumEintritt();
	}
	if (mitglied->hasDatumAustritt()) {
		reduziertesMitglied->setDatumAustritt(mitglied->getDatumAustritt());
	} else {
		reduziertesMitglied->deleteDatumAustritt();
	}
	if (mitglied->hasAnrede()) {
		reduziertesMitglied->setAnrede(mitglied->getAnrede());
	} else {
		reduziertesMitglied->deleteAnrede();
	}
	if (mitglied->hasTitel()) {
		reduziertesMitglied->setTitel(mitglied->getTitel());
	} else {
		reduziertesMitglied->deleteTitel();
	}
	if (mitglied->hasVorname()) {
		reduziertesMitglied->setVorname(mitglied->getVorname());
	} else {
		reduziertesMitglied->deleteVorname();
	}
	if (mitglied->hasNachname()) {
		reduziertesMitglied->setNachname(mitglied->getNachname());
	} else {
		reduziertesMitglied->deleteNachname();
	}

	return reduziertesMitglied;
}

void ExportHelper::exportiereAlleFachlichenDaten(DatabaseContext *context, QString fileName) {
	DataSet *dataSet = new DataSet();
	addAlleFachlichenDaten(context, dataSet);

	context->log("ImportExport", "ExportHelper", 7, "Exportiere fachliche Datensätze");

	exportiere(context, dataSet, fileName);
}

void ExportHelper::exportiereAlleTechnischenDaten(DatabaseContext *context, QString fileName) {
	DataSet *dataSet = new DataSet();
	addAlleTechnischenDaten(context, dataSet);

	context->log("ImportExport", "ExportHelper", 7, "Exportiere technische Datensätze");

	exportiere(context, dataSet, fileName);
}

void ExportHelper::exportiereDatenberechtigungen(DatabaseContext *context, QString fileName) {
	std::map<QString, QVariant> emptyMap = std::map<QString, QVariant>();
	VereinsDatenBerechtigungController vereinsDatenBerechtigungController = VereinsDatenBerechtigungController();

	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	std::vector<VereinsDatenBerechtigung*> vereinsDatenBerechtigungVector 
											  = vereinsDatenBerechtigungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(vereinsDatenBerechtigungVector.size()) + " Datensätze vom Typ VereinsDatenBerechtigung");
	for (std::vector<VereinsDatenBerechtigung*>::const_iterator it = vereinsDatenBerechtigungVector.begin(); 
											  it != vereinsDatenBerechtigungVector.end(); it++) {
		persistents.push_back(*it);
	}

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	// export
	exporter->exportData(context, fileName, dataSet);

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportiereAlleDaten(DatabaseContext *context, QString fileName) {
	DataSet *dataSet = new DataSet();
	addAlleFachlichenDaten(context, dataSet);
	addAlleTechnischenDaten(context, dataSet);

	context->log("ImportExport", "ExportHelper", 7, "Exportiere alle Datensätze");

	exportiere(context, dataSet, fileName);
}

void ExportHelper::addAlleFachlichenDaten(DatabaseContext *context, DataSet *dataSet) {
	dataSet->addAllTuplesOfEntity("Bild");

	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	BildDatenController bildDatenController = BildDatenController();
	DataExporter *exporter = new CSVDataExporter();
	std::map<QString, QVariant> emptyMap = std::map<QString, QVariant>();

	// Do not use the addAll feature for BildDaten, as the addAll feature does not work for BLOBs
	std::set<QString> bildDatenOmitSet = std::set<QString>();
	bildDatenOmitSet.insert("daten");
	std::vector<BildDaten*> bildDatenVector = bildDatenController.get(context, emptyMap, false, bildDatenOmitSet, false, false);
	context->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(bildDatenVector.size()) + " Datensätze vom Typ BildDaten");
	for (std::vector<BildDaten*>::const_iterator it = bildDatenVector.begin(); it != bildDatenVector.end(); it++) {
		BildDaten *bildDaten = *it;
		persistents.push_back(bildDaten);
	}
	exporter->addToDataSetForExport(context, dataSet, persistents);
	delete exporter;

	dataSet->addAllTuplesOfEntity("Durchgang");
	dataSet->addAllTuplesOfEntity("EinzelWertung");
	dataSet->addAllTuplesOfEntity("ExternerJuror");
	dataSet->addAllTuplesOfEntity("FotographischerTitel");
	dataSet->addAllTuplesOfEntity("Funktionstraeger");
	dataSet->addAllTuplesOfEntity("JuriertBei");
	dataSet->addAllTuplesOfEntity("Jurierung");
	dataSet->addAllTuplesOfEntity("Mitglied");
	dataSet->addAllTuplesOfEntity("Sparte");
	dataSet->addAllTuplesOfEntity("TeilnehmerKombinationsWertung");
	dataSet->addAllTuplesOfEntity("TeilnehmerWertung");
	dataSet->addAllTuplesOfEntity("Verein");
	dataSet->addAllTuplesOfEntity("VereinsKombinationsWertung");
	dataSet->addAllTuplesOfEntity("VereinsWertung");
	dataSet->addAllTuplesOfEntity("Wertung");
	dataSet->addAllTuplesOfEntity("Wettbewerb");
	dataSet->addAllTuplesOfEntity("WettbewerbsTeilnahme");
}

void ExportHelper::addFachlicheDaten(DatabaseContext *context, std::vector<Persistent*> &persistents) {
	BildController bildController = BildController();
	BildDatenController bildDatenController = BildDatenController();
	DurchgangController durchgangController = DurchgangController();
	EinzelWertungController einzelWertungController = EinzelWertungController();
	ExternerJurorController externerJurorController = ExternerJurorController();
	FotographischerTitelController fotographischerTitelController = FotographischerTitelController();
	FunktionstraegerController funktionstraegerController = FunktionstraegerController();
	JuriertBeiController juriertBeiController = JuriertBeiController();
	JurierungController jurierungController = JurierungController();
	MitgliedController mitgliedController = MitgliedController();
	SparteController sparteController = SparteController();
	TeilnehmerKombinationsWertungController teilnehmerKombinationsWertungController 
											= TeilnehmerKombinationsWertungController();
	TeilnehmerWertungController teilnehmerWertungController = TeilnehmerWertungController();
	VereinController vereinController = VereinController();
	VereinsKombinationsWertungController vereinsKombinationsWertungController
											= VereinsKombinationsWertungController();
	VereinsWertungController vereinsWertungController = VereinsWertungController();
	WertungController wertungController = WertungController();
	WettbewerbController wettbewerbController = WettbewerbController();
	WettbewerbsTeilnahmeController wettbewerbsTeilnahmeController = WettbewerbsTeilnahmeController();

	std::map<QString, QVariant> emptyMap = std::map<QString, QVariant>();
	std::vector<Bild*> bildVector = bildController.get(context, emptyMap);
	for (std::vector<Bild*>::const_iterator it = bildVector.begin(); it != bildVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::set<QString> bildDatenOmitSet = std::set<QString>();
	bildDatenOmitSet.insert("daten");
	std::vector<BildDaten*> bildDatenVector = bildDatenController.get(context, emptyMap, false, bildDatenOmitSet, false, false);
	context->log("ImportExport", "ExportHelper", 6, "Exportiere " + QString::number(bildVector.size()) + " Datensätze vom Typ Bild");
	for (std::vector<BildDaten*>::const_iterator it = bildDatenVector.begin(); it != bildDatenVector.end(); it++) {
		BildDaten *bildDaten = *it;
		persistents.push_back(bildDaten);
	}
	std::vector<Durchgang*> durchgangVector = durchgangController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(durchgangVector.size()) + " Datensätze vom Typ Durchgang");
	for (std::vector<Durchgang*>::const_iterator it = durchgangVector.begin(); it != durchgangVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<EinzelWertung*> einzelWertungVector = einzelWertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(einzelWertungVector.size()) + " Datensätze vom Typ EinzelWertung");
	for (std::vector<EinzelWertung*>::const_iterator it = einzelWertungVector.begin(); 
														  it != einzelWertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<ExternerJuror*> externerJurorVector = externerJurorController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(externerJurorVector.size()) + " Datensätze vom Typ ExternerJuror");
	for (std::vector<ExternerJuror*>::const_iterator it = externerJurorVector.begin(); 
														  it != externerJurorVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<FotographischerTitel*> fotographischerTitelVector = fotographischerTitelController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(fotographischerTitelVector.size()) + " Datensätze vom Typ FotographischerTitel");
	for (std::vector<FotographischerTitel*>::const_iterator it = fotographischerTitelVector.begin(); 
														  it != fotographischerTitelVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Funktionstraeger*> funktionstraegerVector = funktionstraegerController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(funktionstraegerVector.size()) + " Datensätze vom Typ Funktionstraeger");
	for (std::vector<Funktionstraeger*>::const_iterator it = funktionstraegerVector.begin(); 
														  it != funktionstraegerVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<JuriertBei*> juriertBeiVector = juriertBeiController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(juriertBeiVector.size()) + " Datensätze vom Typ JuriertBei");
	for (std::vector<JuriertBei*>::const_iterator it = juriertBeiVector.begin(); it != juriertBeiVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Jurierung*> jurierungVector = jurierungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(jurierungVector.size()) + " Datensätze vom Typ Jurierung");
	for (std::vector<Jurierung*>::const_iterator it = jurierungVector.begin(); it != jurierungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Mitglied*> mitgliedVector = mitgliedController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(mitgliedVector.size()) + " Datensätze vom Typ Mitglied");
	for (std::vector<Mitglied*>::const_iterator it = mitgliedVector.begin(); it != mitgliedVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Sparte*> sparteVector = sparteController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(sparteVector.size()) + " Datensätze vom Typ Sparte");
	for (std::vector<Sparte*>::const_iterator it = sparteVector.begin(); it != sparteVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<TeilnehmerKombinationsWertung*> teilnehmerKombinationsWertungVector 
										= teilnehmerKombinationsWertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(teilnehmerKombinationsWertungVector.size()) + " Datensätze vom Typ TeilnehmerKombinationsWertung");
	for (std::vector<TeilnehmerKombinationsWertung*>::const_iterator it = teilnehmerKombinationsWertungVector.begin(); 
										it != teilnehmerKombinationsWertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<TeilnehmerWertung*> teilnehmerWertungVector 
										= teilnehmerWertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(teilnehmerWertungVector.size()) + " Datensätze vom Typ TeilnehmerWertung");
	for (std::vector<TeilnehmerWertung*>::const_iterator it = teilnehmerWertungVector.begin(); 
										it != teilnehmerWertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Verein*> vereinVector = vereinController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(vereinVector.size()) + " Datensätze vom Typ Verein");
	for (std::vector<Verein*>::const_iterator it = vereinVector.begin(); it != vereinVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<VereinsKombinationsWertung*> vereinsKombinationsWertungVector 
										= vereinsKombinationsWertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(vereinsKombinationsWertungVector.size()) + " Datensätze vom Typ VereinsKombinationsWertung");
	for (std::vector<VereinsKombinationsWertung*>::const_iterator it = vereinsKombinationsWertungVector.begin(); 
										it != vereinsKombinationsWertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<VereinsWertung*> vereinsWertungVector 
										= vereinsWertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(vereinsWertungVector.size()) + " Datensätze vom Typ VereinsWertung");
	for (std::vector<VereinsWertung*>::const_iterator it = vereinsWertungVector.begin(); 
										it != vereinsWertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Wertung*> wertungVector = wertungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(wertungVector.size()) + " Datensätze vom Typ Wertung");
	for (std::vector<Wertung*>::const_iterator it = wertungVector.begin(); it != wertungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Wettbewerb*> wettbewerbVector = wettbewerbController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(wettbewerbVector.size()) + " Datensätze vom Typ Wettbewerb");
	for (std::vector<Wettbewerb*>::const_iterator it = wettbewerbVector.begin(); it != wettbewerbVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<WettbewerbsTeilnahme*> wettbewerbsTeilnahmeVector = wettbewerbsTeilnahmeController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(wettbewerbsTeilnahmeVector.size()) + " Datensätze vom Typ WettbewerbsTeilnahme");
	for (std::vector<WettbewerbsTeilnahme*>::const_iterator it = wettbewerbsTeilnahmeVector.begin(); 
										it != wettbewerbsTeilnahmeVector.end(); it++) {
		persistents.push_back(*it);
	}
}

void ExportHelper::addAlleTechnischenDaten(DatabaseContext *context, DataSet *dataSet) {
	dataSet->addAllTuplesOfEntity("Benutzer");
	dataSet->addAllTuplesOfEntity("CodeType");
	dataSet->addAllTuplesOfEntity("CodeValue");
	dataSet->addAllTuplesOfEntity("DataModelInfo");
	dataSet->addAllTuplesOfEntity("EventLog");
	dataSet->addAllTuplesOfEntity("EventLogCategory");
	dataSet->addAllTuplesOfEntity("EventLogSettings");
	dataSet->addAllTuplesOfEntity("GlobaleEinstellung");
	dataSet->addAllTuplesOfEntity("QueryLog");
	dataSet->addAllTuplesOfEntity("QueryLogSettings");
	dataSet->addAllTuplesOfEntity("Session");
}

void ExportHelper::addTechnischeDaten(DatabaseContext *context, std::vector<Persistent*> &persistents) {
	BenutzerController benutzerController = BenutzerController();
	CodeTypeController codeTypeController = CodeTypeController();
	CodeValueController codeValueController = CodeValueController();
	DataModelInfoController dataModelInfoController = DataModelInfoController();
	EventLogController eventLogController = EventLogController();
	EventLogCategoryController eventLogCategoryController = EventLogCategoryController();
	EventLogSettingsController eventLogSettingsController = EventLogSettingsController();
	GlobaleEinstellungController globaleEinstellungController = GlobaleEinstellungController();
	QueryLogController queryLogController = QueryLogController();
	QueryLogSettingsController queryLogSettingsController = QueryLogSettingsController();
	SessionController sessionController = SessionController();

	std::map<QString, QVariant> emptyMap = std::map<QString, QVariant>();
	std::vector<Benutzer*> benutzerVector = benutzerController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(benutzerVector.size()) + " Datensätze vom Typ Benutzer");
	for (std::vector<Benutzer*>::const_iterator it = benutzerVector.begin(); it != benutzerVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<CodeType*> codeTypeVector = codeTypeController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(codeTypeVector.size()) + " Datensätze vom Typ CodeType");
	for (std::vector<CodeType*>::const_iterator it = codeTypeVector.begin(); it != codeTypeVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<CodeValue*> codeValueVector = codeValueController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(codeValueVector.size()) + " Datensätze vom Typ CodeValue");
	for (std::vector<CodeValue*>::const_iterator it = codeValueVector.begin(); it != codeValueVector.end(); it++) {
		persistents.push_back(*it);
	}
	  std::vector<DataModelInfo*> dataModelInfoVector = dataModelInfoController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(dataModelInfoVector.size()) + " Datensätze vom Typ DataModelInfo");
	for (std::vector<DataModelInfo*>::const_iterator it = dataModelInfoVector.begin(); it != dataModelInfoVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<EventLog*> eventLogVector = eventLogController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(eventLogVector.size()) + " Datensätze vom Typ EventLog");
	for (std::vector<EventLog*>::const_iterator it = eventLogVector.begin(); it != eventLogVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<EventLogCategory*> eventLogCategoryVector = eventLogCategoryController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(eventLogCategoryVector.size()) + " Datensätze vom Typ EventLogCategory");
	for (std::vector<EventLogCategory*>::const_iterator it = eventLogCategoryVector.begin(); 
											  it != eventLogCategoryVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<EventLogSettings*> eventLogSettingsVector = eventLogSettingsController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(eventLogSettingsVector.size()) + " Datensätze vom Typ EventLogSettings");
	for (std::vector<EventLogSettings*>::const_iterator it = eventLogSettingsVector.begin(); 
											  it != eventLogSettingsVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<GlobaleEinstellung*> globaleEinstellungVector = globaleEinstellungController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(globaleEinstellungVector.size()) + " Datensätze vom Typ GlobaleEinstellung");
	for (std::vector<GlobaleEinstellung*>::const_iterator it = globaleEinstellungVector.begin(); 
											  it != globaleEinstellungVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<QueryLog*> queryLogVector = queryLogController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(queryLogVector.size()) + " Datensätze vom Typ QueryLog");
	for (std::vector<QueryLog*>::const_iterator it = queryLogVector.begin(); it != queryLogVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<QueryLogSettings*> queryLogSettingsVector = queryLogSettingsController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(queryLogSettingsVector.size()) + " Datensätze vom Typ QueryLogSettings");
	for (std::vector<QueryLogSettings*>::const_iterator it = queryLogSettingsVector.begin(); 
											  it != queryLogSettingsVector.end(); it++) {
		persistents.push_back(*it);
	}
	std::vector<Session*> sessionVector = sessionController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(sessionVector.size()) + " Datensätze vom Typ Session");
	for (std::vector<Session*>::const_iterator it = sessionVector.begin(); it != sessionVector.end(); it++) {
		persistents.push_back(*it);
	}
}

void ExportHelper::exportiere(DatabaseContext *context, DataSet *dataSet, QString fileName) {
	context->log("ImportExport", "ExportHelper", 7, "Exportiere in Datei " + fileName);

	// export
	DataExporter *exporter = new CSVDataExporter();
	exporter->exportData(context, fileName, dataSet);

	context->log("ImportExport", "ExportHelper", 7, "Export beendet.");

	delete exporter;
	delete dataSet;
}

void ExportHelper::exportiere(DatabaseContext *context, std::vector<Persistent*> persistents, QString fileName) {

	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(persistents.size()) + " in Datei " + fileName);
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	// export
	exporter->exportData(context, fileName, dataSet);

	context->log("ImportExport", "ExportHelper", 7, "Export beendet.");

	delete exporter;
	delete dataSet;
}
