#include "AdressdatenExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <set>
#include <vector>

#include <QAbstractListModel>

#include "ExportParameters.h"
#include "ExportPath.h"
#include "Persistent.h"
#include "SortHelper.h"

AdressdatenExportCallStrategy::AdressdatenExportCallStrategy(std::vector<Mitglied*> mitglieder) {
	this->mitglieder = mitglieder;
}

void AdressdatenExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	for (std::vector<Mitglied*>::const_iterator it = mitglieder.begin(); it != mitglieder.end(); it++) {
		persistents.push_back(*it);
	}

	context->log("ImportExport", "AdressdatenExportCallStrategy", 6, "Exportiere die Adressdaten von " + QString::number(persistents.size()) + " Mitgliedern.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> mitgliedsVector = std::vector<ExportPath*>();

	QStringList datumGeburtList = QStringList();
	datumGeburtList << "datumGeburt";
	mitgliedsVector.push_back(new ExportPath(datumGeburtList, "Geburtsdatum"));

	QStringList anredeList = QStringList();
	anredeList << "anrede";
	mitgliedsVector.push_back(new ExportPath(anredeList, "Anrede"));

	QStringList titelList = QStringList();
	titelList << "titel";
	mitgliedsVector.push_back(new ExportPath(titelList, "Titel"));

	QStringList vornameList = QStringList();
	vornameList << "vorname";
	mitgliedsVector.push_back(new ExportPath(vornameList, "Vorname"));

	QStringList nachnameList = QStringList();
	nachnameList << "nachname";
	mitgliedsVector.push_back(new ExportPath(nachnameList, "Nachname"));

	QStringList mitgliedsNummerKomplettList = QStringList();
	mitgliedsNummerKomplettList << "mitgliedsNummerKomplett";
	mitgliedsVector.push_back(new ExportPath(mitgliedsNummerKomplettList, "Mitgliedsnummer"));

	QStringList emailList = QStringList();
	emailList << "email";
	mitgliedsVector.push_back(new ExportPath(emailList, "Mail"));

	QStringList strasseList = QStringList();
	strasseList << "strasse";
	mitgliedsVector.push_back(new ExportPath(strasseList, "Strasse"));

	QStringList plzList = QStringList();
	plzList << "plz";
	mitgliedsVector.push_back(new ExportPath(plzList, "PLZ"));

	QStringList ortList = QStringList();
	ortList << "ort";
	mitgliedsVector.push_back(new ExportPath(ortList, "Ort"));

	QStringList staatList = QStringList();
	staatList << "staat" << "kurzText";
	mitgliedsVector.push_back(new ExportPath(staatList, "Staat"));

	QStringList beitragBezahltBisList = QStringList();
	beitragBezahltBisList << "beitragBezahltBis";
	mitgliedsVector.push_back(new ExportPath(beitragBezahltBisList, "Beitrag Bezahlt Bis"));

	QStringList hoechsterTitelList = QStringList();
	hoechsterTitelList << "hoechsterTitel" << "titel";
	mitgliedsVector.push_back(new ExportPath(hoechsterTitelList, "Hoechster Titel"));

	exportPaths["Mitglied"] = mitgliedsVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


