#include "BildNummernExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <set>
#include <vector>

#include <QAbstractListModel>

#include "Bild.h"
#include "BildController.h"
#include "ExportParameters.h"
#include "ExportPath.h"
#include "Persistent.h"
#include "SortHelper.h"
#include "Wertung.h"
#include "WertungController.h"

BildNummernExportCallStrategy::BildNummernExportCallStrategy(Durchgang *durchgang) {
	this->durchgang = durchgang;
}

void BildNummernExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	WertungController wertungController = WertungController();
	std::vector<Wertung*> wertungen = wertungController.getByDurchgang(context, durchgang);

	std::vector<QAbstractListModel*> models = std::vector<QAbstractListModel*>();
	for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		models.push_back(*it);
	}

	SortHelper::sort(models, Wertung::POSITIONIMDURCHGANG, SortHelper::INT_SORT, Qt::AscendingOrder);

	for (std::vector<QAbstractListModel*>::const_iterator it = models.begin(); it != models.end(); it++) {
		persistents.push_back((Persistent*)(*it));
	}

	context->log("ImportExport", "BildNummernExportCallStrategy", 6, "Exportiere die Reihenfolge von " + QString::number(wertungen.size()) + " Bildern.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> wertungsVector = std::vector<ExportPath*>();

	QStringList reihenfolgeList = QStringList();
	reihenfolgeList << "positionImDurchgang";
	wertungsVector.push_back(new ExportPath(reihenfolgeList, "Reihenfolge"));

	QStringList nummerProWettbewerbList = QStringList();
	nummerProWettbewerbList << "bild" << "bildDaten" << "nummerProWettbewerb";
	wertungsVector.push_back(new ExportPath(nummerProWettbewerbList, "Bildnummer im Wettbewerb"));

	QStringList titelList = QStringList();
	titelList << "bild" << "titel";
	wertungsVector.push_back(new ExportPath(titelList, "Titel"));

	QStringList vornameList = QStringList();
	vornameList << "bild" << "wettbewerbsTeilnahme" << "mitglied" << "vorname";
	wertungsVector.push_back(new ExportPath(vornameList, "Vorname"));

	QStringList nachnameList = QStringList();
	nachnameList << "bild" << "wettbewerbsTeilnahme" << "mitglied" << "nachname";
	wertungsVector.push_back(new ExportPath(nachnameList, "Nachname"));

	QStringList wnrList = QStringList();
	wnrList << "bild" << "wettbewerbsTeilnahme" << "mitglied" << "mitgliedsNummerKomplett";
	wertungsVector.push_back(new ExportPath(wnrList, "Mitgliedsnummer"));

	exportPaths["Wertung"] = wertungsVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


