#include "CompositeExportCallStrategy.h"

#include "ArchiveHelper.h"
#include "ExportHelper.h"

#include <QDir>
#include <QFile>
#include <QFileInfo>

CompositeExportCallStrategy::CompositeExportCallStrategy() {
	this->fileNameToStrategy = std::map<QString, ExportCallStrategy*>();
}

CompositeExportCallStrategy::~CompositeExportCallStrategy() {
	for (std::map<QString, ExportCallStrategy*>::iterator it = fileNameToStrategy.begin(); it != fileNameToStrategy.end(); it++) {
		delete it->second;
	}
}

void CompositeExportCallStrategy::addStrategy(QString fileName, ExportCallStrategy* callStrategy) {
	this->fileNameToStrategy[fileName] = callStrategy;
}

void CompositeExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {
	QFile *file = new QFile(fileName);
	QFileInfo fileInfo = QFileInfo(*file);
	QDir dir = fileInfo.absoluteDir();

	for (std::map<QString, ExportCallStrategy*>::const_iterator it = fileNameToStrategy.begin(); it != fileNameToStrategy.end(); it++) {
		QString fileNameWithoutSuffix = it->first;
		fileNameWithoutSuffix = ArchiveHelper::convertFileNameForOutput(fileNameWithoutSuffix);
		ExportCallStrategy *callStrategy = it->second;

		QString completeFileName = dir.filePath(fileNameWithoutSuffix + ".csv");
		callStrategy->exportData(context, completeFileName);
	}
}
