#include "FunktionstraegerExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"
#include "Funktionstraeger.h"
#include "FunktionstraegerController.h"

#include <set>
#include <vector>

#include <QAbstractListModel>

#include "ExportParameters.h"
#include "ExportPath.h"
#include "Persistent.h"
#include "SortHelper.h"

FunktionstraegerExportCallStrategy::FunktionstraegerExportCallStrategy() {
}

void FunktionstraegerExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	FunktionstraegerController funktionstraegerController = FunktionstraegerController();

	std::map<QString, QVariant> emptyMap = std::map<QString, QVariant>();
	std::vector<Funktionstraeger*> funktionstraegerVector = funktionstraegerController.get(context, emptyMap);
	context->log("ImportExport", "ExportHelper", 7, "Exportiere " + QString::number(funktionstraegerVector.size()) + " Datensätze vom Typ Funktionstraeger");
	for (std::vector<Funktionstraeger*>::const_iterator it = funktionstraegerVector.begin(); 
														  it != funktionstraegerVector.end(); it++) {
		Funktionstraeger *funktionstraeger = *it;
		if (funktionstraeger->hasFunktionsCode()) {
			persistents.push_back(*it);
		}
	}

	context->log("ImportExport", "FunktionstraegerExportCallStrategy", 6, "Exportiere " + QString::number(persistents.size()) + " Funktionstraeger.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> funktionstraegerPathVector = std::vector<ExportPath*>();

	QStringList vereinsList = QStringList();
	vereinsList << "verein" << "name";
	funktionstraegerPathVector.push_back(new ExportPath(vereinsList, "Verein"));

	QStringList mitgliedsnrList = QStringList();
	mitgliedsnrList << "mitglied" << "mitgliedsNummerKomplett";
	funktionstraegerPathVector.push_back(new ExportPath(mitgliedsnrList, "Mitgliedsnummer"));

	QStringList vornameList = QStringList();
	vornameList << "mitglied" << "vorname";
	funktionstraegerPathVector.push_back(new ExportPath(vornameList, "Vorname"));

	QStringList nachnameList = QStringList();
	nachnameList << "mitglied" << "nachname";
	funktionstraegerPathVector.push_back(new ExportPath(nachnameList, "Nachname"));

	QStringList funktionsList = QStringList();
	funktionsList << "funktionsCode" << "kurzText";
	funktionstraegerPathVector.push_back(new ExportPath(funktionsList, "Funktion"));

	QStringList funktionsFreitextList = QStringList();
	funktionsFreitextList << "funktion";
	funktionstraegerPathVector.push_back(new ExportPath(funktionsFreitextList, "Funktion (Freitext)"));

	QStringList anredeList = QStringList();
	anredeList << "mitglied" << "anrede";
	funktionstraegerPathVector.push_back(new ExportPath(anredeList, "Anrede"));

	QStringList titelList = QStringList();
	titelList << "mitglied" << "titel";
	funktionstraegerPathVector.push_back(new ExportPath(titelList, "Titel"));

	QStringList emailList = QStringList();
	emailList << "mitglied" << "email";
	funktionstraegerPathVector.push_back(new ExportPath(emailList, "Mail"));

	QStringList strasseList = QStringList();
	strasseList << "mitglied" << "strasse";
	funktionstraegerPathVector.push_back(new ExportPath(strasseList, "Strasse"));

	QStringList plzList = QStringList();
	plzList << "mitglied" << "plz";
	funktionstraegerPathVector.push_back(new ExportPath(plzList, "PLZ"));

	QStringList ortList = QStringList();
	ortList << "mitglied" << "ort";
	funktionstraegerPathVector.push_back(new ExportPath(ortList, "Ort"));

	QStringList staatList = QStringList();
	staatList << "mitglied" << "staat" << "kurzText";
	funktionstraegerPathVector.push_back(new ExportPath(staatList, "Staat"));

	exportPaths["Funktionstraeger"] = funktionstraegerPathVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


