#include "StatistikDataExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <set>
#include <vector>

#include <QAbstractListModel>

#include "ExportParameters.h"
#include "ExportPath.h"
#include "Persistent.h"
#include "StatistikConverter.h"
#include "StatistikData.h"
#include "StatistikDataController.h"
#include "StatistikDataSearchModel.h"

StatistikDataExportCallStrategy::StatistikDataExportCallStrategy(Wettbewerb *wettbewerb) {
	this->wettbewerb = wettbewerb;
}

void StatistikDataExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {
	std::vector<Persistent*> persistents = std::vector<Persistent*>();

	StatistikDataController controller = StatistikDataController();
	StatistikDataSearchModel *searchModel = new StatistikDataSearchModel();
	searchModel->setWettbewerb(wettbewerb->getId());
	searchModel->setTopLevel(true);

	std::vector<StatistikData*> dataVector = controller.get(context, searchModel);
	delete searchModel;

	for (std::vector<StatistikData*>::const_iterator it = dataVector.begin(); it != dataVector.end(); it++) {
		persistents.push_back(*it);
	}

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> dataPathVector = std::vector<ExportPath*>();

	QStringList idList = QStringList();
	idList << "id";
	dataPathVector.push_back(new ExportPath(idList, "Id", new StatistikConverter()));

	exportPaths["StatistikData"] = dataPathVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}
