#include "TeilnehmerKombinationsWertungExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"
#include "ExportPath.h"

#include <set>

#include "ExportParameters.h"
#include "Persistent.h"

TeilnehmerKombinationsWertungExportCallStrategy::TeilnehmerKombinationsWertungExportCallStrategy(std::vector<TeilnehmerKombinationsWertung*> teilnehmerKombinationsWertungen) {
	this->teilnehmerKombinationsWertungen = teilnehmerKombinationsWertungen;
}

void TeilnehmerKombinationsWertungExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();
	for (std::vector<TeilnehmerKombinationsWertung*>::const_iterator it2 = teilnehmerKombinationsWertungen.begin(); it2 != teilnehmerKombinationsWertungen.end(); it2++) {
		persistents.push_back(*it2);
	}

	context->log("ImportExport", "TeilnehmerKombinationsWertungExportCallStrategy", 6, "Exportiere " + QString(teilnehmerKombinationsWertungen.size()) + " TeilnehmerKombinationsWertungen.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> wertungVector = std::vector<ExportPath*>();

	QStringList vereinList = QStringList();
	vereinList << "verein" << "name";
	wertungVector.push_back(new ExportPath(vereinList, "Verein"));

	QStringList mitgliedsnrList = QStringList();
	mitgliedsnrList << "wettbewerbsTeilnahme" << "mitglied" << "mitgliedsNummerKomplett";
	wertungVector.push_back(new ExportPath(mitgliedsnrList, "Mitgliedsnummer"));

	QStringList vornameList = QStringList();
	vornameList << "wettbewerbsTeilnahme" << "mitglied" << "vorname";
	wertungVector.push_back(new ExportPath(vornameList, "Vorname"));

	QStringList nachnameList = QStringList();
	nachnameList << "wettbewerbsTeilnahme" << "mitglied" << "nachname";
	wertungVector.push_back(new ExportPath(nachnameList, "Nachname"));

	QStringList punkteList = QStringList();
	punkteList << "punkte";
	wertungVector.push_back(new ExportPath(punkteList, "Punkte"));

	exportPaths["TeilnehmerKombinationsWertung"] = wertungVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


