#include "TeilnehmerWertungExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <set>

#include "ExportParameters.h"
#include "Persistent.h"

TeilnehmerWertungExportCallStrategy::TeilnehmerWertungExportCallStrategy(std::vector<TeilnehmerWertung*> teilnehmerWertungen) {
	this->teilnehmerWertungen = teilnehmerWertungen;
}

void TeilnehmerWertungExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();
	for (std::vector<TeilnehmerWertung*>::const_iterator it2 = teilnehmerWertungen.begin(); it2 != teilnehmerWertungen.end(); it2++) {
		persistents.push_back(*it2);
	}

	context->log("ImportExport", "TeilnehmerWertungExportCallStrategy", 6, "Exportiere " + QString(teilnehmerWertungen.size()) + " TeilnehmerWertungen.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> wertungVector = std::vector<ExportPath*>();

	QStringList vornameList = QStringList();
	vornameList << "wettbewerbsTeilnahme" << "mitglied" << "vorname";
	wertungVector.push_back(new ExportPath(vornameList, "Vorname"));

	QStringList nachnameList = QStringList();
	nachnameList << "wettbewerbsTeilnahme" << "mitglied" << "nachname";
	wertungVector.push_back(new ExportPath(nachnameList, "Nachname"));

	QStringList mitgliedsnrList = QStringList();
	mitgliedsnrList << "wettbewerbsTeilnahme" << "mitglied" << "mitgliedsNummerKomplett";
	wertungVector.push_back(new ExportPath(mitgliedsnrList, "Mitgliedsnummer"));

	QStringList vereinList = QStringList();
	vereinList << "verein" << "name";
	wertungVector.push_back(new ExportPath(vereinList, "Verein"));

	QStringList sparteList = QStringList();
	sparteList << "sparte" << "name";
	wertungVector.push_back(new ExportPath(sparteList, "Sparte"));

	QStringList punkteList = QStringList();
	punkteList << "punkte";
	wertungVector.push_back(new ExportPath(punkteList, "Punkte"));

	exportPaths["TeilnehmerWertung"] = wertungVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


