#include "VereinsWertungExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <set>

#include "ExportParameters.h"
#include "Persistent.h"

VereinsWertungExportCallStrategy::VereinsWertungExportCallStrategy(std::vector<VereinsWertung*> vereinsWertungen) {
	this->vereinsWertungen = vereinsWertungen;
}

void VereinsWertungExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = std::vector<Persistent*>();
	for (std::vector<VereinsWertung*>::const_iterator it2 = vereinsWertungen.begin(); it2 != vereinsWertungen.end(); it2++) {
		persistents.push_back(*it2);
	}

	context->log("ImportExport", "VereinsWertungExportCallStrategy", 6, "Exportiere " + QString(vereinsWertungen.size()) + " VereinsWertungen.");

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	std::map<QString, std::vector<ExportPath*> > exportPaths = std::map<QString, std::vector<ExportPath*> >();
	std::vector<ExportPath*> wertungVector = std::vector<ExportPath*>();

	QStringList vereinsNummerList = QStringList();
	vereinsNummerList << "verein" << "vereinsNummerKomplett";
	wertungVector.push_back(new ExportPath(vereinsNummerList, "Vereinsnummer"));

	QStringList vereinList = QStringList();
	vereinList << "verein" << "name";
	wertungVector.push_back(new ExportPath(vereinList, "Verein"));

	QStringList sparteList = QStringList();
	sparteList << "sparte" << "name";
	wertungVector.push_back(new ExportPath(sparteList, "Sparte"));

	QStringList punkteList = QStringList();
	punkteList << "punkte";
	wertungVector.push_back(new ExportPath(punkteList, "Punkte"));

	exportPaths["VereinsWertung"] = wertungVector;

	ExportParameters *exportParameters = new ExportParameters(false, false, false, true, exportPaths);

	// export
	exporter->exportData(context, fileName, dataSet, exportParameters);

	delete exporter;
	delete dataSet;
	delete exportParameters;
}


