#include "WettbewerbMitMitgliedernExportCallStrategy.h"

#include "CSVDataExporter.h"
#include "DataExporter.h"
#include "DataSet.h"
#include "ExportHelper.h"

#include <vector>

#include "ExternerJuror.h"
#include "ExternerJurorController.h"
#include "JuriertBei.h"
#include "JuriertBeiController.h"
#include "Mitglied.h"
#include "MitgliedController.h"
#include "Persistent.h"
#include "WettbewerbsTeilnahme.h"
#include "WettbewerbsTeilnahmeController.h"
#include "Verein.h"
#include "VereinController.h"

WettbewerbMitMitgliedernExportCallStrategy::WettbewerbMitMitgliedernExportCallStrategy(Wettbewerb *wettbewerb) {
	this->wettbewerb = wettbewerb;
}

void WettbewerbMitMitgliedernExportCallStrategy::exportData(DatabaseContext *context, QString fileName) {

	std::vector<Persistent*> persistents = ExportHelper::getPersistentsZuWettbewerb(context, wettbewerb);

	WettbewerbsTeilnahmeController teilnahmeController = WettbewerbsTeilnahmeController();
	MitgliedController mitgliedsController = MitgliedController();
	VereinController vereinController = VereinController();

	// Die ids aller Vereine
	std::set<qlonglong> mitgliedsIds = std::set<qlonglong>();
	std::set<qlonglong> vereinsIds = std::set<qlonglong>();
	std::set<qlonglong> externerJurorIds = std::set<qlonglong>();
	std::vector<WettbewerbsTeilnahme*> teilnahmeVector = teilnahmeController.getByWettbewerb(context, wettbewerb);
	for (std::vector<WettbewerbsTeilnahme*>::const_iterator it = teilnahmeVector.begin(); 
													  it != teilnahmeVector.end(); it++) {
		// Die Teilnahme-Instanzen nicht hinzufügen, wir haben es schon vorher gemacht.
		WettbewerbsTeilnahme *teilnahme = *it;
		qlonglong mitgliedsId = teilnahme->getMitglied();
		mitgliedsIds.insert(mitgliedsId);
	}

	JuriertBeiController juriertBeiController = JuriertBeiController();
	std::vector<JuriertBei*> juriertBeiVector = juriertBeiController.getByWettbewerb(context, wettbewerb);
	for (std::vector<JuriertBei*>::const_iterator it = juriertBeiVector.begin(); it != juriertBeiVector.end(); it++) {
		// Die JuriertBei-Instanzen nicht hinzufügen, das machen wir in
		// getPersistentsZuWettbewerb.  Wohl aber müssen wir uns hier um
		// die dranhängenden Mitglieder und externe Juroren kümmern.
		// Die werden nur zum Jurierungsrechner hin, aber nicht davon weg
		// transferiert.
		JuriertBei *juriertBei = *it;
		if (juriertBei->hasMitglied()) {
			mitgliedsIds.insert(juriertBei->getMitglied());
		}
		if (juriertBei->hasExternerJuror()) {
			externerJurorIds.insert(juriertBei->getExternerJuror());
		}
	}

	for (std::set<qlonglong>::const_iterator it = mitgliedsIds.begin(); it != mitgliedsIds.end(); it++) {
		qlonglong mitgliedsId = *it;
		Mitglied *mitglied = mitgliedsController.getByPrimaryKey(context, mitgliedsId);
		if (mitglied->hasVerein()) {
			vereinsIds.insert(mitglied->getVerein());
		}
		Mitglied *reduziertesMitglied = ExportHelper::filtereMitgliedsAttribute(context, mitglied);
		delete mitglied;

		persistents.push_back(reduziertesMitglied);
	}

	ExternerJurorController externerJurorController = ExternerJurorController();
	for (std::set<qlonglong>::const_iterator it = externerJurorIds.begin(); it != externerJurorIds.end(); it++) {
		qlonglong externerJurorId = *it;
		ExternerJuror *externerJuror = externerJurorController.getByPrimaryKey(context, externerJurorId);
		persistents.push_back(externerJuror);
	}

	for (std::set<qlonglong>::const_iterator it = vereinsIds.begin(); it != vereinsIds.end(); it++) {
		qlonglong vereinsId = *it;
		Verein *verein = vereinController.getByPrimaryKey(context, vereinsId);
		persistents.push_back(verein);
	}

	// construct Exporter and DataSet
	DataExporter *exporter = new CSVDataExporter();
	DataSet *dataSet = exporter->constructDataSetForExport();
	exporter->addToDataSetForExport(context, dataSet, persistents);

	// export
	exporter->exportData(context, fileName, dataSet);

	delete exporter;
	delete dataSet;
}


