#include "EinzelWertungenConverter.h"

#include <QObject>


#include "BlockExportColumn.h"
#include "DataExportColumn.h"
#include "Durchgang.h"
#include "EinzelWertungsListeJoinController.h"
#include "WertungsHelper.h"

#include "EinzelwertungenExterneJurorenJoinController.h"
#include "EinzelwertungenMitgliederJoinController.h"
#include "EinzelwertungsAnzeigeJoinModel.h"
#include "EinzelwertungsAnzeigeJoinSearchModel.h"

EinzelWertungenConverter::EinzelWertungenConverter(std::vector<EinzelWertungsListeJoinModel*> joinModels, std::map<qlonglong, Person*> juriertBeiIdToPerson) {
	this->joinModels = joinModels;
	this->juriertBeiIdToPerson = juriertBeiIdToPerson;
}

void EinzelWertungenConverter::convert(DatabaseContext *context, DataExportColumn *exportColumn) {
	context->log("ImportExport", "EinzelWertungenConverter", 6, "EinzelWertungenConverter gestartet.");


	// Problem: The joinModels are unordered.  But the Bild ids given in the exportColumn are ordered.
	// So, here for each joinModel (identified by its EinzelWertung id in a unique way)
	// we must find out its corresponding row, as given by the exportColumn ids
	std::map<qlonglong, int> einzelWertungsIdToOrder = std::map<qlonglong, int>();
	for (std::vector<EinzelWertungsListeJoinModel*>::const_iterator it = joinModels.begin(); it != joinModels.end(); it++) {
		EinzelWertungsListeJoinModel *joinModel = *it;
		int index = exportColumn->getFirstIndexOfValue(QString::number(joinModel->getBildId()));
		einzelWertungsIdToOrder[joinModel->getEinzelWertungId()] = index;
	}


	BlockExportColumn *durchgangsBlockColumn = new BlockExportColumn(QObject::trUtf8("Einzelwertungen der Juroren in den Durchgaengen"));
	ExportBlock *durchgangsBlock = new ExportBlock();
	durchgangsBlockColumn->setExportBlock(durchgangsBlock);

	std::map<qlonglong, Durchgang*> durchgangsIdToDurchgang = std::map<qlonglong, Durchgang*>();
	for (std::vector<EinzelWertungsListeJoinModel*>::const_iterator it = joinModels.begin(); it != joinModels.end(); it++) {
		Durchgang *durchgang = EinzelWertungsListeJoinController::extractDurchgang(*it);
		durchgangsIdToDurchgang[durchgang->getId()] = durchgang;
	}

	std::vector<Durchgang*> durchgaenge = std::vector<Durchgang*>();
	// TODO: Sortierung beachten!
	for (std::map<qlonglong, Durchgang*>::const_iterator it = durchgangsIdToDurchgang.begin(); it != durchgangsIdToDurchgang.end(); it++) {
		durchgaenge.push_back(it->second);
	}

	std::map<QString, BlockExportColumn*> durchgangsNameToColumn = std::map<QString, BlockExportColumn*>();
	std::map<QString, DataExportColumn*> durchgangsNameToWertungsColumn = std::map<QString, DataExportColumn*>();
	for (std::vector<Durchgang*>::const_iterator it = durchgaenge.begin(); it != durchgaenge.end(); it++) {
		Durchgang *durchgang = *it;

		QString durchgangsString = QString::number(durchgang->getId());
		if (WertungsHelper::istErsterDurchgang(context, durchgang)) {
			durchgangsString = QObject::trUtf8("Erster Durchgang");
		} else if (WertungsHelper::istZweiterDurchgang(context, durchgang)) {
			durchgangsString = QObject::trUtf8("Zweiter Durchgang");
		}

		BlockExportColumn *currDurchgangsBlockColumn;
		DataExportColumn *currWertungsColumn;
		ExportBlock *currDurchgangsBlock;
		bool blockNeu;
		if (durchgangsNameToColumn.find(durchgangsString) == durchgangsNameToColumn.end()) {
			currDurchgangsBlockColumn = new BlockExportColumn(durchgangsString);
			currDurchgangsBlock = new ExportBlock();
			currWertungsColumn = new DataExportColumn("Summe", NULL);
			currDurchgangsBlockColumn->setExportBlock(currDurchgangsBlock);
			durchgangsNameToColumn[durchgangsString] = currDurchgangsBlockColumn;
			durchgangsNameToWertungsColumn[durchgangsString] = currWertungsColumn;
			blockNeu = true;
		} else {
			currDurchgangsBlockColumn = durchgangsNameToColumn[durchgangsString];
			currWertungsColumn = durchgangsNameToWertungsColumn[durchgangsString];
			currDurchgangsBlock = currDurchgangsBlockColumn->getExportBlock();
			blockNeu = false;
		}

		std::map<qlonglong, DataExportColumn*> jurorIdToEinzelWertungColumn = std::map<qlonglong, DataExportColumn*>();
		for (int n = 0; n < joinModels.size(); n++) {
			EinzelWertungsListeJoinModel *currJoinModel = joinModels[n];
			if (currJoinModel->getDurchgangId() == durchgang->getId() && currJoinModel->hasEinzelWertungJuror() 
				&& currJoinModel->hasJuriertBeiId() && juriertBeiIdToPerson.find(currJoinModel->getJuriertBeiId()) != juriertBeiIdToPerson.end()) {
				qlonglong einzelWertungId = currJoinModel->getEinzelWertungId();
				qlonglong juriertBeiId = currJoinModel->getJuriertBeiId();
				QString bewertung = (currJoinModel->hasEinzelWertungBewertung() ? QString::number(currJoinModel->getEinzelWertungBewertung()) : QObject::trUtf8("-"));
				int row = einzelWertungsIdToOrder[einzelWertungId];

				if (currDurchgangsBlock->hasColumn(QString::number(juriertBeiId))) {
					DataExportColumn *dataColumn = (DataExportColumn*)(currDurchgangsBlock->getColumn(QString::number(juriertBeiId)));
					dataColumn->setValue(row, bewertung);
				} else {
					Person *person = juriertBeiIdToPerson[juriertBeiId];
					QString vorname = (person->hasVorname() ? person->getVorname() : QObject::trUtf8("(kein Vorname)"));
					QString nachname = (person->hasNachname() ? person->getNachname() : QObject::trUtf8("(kein Nachname)"));
					DataExportColumn *dataColumn = new DataExportColumn(vorname + " " + nachname, NULL);
					dataColumn->setValue(row, bewertung);
					currDurchgangsBlock->addColumn(QString::number(juriertBeiId), dataColumn);
				}
/*
				bool found = false;
				int z = 0;
				std::vector<ExportColumn*> dataColumns = currDurchgangsBlock->getAllColumns();
				for (std::vector<ExportColumn*>::const_iterator it = dataColumns.begin(); it != dataColumns.end(); it++) {
					DataExportColumn *dataColumn = (DataExportColumn*)(*it);
					if (!dataColumn->hasValue(row)) {
						dataColumn->setValue(row, bewertung);
						found = true;
						break;
					}
					z++;
				}

				if (!found) {
					DataExportColumn *newColumn = new DataExportColumn("", NULL);
					newColumn->setValue(row, bewertung);
					currDurchgangsBlock->addColumn(QString::number(z), newColumn);
				}
*/
				/*
				qlonglong jurorId = currJoinModel->getEinzelWertungJuror();
				if (jurorIdToEinzelWertungColumn.find(jurorId) == jurorIdToEinzelWertungColumn.end()) {
					// Alternativ: JuriertBei-Id als Caption (QString::number(jurorId)), oder der Name des Jurors
					jurorIdToEinzelWertungColumn[jurorId] = new DataExportColumn("", NULL);
					currDurchgangsBlock->addColumn(QString::number(jurorId), jurorIdToEinzelWertungColumn[jurorId]);
				}

				qlonglong einzelWertungId = currJoinModel->getEinzelWertungId();
				int row = einzelWertungsIdToOrder[einzelWertungId];

				DataExportColumn *dataExportColumn = jurorIdToEinzelWertungColumn[jurorId];
				QString bewertung = (currJoinModel->hasEinzelWertungBewertung() ? QString::number(currJoinModel->getEinzelWertungBewertung()) : QObject::trUtf8("-"));
				dataExportColumn->setValue(row, bewertung);
				context->log("ImportExport", "EinzelWertungenConverter", 1, "Setze Bewertung " + bewertung + " fuer Zeile " + QString::number(row) + " und Juroren-Id "
																			 + QString::number(jurorId));
				*/
				QString summe = (currJoinModel->hasWertungBewertung() ? QString::number(currJoinModel->getWertungBewertung()) : QObject::trUtf8("-"));
				currWertungsColumn->setValue(row, summe);
			}
		}
		//currDurchgangsBlock->addColumn("Summe", wertungsColumn);

		// Fill up rows, they should all have the length of the longest one
		int numberOfRows = exportColumn->getSuperParent()->getMaxDataRow() + 1;
		for (std::map<qlonglong, DataExportColumn*>::const_iterator it = jurorIdToEinzelWertungColumn.begin(); it != jurorIdToEinzelWertungColumn.end(); it++) {
			DataExportColumn *column = it->second;
			for (int z = 0; z < numberOfRows; z++) {
				if (!column->hasValue(z)) {
					column->setValue(z, "");
				}
			}
		}

		if (blockNeu) {
			durchgangsBlock->addColumn(QString::number(durchgang->getId()), currDurchgangsBlockColumn);
		}
	}

	for (std::map<QString, BlockExportColumn*>::const_iterator it = durchgangsNameToColumn.begin(); it != durchgangsNameToColumn.end(); it++) {
		QString durchgangsName = it->first;
		BlockExportColumn *blockColumn = it->second;
		blockColumn->getExportBlock()->addColumn("Summe", durchgangsNameToWertungsColumn[durchgangsName]);
	}

	exportColumn->getParent()->exchangeColumn(exportColumn->getKey(), durchgangsBlockColumn);

	context->log("ImportExport", "EinzelWertungenConverter", 6, "EinzelWertungenConverter fertig.");
}

std::vector<QVariant> EinzelWertungenConverter::convertData(DatabaseContext *context, QVariant data) {
/*
	EinzelwertungenExterneJurorenJoinController externeJurorenController = EinzelwertungenExterneJurorenJoinController();
	EinzelwertungenMitgliederJoinController mitgliederController = EinzelwertungenMitgliederJoinController();
	EinzelwertungsAnzeigeJoinSearchModel *searchModel = new EinzelwertungsAnzeigeJoinSearchModel();
	searchModel->setEinzelWertungWertung(data.toLongLong());

	std::vector<EinzelwertungsAnzeigeJoinModel*> einzelWertungen = externeJurorenController.get(context, searchModel);

	std::vector<EinzelwertungsAnzeigeJoinModel*> einzelWertungenMitglieder = mitgliederController.get(context, searchModel);
	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelWertungenMitglieder.begin(); it != einzelWertungenMitglieder.end(); it++) {
		einzelWertungen.push_back(*it);
	}
	delete searchModel;

	std::vector<QVariant> retData = std::vector<QVariant>();

	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelWertungen.begin(); it != einzelWertungen.end(); it++) {
		EinzelwertungsAnzeigeJoinModel *einzelWertung = *it;

		QString vorname = (einzelWertung->hasPersonVorname() ? einzelWertung->getPersonVorname() : "ohne Vorname");
		QString nachname = (einzelWertung->hasPersonNachname() ? einzelWertung->getPersonNachname() : "ohne Nachname");
		QString bewertung = (einzelWertung->hasEinzelWertungBewertung() ? QString::number(einzelWertung->getEinzelWertungBewertung()) : "ohne Bewertung");

		retData.push_back(QVariant(vorname));
		retData.push_back(QVariant(nachname));
		retData.push_back(QVariant(bewertung));
	}

	return retData;*/
	return std::vector<QVariant>();
}


