#include "StatistikConverter.h"

#include <QObject>


#include "BlockExportColumn.h"
#include "DataExportColumn.h"
#include "Durchgang.h"
#include "StatistikData.h"
#include "StatistikDataController.h"

StatistikConverter::StatistikConverter() {
}

void StatistikConverter::convert(DatabaseContext *context, DataExportColumn *exportColumn) {
	context->log("ImportExport", "StatistikConverter", 6, "StatistikConverter gestartet.");

	StatistikDataController controller = StatistikDataController();

	std::vector<QString> topIds = exportColumn->getData();
	std::vector<StatistikData*> topDataVector = std::vector<StatistikData*>();
	for (std::vector<QString>::const_iterator it = topIds.begin(); it != topIds.end(); it++) {
		QString topIdString = *it;

		bool ok;
		qlonglong primaryKey = topIdString.toLongLong(&ok);
		if (!ok) {
			// todo maybe error message / exception
			continue;
		}
		topDataVector.push_back(controller.getByPrimaryKey(context, primaryKey));
	}

	BlockExportColumn *blockColumn = new BlockExportColumn(QObject::trUtf8("Statistik"));
	ExportBlock *exportBlock = new ExportBlock();
	blockColumn->setExportBlock(exportBlock);

	std::vector<DataExportColumn*> dataColumns = std::vector<DataExportColumn*>();
	fillDataColumns(context, dataColumns, topDataVector, 0, 0);

	for (unsigned int n = 0; n < dataColumns.size(); n++) {
		exportBlock->addColumn(QString::number(n), dataColumns[n]);
	}

	exportColumn->getParent()->exchangeColumn(exportColumn->getKey(), blockColumn);

	context->log("ImportExport", "StatistikConverter", 6, "StatistikConverter fertig.");
}

int StatistikConverter::fillDataColumns(DatabaseContext *context, std::vector<DataExportColumn*> &dataColumns, std::vector<StatistikData*> statistikData, int depth, int startRow) {
	int row = startRow;

	for (std::vector<StatistikData*>::const_iterator it = statistikData.begin(); it != statistikData.end(); it++) {
		StatistikData *data = *it;
		QString text = (data->hasText() ? data->getText() : QObject::trUtf8("(keine Daten"));
		for (int n = dataColumns.size(); n <= depth; n++) {
			dataColumns.push_back(new DataExportColumn("", NULL));
		}
		dataColumns[depth]->setValue(row, text);

		StatistikDataController controller = StatistikDataController();
		std::vector<StatistikData*> children = controller.getByParent(context, data);
		row = fillDataColumns(context, dataColumns, children, depth + 1, row + 1) + 1;
	}

	return row;
}

std::vector<QVariant> StatistikConverter::convertData(DatabaseContext *context, QVariant data) {
	return std::vector<QVariant>();
}


