#include "SummenWertungConverter.h"

#include <QObject>


#include "BlockExportColumn.h"
#include "DataExportColumn.h"
#include "Durchgang.h"
#include "EinzelWertungsListeJoinController.h"
#include "WertungsHelper.h"

#include "EinzelwertungenExterneJurorenJoinController.h"
#include "EinzelwertungenMitgliederJoinController.h"
#include "EinzelwertungsAnzeigeJoinModel.h"
#include "EinzelwertungsAnzeigeJoinSearchModel.h"

SummenWertungConverter::SummenWertungConverter(std::vector<EinzelWertungsListeJoinModel*> joinModels) {
	this->joinModels = joinModels;
}

void SummenWertungConverter::convert(DatabaseContext *context, DataExportColumn *exportColumn) {
	context->log("ImportExport", "SummenWertungConverter", 6, "SummenWertungConverter gestartet.");

	// Map MitgliedIds to JoinModels.  As all joinModels to one Mitglied share the same TeilnehmerWertung,
	// we can ignore the fact that there are multiple joinModel instances for each Mitglied.
	std::map<qlonglong, EinzelWertungsListeJoinModel*> mitgliedIdToJoinModel = std::map<qlonglong, EinzelWertungsListeJoinModel*>();
	for (std::vector<EinzelWertungsListeJoinModel*>::const_iterator it = joinModels.begin(); it != joinModels.end(); it++) {
		EinzelWertungsListeJoinModel *joinModel = *it;
		qlonglong mitgliedId = joinModel->getMitgliedId();
		mitgliedIdToJoinModel[mitgliedId] = joinModel;
	}

	std::vector<QString> oldData = exportColumn->getData();
	std::set<qlonglong> alreadyProcessedMitgliedIds = std::set<qlonglong>();
	for (int n = 0; n < oldData.size(); n++) {
		QString mitgliedIdString = oldData[n];
		qlonglong mitgliedId = mitgliedIdString.toLongLong();
		if (alreadyProcessedMitgliedIds.find(mitgliedId) == alreadyProcessedMitgliedIds.end()) {
			alreadyProcessedMitgliedIds.insert(mitgliedId);

			EinzelWertungsListeJoinModel *joinModel = mitgliedIdToJoinModel[mitgliedId];
			int summenWertung = (joinModel->hasTeilnehmerWertungPunkte() ? joinModel->getTeilnehmerWertungPunkte() : 0);
			exportColumn->setValue(n, QString::number(summenWertung));
		} else {
			exportColumn->setValue(n, "");
		}
	}

	context->log("ImportExport", "SummenWertungConverter", 6, "SummenWertungConverter fertig.");
}

std::vector<QVariant> SummenWertungConverter::convertData(DatabaseContext *context, QVariant data) {
/*
	EinzelwertungenExterneJurorenJoinController externeJurorenController = EinzelwertungenExterneJurorenJoinController();
	EinzelwertungenMitgliederJoinController mitgliederController = EinzelwertungenMitgliederJoinController();
	EinzelwertungsAnzeigeJoinSearchModel *searchModel = new EinzelwertungsAnzeigeJoinSearchModel();
	searchModel->setEinzelWertungWertung(data.toLongLong());

	std::vector<EinzelwertungsAnzeigeJoinModel*> einzelWertungen = externeJurorenController.get(context, searchModel);

	std::vector<EinzelwertungsAnzeigeJoinModel*> einzelWertungenMitglieder = mitgliederController.get(context, searchModel);
	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelWertungenMitglieder.begin(); it != einzelWertungenMitglieder.end(); it++) {
		einzelWertungen.push_back(*it);
	}
	delete searchModel;

	std::vector<QVariant> retData = std::vector<QVariant>();

	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelWertungen.begin(); it != einzelWertungen.end(); it++) {
		EinzelwertungsAnzeigeJoinModel *einzelWertung = *it;

		QString vorname = (einzelWertung->hasPersonVorname() ? einzelWertung->getPersonVorname() : "ohne Vorname");
		QString nachname = (einzelWertung->hasPersonNachname() ? einzelWertung->getPersonNachname() : "ohne Nachname");
		QString bewertung = (einzelWertung->hasEinzelWertungBewertung() ? QString::number(einzelWertung->getEinzelWertungBewertung()) : "ohne Bewertung");

		retData.push_back(QVariant(vorname));
		retData.push_back(QVariant(nachname));
		retData.push_back(QVariant(bewertung));
	}

	return retData;*/
	return std::vector<QVariant>();
}


