#ifndef CSVDATAIMPORTER_H
#define CSVDATAIMPORTER_H

#include <QFile>
#include <QObject>
#include <QString>

#include <map>
#include <vector>

#include "CSVFormat.h"
#include "DataImporter.h"
#include "DataSet.h"
#include "ImpExpSchema.h"

typedef std::vector<std::vector<QString> > CSVFileContents;
typedef std::vector<QString> CSVLine;

class CSVDataImporter : public DataImporter {
	Q_OBJECT

public:
	virtual ImpExpMetaInformation extractMetaInformation(DatabaseContext *context, QFile *file);

protected:
	virtual DataSet* readData(QFile* file);

private:
	void throwDuplicateMetaInfoLineException(QString lineType);
	void throwMissingMetaInfoLineException(QString lineType);

	CSVFileContents checkFileAndSplit(DatabaseContext *context, QFile* file);
	std::vector<QString> splitLine(int lineNumber, QString line, QString separator);

	DataSet* parseContents(CSVFileContents contents);

	/** Returns wether all of the line parts of the given line have length 0.
	 *  @return wether all of the line parts of the given line have length 0.
	 */
	bool isLineEmpty(std::vector<QString> lineParts);

	QString removeNonEscapedQuotes(QString s);

	CSVFormat *csvFormat;
};
#endif
