#include "ExtractThread.h"

#include "ArchiveHelper.h"
#include "Exception.h"

ExtractThread::ExtractThread(DatabaseContext *context, QString fileName) {
	this->context = context;
	DatenbankVerbindung *andereVerbindung = context->getDatenbankVerbindung();
	this->datenbankVerbindung = andereVerbindung->copy();
	this->session = context->getSession()->copy();

	this->fileName = fileName;
	QObject::connect(this, SIGNAL(finished()), this, SLOT(deleteLater()));
}

void ExtractThread::run() {
	try {
		context->log("Archive", "ExtractThread", 8, "Starting ExtractThread");

		this->context = new DatabaseContext(this->datenbankVerbindung, session);

		QString importFileName;
		QDir tempDir = QDir();
		bool createdTempDir = false;
		if (ArchiveHelper::isArchive(fileName)) {
			emit statusMessage(QObject::trUtf8("Entpacke das Archiv...  Bitte haben Sie Geduld, je nach Dateigroesse kann das einige Zeit dauern."));

			tempDir = ArchiveHelper::createArchiveTempDirectoryForFile(context, fileName);
			createdTempDir = true;
			context->log("Archive", "ExtractThread", 6, "Extracting archive " + fileName + " into directory " + tempDir.absolutePath());

			importFileName = ArchiveHelper::prepareForArchiveImport(context, tempDir, fileName);
			emit statusMessage(QObject::trUtf8("Entpacke das Archiv...  Abgeschlossen."));
		} else {
			importFileName = fileName;
			createdTempDir = false;
		}
		context->log("Archive", "ExtractThread", 8, "Finished ExtractThread");
		emit extractionFinished(createdTempDir, tempDir.absolutePath(), importFileName);
	} catch (Exception e) {
		context->log("Archive", "ExtractThread", 5, "Throwing exception with title " + e.getShortText()
						+ " and text " + e.getLongText());
		emit extractionFailed(e.getShortText(), e.getLongText());
	} catch (PersistenceException e) {
		context->log("Archive", "ExtractThread", 5, "Throwing exception with title " + e.getShortText()
						+ " and text " + e.getLongText());
		emit extractionFailed(e.getShortText(), e.getLongText());
	}
}
