#ifndef FOREIGNKEYDESTINATION_H
#define FOREIGNKEYDESTINATION_H

#include <QVariant>

class DataTuple;

/** During import of a data file, foreign references have to be resolved.
 *  Of course, their destinations have to exist.  However, they can exist
 *  in two places:  Either in the database or in the currently loaded
 *  data.  This class stores this information.  See also ...
 */
class ForeignKeyDestination {

public:
	ForeignKeyDestination();
	ForeignKeyDestination(QVariant keyInDatabase);
	ForeignKeyDestination(DataTuple *loadedTuple);

	bool isNull();

	bool pointsToDatabase();
	QVariant getKeyInDatabase();
	void setKeyInDatabase(QVariant keyInDatabase);
	bool pointsToTuple();
	DataTuple *getLoadedTuple();
	void setLoadedTuple(DataTuple *loadedTuple);

private:
	/** Foreign key reference in fact is NULL, so it points nowhere.
	 */
	bool fkIsNull;

	/** Some key in the database.  If this destination does not point
     *  to a database key, the invalid QVariant.
	 */
	QVariant keyInDatabase;

	/** Some currently loaded data tuple.  If this destination does not
	 *  point to a currently loaded data tuple, this pointer is NULL.
	 */
	DataTuple *loadedTuple;
};
#endif
