#include "BenutzerImportStrategy.h"

#include <QString>

#include "AttributeData.h"
#include "Benutzer.h"
#include "BenutzerController.h"
#include "BenutzerHelper.h"
#include "BenutzerSucheSearchModel.h"
#include "DataHelper.h"
#include "ImportException.h"

void BenutzerImportStrategy::checkAndConvertBeforeResolve(DatabaseContext *context, DataSet *dataSet, DataTuple *dataTuple, ImportResult *importResult) {
	if (dataTuple->getEntityDBName() != "Benutzer") {
		throw ImportException("ImportExport", "BenutzerImportStrategy", trUtf8("Technischer Fehler"),
							trUtf8("BenutzerImportStrategy falsch aufgerufen"));
	} else {
		testForMustValue(dataTuple, "benutzername");
	}
}

void BenutzerImportStrategy::postProcessAfterWrite(DatabaseContext *context, DataSet *dataSet, DataTuple *dataTuple, ImportResult *importResult) {
	if (dataTuple->getEntityDBName() != "Benutzer") {
		throw ImportException("ImportExport", "BenutzerImportStrategy", trUtf8("Technischer Fehler"),
							trUtf8("BenutzerImportStrategy falsch aufgerufen"));
	} else {
		AttributeData *attributeData = dataTuple->getAttributeData("benutzername");
		QString benutzerName = attributeData->getValue().toString();

		BenutzerSucheSearchModel *searchModel = new BenutzerSucheSearchModel();
		searchModel->setBenutzername(benutzerName);

		BenutzerController benutzerController = BenutzerController();
		std::vector<Benutzer*> benutzerVector = benutzerController.get(context, searchModel);

		if (benutzerVector.size() != 1) {
			DataHelper::deleteVectorContents(benutzerVector);
			throw ImportException("ImportExport", "BenutzerImportStrategy",
								  trUtf8("Nicht eindeutiger / fehlender Benutzername"), 
								  trUtf8("Bei Abschluss des Imports ist der Benutzer %1 nicht "
										 "oder mehr als einmal auf der Datenbank vorhanden.").arg(benutzerName));
		} else {
			Benutzer *benutzer = benutzerVector[0];
			QString name = (benutzer->hasBenutzername() ? benutzer->getBenutzername() : "<namenlos>");
			context->log("ImportExport", "BenutzerImportStrategy", 5, "postProcessAfterWrite: Aktualisiere Benutzer " + name);
			BenutzerHelper::aktualisiereDBBenutzer(context, benutzer, false);

			DataHelper::deleteVectorContents(benutzerVector);
		}
	}
}
