#include "ExternerJurorImportStrategy.h"

#include "AttributeData.h"
#include "DataImporter.h"
#include "ImportException.h"
#include "MitgliedsPruefer.h"

void ExternerJurorImportStrategy::checkAndConvertBeforeResolve(DatabaseContext *context, DataSet *dataSet,
														  DataTuple *dataTuple, ImportResult *importResult) {
	if (dataTuple->getEntityDBName() != "ExternerJuror") {
		throw ImportException("ImportExport", "ExternerJurorImportStrategy", trUtf8("Technischer Fehler"),
							trUtf8("ExternerJurorImportStrategy falsch aufgerufen"));
	} else {
		if (dataTuple->hasAttributeData("telefon")) {
			AttributeData *telefonAD = dataTuple->getAttributeData("telefon");
			if (telefonAD->isValue()) {
				QString telefon = telefonAD->getValue().toString();
				if (!MitgliedsPruefer::istTelefonnrSyntaxKorrekt(telefon)) {
					throw ImportException("ImportExport", "ExternerJurorImportStrategy", trUtf8("Telefonnummer falsch angegeben."),
										  trUtf8("Die Telefonnummer %1 wurde falsch angegeben. ").arg(telefon) + MitgliedsPruefer::getTelefonSyntaxInkorrektText());
				}
			}
		}

		if (dataTuple->hasAttributeData("mobil")) {
			AttributeData *mobilAD = dataTuple->getAttributeData("mobil");
			if (mobilAD->isValue()) {
				QString mobil = mobilAD->getValue().toString();
				if (!MitgliedsPruefer::istTelefonnrSyntaxKorrekt(mobil)) {
					throw ImportException("ImportExport", "ExternerJurorImportStrategy", trUtf8("Mobilfunknummer falsch angegeben."),
										  trUtf8("Die Mobilfunknummer %1 wurde falsch angegeben. ").arg(mobil) + MitgliedsPruefer::getMobilSyntaxInkorrektText());
				}
			}
		}
	}
}
