#include "VereinImportStrategy.h"

#include "AttributeData.h"
#include "DataImporter.h"
#include "ImportException.h"

void VereinImportStrategy::checkAndConvertBeforeResolve(DatabaseContext *context, DataSet *dataSet,
														  DataTuple *dataTuple, ImportResult *importResult) {
	if (dataTuple->getEntityDBName() != "Verein") {
		throw ImportException("ImportExport", "VereinImportStrategy", trUtf8("Technischer Fehler"),
							trUtf8("VereinImportStrategy falsch aufgerufen"));
	} else {
		testForMustValue(dataTuple, "vereinsNummerLokal");
		testForMustValue(dataTuple, "name");
		testForMustValue(dataTuple, "inaktiv");

		QString tupleAddress = QString::number((qlonglong)dataTuple, 16);
		context->log("ImportExport", "VereinImportStrategy", 5, "Calling VereinImportStrategy for tuple " + tupleAddress);

		// Die globale Vereinsnummer wird aus den restlichen Daten berechnet,
		// eine möglicherweise in den Importdaten vorhandene Nummer wird nicht
		// beachtet
		if (dataTuple->hasAttributeData("vereinsNummerLokal")
			&& dataTuple->hasAttributeData("dachverband.vereinsNummerKomplett")) {
			AttributeData *lokalnrAD = dataTuple->getAttributeData("vereinsNummerLokal");
			AttributeData *dachverbandsnrAD = dataTuple->getAttributeData("dachverband.vereinsNummerKomplett");
			if (lokalnrAD->isValue() && dachverbandsnrAD->isValue()) {
				QString lokalnr = lokalnrAD->getValue().toString();
				QString dachverbandsnr = dachverbandsnrAD->getValue().toString();
				QString vereinsnrKomplett = dachverbandsnr + lokalnr;
//				std::cout << "setting vereinsnrKomplett " << qPrintable(vereinsnrKomplett);
				dataTuple->setValue("vereinsNummerKomplett", vereinsnrKomplett);
				context->log("ImportExport", "VereinImportStrategy", 2, "vereinsNummerKomplett " + vereinsnrKomplett + " berechnet.");
			}
		}


		DataImporter::addTypeForCodeValueToTuple(dataTuple, "staat", "Staat");

		context->log("ImportExport", "VereinImportStrategy", 5, "Finished VereinImportStrategy for tuple " + tupleAddress);
	}
}
