#include "ErgebnisHelper.h"

#include <algorithm>

#include <math.h>

#include "BildController.h"
#include "BilderNachWettbewerbJoinController.h"
#include "BilderNachWettbewerbJoinModel.h"
#include "BilderNachWettbewerbJoinSearchModel.h"
#include "CodeManager.h"
#include "DataHelper.h"
#include "Exception.h"
#include "Sparte.h"
#include "SparteController.h"
#include "TeilnehmerErgebnisJoinController.h"
#include "TeilnehmerErgebnisJoinModel.h"
#include "TeilnehmerErgebnisJoinSearchModel.h"
#include "TeilnehmerWertungController.h"
#include "TeilnehmerKombinationsWertungController.h"
#include "VereinsWertungController.h"
#include "VereinsKombinationsWertungController.h"
#include "WettbewerbsHelper.h"

std::vector<TeilnehmerWertung*> ErgebnisHelper::berechneTeilnehmerWertungen(DatabaseContext *context,
																		    Wettbewerb *wettbewerb, qlonglong jugendFilter) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Berechne TeilnehmerWertungen mit jugendFilter " + QString::number(jugendFilter));

	int anzahlBilderFuerWertung = 0;
	if (wettbewerb->hasAnzahlBilderFuerWertung()) {
		anzahlBilderFuerWertung = wettbewerb->getAnzahlBilderFuerWertung();
	}
	context->log("Ergebnisse", "ErgebnisHelper", 4, "  Parameter Anzahl Bilder fuer Wertung ist " + QString::number(anzahlBilderFuerWertung));

	TeilnehmerErgebnisJoinSearchModel *searchModel = new TeilnehmerErgebnisJoinSearchModel();
	searchModel->setTeilnahmeWettbewerb(wettbewerb->getId());

	CodeManager *codeManager = context->getCodeManager();
	qlonglong jugendCodeId = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	qlonglong erwCodeId = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();

	// Datengrundlage holen
	TeilnehmerErgebnisJoinController joinController = TeilnehmerErgebnisJoinController();
	std::vector<TeilnehmerErgebnisJoinModel*> bildWertungen = std::vector<TeilnehmerErgebnisJoinModel*>();

	if (jugendFilter == jugendCodeId) {
		qlonglong nurJugendModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_JUGEND")->getId();
		searchModel->setSparteDurchgangsModus(nurJugendModus);
		std::vector<TeilnehmerErgebnisJoinModel*> nurJugendWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(nurJugendWertungen.size()) + " Ergebnistupel fuer NUR_JUGEND bei Jugend");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = nurJugendWertungen.begin(); it != nurJugendWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istJugend(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}

		qlonglong allesModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "ERWACHSENE_UND_JUGEND")->getId();
		searchModel->setSparteDurchgangsModus(allesModus);
		std::vector<TeilnehmerErgebnisJoinModel*> allesWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(allesWertungen.size()) + " Ergebnistupel fuer ERWACHSENE_UND_JUGEND bei Jugend");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = allesWertungen.begin(); it != allesWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istJugend(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}

		qlonglong nurErwachseneModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_ERWACHSENE")->getId();
		searchModel->setMitgliedJugend(nurErwachseneModus);
		std::vector<TeilnehmerErgebnisJoinModel*> jugendWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(jugendWertungen.size()) + " Ergebnistupel fuer NUR_ERWACHSENE bei Jugendlichen");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = jugendWertungen.begin(); it != jugendWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istJugend(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}
	}
	if (jugendFilter == erwCodeId) {
		qlonglong nurErwModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_ERWACHSENE")->getId();
		searchModel->setSparteDurchgangsModus(nurErwModus);
		std::vector<TeilnehmerErgebnisJoinModel*> nurErwWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(nurErwWertungen.size()) + " Ergebnistupel fuer NUR_ERWACHSENE bei Erwachsenen");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = nurErwWertungen.begin(); it != nurErwWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istErwachsener(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}

		qlonglong allesModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "ERWACHSENE_UND_JUGEND")->getId();
		searchModel->setSparteDurchgangsModus(allesModus);
		std::vector<TeilnehmerErgebnisJoinModel*> allesWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(allesWertungen.size()) + " Ergebnistupel fuer ERWACHSENE_UND_JUGEND bei Erwachsenen");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = allesWertungen.begin(); it != allesWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istErwachsener(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}

		qlonglong jugendAlsErwModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "JUGEND_ALS_ERWACHSENE")->getId();
		searchModel->setSparteDurchgangsModus(jugendAlsErwModus);
		std::vector<TeilnehmerErgebnisJoinModel*> erwWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(erwWertungen.size()) + " Ergebnistupel fuer JUGEND_ALS_ERW bei Erwachsenen");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = erwWertungen.begin(); it != erwWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			bildWertungen.push_back(joinModel);
		}

		qlonglong nurJugendModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_JUGEND")->getId();
		searchModel->setMitgliedJugend(nurJugendModus);
		std::vector<TeilnehmerErgebnisJoinModel*> jugendWertungen = joinController.get(context, searchModel);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "  TeilnehmerErgebnisJoin lieferte " + QString::number(jugendWertungen.size()) + " Ergebnistupel fuer NUR_JUGEND bei Erwachsenen");
		for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = jugendWertungen.begin(); it != jugendWertungen.end(); it++) {
			TeilnehmerErgebnisJoinModel *joinModel = *it;
			if (istErwachsener(context, wettbewerb, joinModel)) {
				bildWertungen.push_back(joinModel);
			} else {
				delete joinModel;
			}
		}
	}

	context->log("Ergebnisse", "ErgebnisHelper", 7, "  TeilnehmerErgebnisJoin lieferte " + QString::number(bildWertungen.size()) + " Ergebnistupel");

	// Erstmal nach Mitgliedern sortieren
	std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*> mitgliedsIdToBildWertungen
		= std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>();
	std::map<qlonglong, qlonglong> mitgliedsIdToVereinsId = std::map<qlonglong, qlonglong>();
	std::map<qlonglong, qlonglong> mitgliedsIdToTeilnahmeId = std::map<qlonglong, qlonglong>();
	std::set<qlonglong> mitgliedsIds = std::set<qlonglong>();
	for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = bildWertungen.begin();
																   it != bildWertungen.end(); it++) {
		TeilnehmerErgebnisJoinModel* ergebnis = *it;
		QString mitgliedsId = (ergebnis->hasMitgliedId() ? QString::number(ergebnis->getMitgliedId()) : "(ohne Mitglieds-ID)");
		QString teilnahmeId = (ergebnis->hasTeilnahmeId() ? QString::number(ergebnis->getTeilnahmeId()) : "(ohne Teilnahme-Id)");
		QString vereinsId = (ergebnis->hasVereinId() ? QString::number(ergebnis->getVereinId()) : "(ohne Vereins-ID)");
		QString vereinsName = (ergebnis->hasVereinName() ? ergebnis->getVereinName() : "(ohne Vereins-Name)");
		QString vorname = (ergebnis->hasMitgliedVorname() ? ergebnis->getMitgliedVorname() : "(ohne Vorname)");
		QString nachname = (ergebnis->hasMitgliedNachname() ? ergebnis->getMitgliedNachname() : "(ohne Nachname)");
		QString mitgliedsnr = (ergebnis->hasMitgliedMitgliedsNummerKomplett() ? ergebnis->getMitgliedMitgliedsNummerKomplett() : "(ohne Mitgliedsnr)");
		QString titel = (ergebnis->hasBildTitel() ? ergebnis->getBildTitel() : "(ohne Titel)");
		QString sparte = (ergebnis->hasSparteName() ? ergebnis->getSparteName() : "(ohne Sparten-Name)");
		QString spartenId = (ergebnis->hasSparteId() ? QString::number(ergebnis->getSparteId()) : "(ohne Sparten-Id)");
		QString bewertung = (ergebnis->hasBildBewertung() ? QString::number(ergebnis->getBildBewertung()) : "(ohne Bewertung)");
		context->log("Ergebnisse", "ErgebnisHelper", 6, "    Untersuche Bewertung fuer Mitglied " + vorname + " " + nachname + " mit Mitgliedsnr. " + mitgliedsnr
									                    + " , Mitglieds-ID " + mitgliedsId + " sowie Teilnahme-ID " + teilnahmeId);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "    ... Mitglied ist im Verein " + vereinsName + " mit der ID " + vereinsId);
		context->log("Ergebnisse", "ErgebnisHelper", 6, "    ... Bewertung fuer das Bild " + titel + " in Sparte " + sparte + " mit id " + spartenId + " ist " + bewertung);

		if (ergebnis->hasTeilnahmeMitglied() && ergebnis->hasMitgliedVerein() && ergebnis->hasTeilnahmeId()) {
			qlonglong mitgliedsId = ergebnis->getTeilnahmeMitglied();
			context->log("Ergebnisse", "ErgebnisHelper", 1, "      Ordne es dem Mitglied zu.");
			mitgliedsIdToBildWertungen.insert(std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>
												::value_type(mitgliedsId, ergebnis));
			mitgliedsIds.insert(mitgliedsId);
			mitgliedsIdToVereinsId[mitgliedsId] = ergebnis->getMitgliedVerein();
			mitgliedsIdToTeilnahmeId[mitgliedsId] = ergebnis->getTeilnahmeId();
		} else {
			QString titel = QObject::trUtf8("Interner Fehler");
			QString text = QObject::trUtf8("Konnte die Bildbewertung fuer das Bild %1 nicht dem Mitglied %2 %3 zuordnen.").arg(titel).arg(vorname).arg(nachname);
			throw Exception("Ergebnisse", "ErgebnisHelper", titel, text);
		}
	}

	context->log("Ergebnisse", "ErgebnisHelper", 7, "  Sortiere die Bilder pro Mitglied nach Sparten...");
	std::map<qlonglong, Sparte*> sparteIdToSparte = std::map<qlonglong, Sparte*>();
	SparteController sparteController = SparteController();

	// Dann fuer jedes Mitglied einzeln die Bilder nach Sparten sortieren,
	std::vector<TeilnehmerWertung*> retVector = std::vector<TeilnehmerWertung*>();
	for (std::set<qlonglong>::const_iterator it = mitgliedsIds.begin(); it != mitgliedsIds.end(); it++) {
		qlonglong mitgliedsId = *it;
		context->log("Ergebnisse", "ErgebnisHelper", 4, "    Untersuche Mitglied mit id " + QString::number(mitgliedsId));

		// Bilder nach Sparten sortieren
		std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*> sparteIdToBildWertungen
			= std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>();
		// und die Sparten-Ids nochmal separat ablegen
		std::set<qlonglong> spartenIds = std::set<qlonglong>();
		for (std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>::const_iterator it2
			= mitgliedsIdToBildWertungen.find(mitgliedsId);
				it2 != mitgliedsIdToBildWertungen.end() && it2 != mitgliedsIdToBildWertungen.upper_bound(mitgliedsId);
				it2++) {
			TeilnehmerErgebnisJoinModel* ergebnis = it2->second;
			if (ergebnis->hasSparteId()) {
				qlonglong sparteId = ergebnis->getSparteId();
				sparteIdToBildWertungen.insert(std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>
											  ::value_type(sparteId, ergebnis));
				spartenIds.insert(sparteId);

				if (sparteIdToSparte.find(sparteId) == sparteIdToSparte.end()) {
					Sparte *sparte = sparteController.getByPrimaryKey(context, sparteId);
					sparteIdToSparte[sparteId] = sparte;
				}
			}
		}

		// Über alle Sparten iterieren
		qlonglong jugendAlsErwModus = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "JUGEND_ALS_ERWACHSENE")->getId();
		for (std::set<qlonglong>::const_iterator it2 = spartenIds.begin(); it2 != spartenIds.end(); it2++) {
			qlonglong spartenId = *it2;
			context->log("Ergebnisse", "ErgebnisHelper", 4, "      Untersuche Sparte " + QString::number(spartenId));

			// Die Bewertungen der aktuellen Sparte in einen vector verfrachten
			std::vector<int> bewertungen = std::vector<int>();
			for (std::multimap<qlonglong, TeilnehmerErgebnisJoinModel*>::const_iterator it3
				= sparteIdToBildWertungen.find(spartenId);
				  it3 != sparteIdToBildWertungen.end() && it3 != sparteIdToBildWertungen.upper_bound(spartenId); it3++) {
				TeilnehmerErgebnisJoinModel* ergebnis = it3->second;
				if (ergebnis->hasBildBewertung()) {
					bewertungen.push_back(ergebnis->getBildBewertung());
				}
			}

			// Sortieren
			std::sort(bewertungen.begin(), bewertungen.end(), std::greater<int>());


			int punkteSumme = 0;
			for (int n = 0; n < (int)bewertungen.size() && n < anzahlBilderFuerWertung; n++) {
				punkteSumme += bewertungen[n];
			}

			context->log("Ergebnisse", "ErgebnisHelper", 5, "      Lege neue TeilnehmerWertung fuer Mitglied "
									  + QString::number(mitgliedsId) + ", Sparte "
									  + QString::number(spartenId) + " mit Punktesumme "
									  + QString::number(punkteSumme) + " an.");

			TeilnehmerWertung *teilnehmerWertung = new TeilnehmerWertung();
			teilnehmerWertung->setWettbewerb(wettbewerb->getId());
			teilnehmerWertung->setWettbewerbsTeilnahme(mitgliedsIdToTeilnahmeId[mitgliedsId]);
			teilnehmerWertung->setSparte(spartenId);
			teilnehmerWertung->setVerein(mitgliedsIdToVereinsId[mitgliedsId]);
			teilnehmerWertung->setPunkte(punkteSumme);

			if (sparteIdToSparte.find(spartenId) != sparteIdToSparte.end()) {
				Sparte *sparte = sparteIdToSparte[spartenId];
				if (sparte->hasDurchgangsModus() && sparte->getDurchgangsModus() == jugendAlsErwModus) {
					teilnehmerWertung->setJugend(erwCodeId);
				} else {
					teilnehmerWertung->setJugend(jugendFilter);
				}
			} else {
				context->log("Ergebnisse", "ErgebnisHelper", 9, "      Sparten-Instanz " + QString::number(spartenId) + " nicht gefunden.");
			}

			retVector.push_back(teilnehmerWertung);
		}
	}

	for (std::vector<TeilnehmerErgebnisJoinModel*>::const_iterator it = bildWertungen.begin();
																   it != bildWertungen.end(); it++) {
		delete *it;
	}

	context->log("Ergebnisse", "ErgebnisHelper", 8, QString::number(retVector.size()) + " TeilnehmerWertungen mit jugendFilter " + QString::number(jugendFilter) + " berechnet.");

	return retVector;
}

// TODO: Speicherverwaltung / delete

bool ErgebnisHelper::sindTeilnehmerWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb) {
	TeilnehmerWertungController controller = TeilnehmerWertungController();
	std::vector<TeilnehmerWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	return wertungen.size() > 0;
}

void ErgebnisHelper::schreibeTeilnehmerWertungen(DatabaseContext *context, std::vector<TeilnehmerWertung*> wertungen) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Schreibe " + QString::number(wertungen.size()) + " Teilnehmer-Wertungen auf die Datenbank...");
	TeilnehmerWertungController controller = TeilnehmerWertungController();
	for (std::vector<TeilnehmerWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		TeilnehmerWertung *wertung = *it;
		controller.insertOrUpdate(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "... Schreiben von " + QString::number(wertungen.size()) + " Teilnehmer-Wertungen beendet.");
}

int ErgebnisHelper::loescheTeilnehmerWertungen(DatabaseContext *context, Wettbewerb *wettbewerb) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loesche alle Teilnehmer-Wertungen...");
	TeilnehmerWertungController controller = TeilnehmerWertungController();
	std::vector<TeilnehmerWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	for (std::vector<TeilnehmerWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		TeilnehmerWertung *wertung = *it;
		controller.remove(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loeschen von " + QString::number(wertungen.size()) + " Teilnehmer-Wertungen beendet.");
	return wertungen.size();
}


std::vector<TeilnehmerKombinationsWertung*> ErgebnisHelper::berechneTeilnehmerKombinationsWertungen(
		DatabaseContext *context, Wettbewerb *wettbewerb, std::vector<TeilnehmerWertung*> teilnehmerWertungen, qlonglong jugendFilter) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Berechne TeilnehmerKombinationsWertungen mit jugendFilter " + QString::number(jugendFilter)
					  + " auf Basis von " + QString::number(teilnehmerWertungen.size()) + " TeilnehmerWertungen");

	int anzahlSpartenFuerKombination = 0;
	if (wettbewerb->hasAnzahlSpartenFuerKombination()) {
		anzahlSpartenFuerKombination = wettbewerb->getAnzahlSpartenFuerKombination();
	}
	context->log("Ergebnisse", "ErgebnisHelper", 4, "  Parameter Anzahl Sparten fuer Kombination ist " + QString::number(anzahlSpartenFuerKombination));

	std::vector<TeilnehmerKombinationsWertung*> retVector = std::vector<TeilnehmerKombinationsWertung*>();

	SparteController sparteController = SparteController();
	std::vector<Sparte*> sparten = sparteController.getByWettbewerb(context, wettbewerb);
	std::map<qlonglong, Sparte*> sparteIdToSparte = std::map<qlonglong, Sparte*>();
	for (std::vector<Sparte*>::const_iterator it = sparten.begin(); it != sparten.end(); it++) {
		Sparte *sparte = *it;
		QString spartenId = (sparte->hasId() ? QString::number(sparte->getId()) : "(ohne Sparten-Id)");
		QString spartenName = (sparte->hasName() ? sparte->getName() : "(ohne Sparten-Name)");
		QString inKombinationVerwendet = (sparte->hasInKombinationVerwendet() ? (sparte->getInKombinationVerwendet() ? "" : "nicht") : "(keine Info)");
		context->log("Ergebnisse", "ErgebnisHelper", 2, "  Sparte " + spartenName + " mit ID " + spartenId 
					  + " registriert.  Wird in Kombination " + inKombinationVerwendet + " verwendet.");
		sparteIdToSparte[sparte->getId()] = sparte;
	}

	// Erstmal nach Mitgliedern sortieren, Sparten die nicht fuer die Kombination zählen gleich aussortieren
	std::multimap<qlonglong, TeilnehmerWertung*> wtToWertungen = std::multimap<qlonglong, TeilnehmerWertung*>();
	std::set<qlonglong> wtIds = std::set<qlonglong>();
	for (std::vector<TeilnehmerWertung*>::const_iterator it = teilnehmerWertungen.begin();
					  it != teilnehmerWertungen.end(); it++) {
		TeilnehmerWertung *wertung = *it;
		if (wertung->hasSparte()) {
			Sparte *sparte = sparteIdToSparte[wertung->getSparte()];
			// wir ignorieren alle Wertungen in Sparten, bei denen das inKombinationVerwendet-Flag
			// nicht gesetzt ist
			if (sparte->hasInKombinationVerwendet() && sparte->getInKombinationVerwendet()) {
				qlonglong wtId = wertung->getWettbewerbsTeilnahme();
				wtToWertungen.insert(std::multimap<qlonglong, TeilnehmerWertung*>::value_type(wtId, wertung));
				wtIds.insert(wtId);
			}
		}
	}

	// über alle Mitglieder iterieren...
	for (std::set<qlonglong>::const_iterator it = wtIds.begin(); it != wtIds.end(); it++) {
		std::set<qlonglong> spartenIdsDiesesMitglieds = std::set<qlonglong>();

		qlonglong wtId = *it;
		QVariant vereinsId = QVariant();

		context->log("Ergebnisse", "ErgebnisHelper", 4, "    Inspiziere WettbewerbsTeilnahme-Id " + QString::number(wtId));

		std::vector<int> bewertungen = std::vector<int>();

		for (std::multimap<qlonglong, TeilnehmerWertung*>::const_iterator it2 = wtToWertungen.find(wtId);
				it2 != wtToWertungen.end() && it2 != wtToWertungen.upper_bound(wtId); it2++) {
			// jeweils die Gesamtpunkte usw. ausrechnen
			TeilnehmerWertung *wertung = it2->second;
			if (wertung->hasPunkte()) {
				bewertungen.push_back(wertung->getPunkte());
				context->log("Ergebnisse", "ErgebnisHelper", 1, "      Registriere Punktesumme " + QString::number(wertung->getPunkte()));
			}
			vereinsId = (wertung->hasVerein() ? wertung->getVerein() : QVariant());
			spartenIdsDiesesMitglieds.insert(wertung->getSparte());
		}

		context->log("Ergebnisse", "ErgebnisHelper", 4, "    Es liegt in " + QString::number(spartenIdsDiesesMitglieds.size())
													  + " Sparten des Mitglieds eine Einreichung vor.");

		// Prüfen, ob in genügend Sparten eine Einreichung vorliegt, oder ob kraft Wettbewerbseinstellungen die Anzahl der Sparten in
		// denen eingereicht wurde egal ist hierfür.
		// Die Filterung auf Sparten mit inKombinationVerwendet == true passiert schon weiter oben,
		// daher kommen hier nur relevante Sparten an.
		if (((wettbewerb->hasNurVollstaendigeKombination() && !wettbewerb->getNurVollstaendigeKombination())
		   || (int)spartenIdsDiesesMitglieds.size() >= anzahlSpartenFuerKombination) && vereinsId.isValid()) {
			// Sortieren
			std::sort(bewertungen.begin(), bewertungen.end(), std::greater<int>());

			// Kombinations-Bewertung ermitteln
			int punkteSumme = 0;
			for (int n = 0; n < (int)bewertungen.size() && n < anzahlSpartenFuerKombination; n++) {
				punkteSumme += bewertungen[n];
			}

			context->log("Ergebnisse", "ErgebnisHelper", 4, "    Punktesumme ist " + QString::number(punkteSumme));

			TeilnehmerKombinationsWertung *kombinationsWertung = new TeilnehmerKombinationsWertung();
			kombinationsWertung->setWettbewerb(wettbewerb->getId());
			kombinationsWertung->setWettbewerbsTeilnahme(wtId);
			kombinationsWertung->setVerein(vereinsId.toLongLong());
			kombinationsWertung->setPunkte(punkteSumme);
			kombinationsWertung->setJugend(jugendFilter);
			retVector.push_back(kombinationsWertung);
		} else {
			context->log("Ergebnisse", "ErgebnisHelper", 4, "    Es liegt in nicht genug Sparten eine Einreichung vor.");
		}
	}

	DataHelper::deleteVectorContents(sparten);
	context->log("Ergebnisse", "ErgebnisHelper", 8, QString::number(retVector.size()) + " TeilnehmerKombinationsWertungen mit jugendFilter " + QString::number(jugendFilter) + " berechnet.");
	return retVector;
}

bool ErgebnisHelper::sindTeilnehmerKombinationsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb) {
	TeilnehmerKombinationsWertungController controller = TeilnehmerKombinationsWertungController();
	std::vector<TeilnehmerKombinationsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	return wertungen.size() > 0;
}

void ErgebnisHelper::schreibeTeilnehmerKombinationsWertungen(DatabaseContext *context, 
			std::vector<TeilnehmerKombinationsWertung*> wertungen) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Schreibe " + QString::number(wertungen.size()) + " TeilnehmerKombinationsWertungen auf die Datenbank...");
	TeilnehmerKombinationsWertungController controller = TeilnehmerKombinationsWertungController();
	for (std::vector<TeilnehmerKombinationsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		TeilnehmerKombinationsWertung *wertung = *it;
		controller.insertOrUpdate(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "... Schreiben von " + QString::number(wertungen.size()) + " TeilnehmerKombinationsWertungen beendet.");
}

int ErgebnisHelper::loescheTeilnehmerKombinationsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loesche alle TeilnehmerKombinationsWertungen...");
	TeilnehmerKombinationsWertungController controller = TeilnehmerKombinationsWertungController();
	std::vector<TeilnehmerKombinationsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	for (std::vector<TeilnehmerKombinationsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		TeilnehmerKombinationsWertung *wertung = *it;
		controller.remove(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loeschen von " + QString::number(wertungen.size()) + " TeilnehmerKombinationsWertungen beendet.");
	return wertungen.size();
}


std::vector<VereinsWertung*> ErgebnisHelper::berechneVereinsWertungen(DatabaseContext *context,
				Wettbewerb *wettbewerb, std::vector<TeilnehmerWertung*> teilnehmerWertungen, qlonglong jugendFilter) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Berechne VereinsWertungen mit jugendFilter " + QString::number(jugendFilter));

	int anzahlTeilnehmerFuerVereinsWertung = 0;
	if (wettbewerb->hasAnzahlTeilnehmerFuerVereinsW()) {
		anzahlTeilnehmerFuerVereinsWertung = wettbewerb->getAnzahlTeilnehmerFuerVereinsW();
	}
	context->log("Ergebnisse", "ErgebnisHelper", 4, "  Parameter Anzahl Teilnehmer fuer Vereinswertung ist " + QString::number(anzahlTeilnehmerFuerVereinsWertung));

	std::vector<VereinsWertung*> retVector = std::vector<VereinsWertung*>();

	SparteController sparteController = SparteController();
	std::vector<Sparte*> sparten = sparteController.getByWettbewerb(context, wettbewerb);

	// Erstmal nach Vereinen sortieren
	std::multimap<qlonglong, TeilnehmerWertung*> vereinToWertungen = std::multimap<qlonglong, TeilnehmerWertung*>();
	std::set<qlonglong> vereinsIds = std::set<qlonglong>();
	for (std::vector<TeilnehmerWertung*>::const_iterator it = teilnehmerWertungen.begin();
					  it != teilnehmerWertungen.end(); it++) {
		TeilnehmerWertung *wertung = *it;
		qlonglong vereinsId = wertung->getVerein();
		vereinToWertungen.insert(std::multimap<qlonglong, TeilnehmerWertung*>::value_type(vereinsId, wertung));
		vereinsIds.insert(vereinsId);
	}

	// über alle Vereine iterieren
	for (std::set<qlonglong>::const_iterator it = vereinsIds.begin(); it != vereinsIds.end(); it++) {
		qlonglong vereinsId = *it;

		context->log("Ergebnisse", "ErgebnisHelper", 4, "  Untersuche Verein mit ID " + QString::number(vereinsId));

		std::multimap<qlonglong, TeilnehmerWertung*> sparteIdToWertungen = std::multimap<qlonglong, TeilnehmerWertung*>();

		// Für den aktuellen Verein nach Sparten kategorisieren
		for (std::multimap<qlonglong, TeilnehmerWertung*>::const_iterator it2 = vereinToWertungen.find(vereinsId);
				it2 != vereinToWertungen.end() && it2 != vereinToWertungen.upper_bound(vereinsId); it2++) {
			TeilnehmerWertung *wertung = it2->second;
			qlonglong sparteId = wertung->getSparte();
			sparteIdToWertungen.insert(std::multimap<qlonglong, TeilnehmerWertung*>::value_type(sparteId, wertung));
		}

		// über die Sparten iterieren
		for (std::vector<Sparte*>::const_iterator it2 = sparten.begin(); it2 != sparten.end(); it2++) {
			Sparte *sparte = *it2;
			qlonglong sparteId = sparte->getId();
			context->log("Ergebnisse", "ErgebnisHelper", 4, "    Untersuche Sparte mit ID " + sparteId);

			std::vector<int> bewertungen = std::vector<int>();
			// Für die aktuelle Sparte die Bewertungen extrahieren
			for (std::multimap<qlonglong, TeilnehmerWertung*>::const_iterator it3 = sparteIdToWertungen.find(sparteId);
				it3 != sparteIdToWertungen.end() && it3 != sparteIdToWertungen.upper_bound(sparteId); it3++) {
				TeilnehmerWertung *wertung = it3->second;
				bewertungen.push_back(wertung->getPunkte());
			}

			context->log("Ergebnisse", "ErgebnisHelper", 4, "    " + QString::number(bewertungen.size()) + " Teilnehmer-Wertungen extrahiert.");

			if (bewertungen.size() > 0) {
				std::sort(bewertungen.begin(), bewertungen.end(), std::greater<int>());
				int vereinsPunkte = 0;
				for (int n = 0; n < (int)bewertungen.size() && n < anzahlTeilnehmerFuerVereinsWertung; n++) {
					vereinsPunkte += bewertungen[n];
				}

				context->log("Ergebnisse", "ErgebnisHelper", 4, "    " + QString::number(vereinsPunkte) + " Punkte fuer Verein mit der ID "
								+ QString::number(vereinsId) + " in Sparte " + QString::number(sparteId) + " registriert.");

				VereinsWertung *vereinsWertung = new VereinsWertung();
				vereinsWertung->setWettbewerb(wettbewerb->getId());
				vereinsWertung->setVerein(vereinsId);
				vereinsWertung->setSparte(sparteId);
				vereinsWertung->setPunkte(vereinsPunkte);
				vereinsWertung->setJugend(jugendFilter);
				retVector.push_back(vereinsWertung);
			}
		}
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, QString::number(retVector.size()) + " VereinsWertungen mit jugendFilter " + QString::number(jugendFilter) + " berechnet.");
	return retVector;
}

bool ErgebnisHelper::sindVereinsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb) {
	VereinsWertungController controller = VereinsWertungController();
	std::vector<VereinsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	return wertungen.size() > 0;
}

void ErgebnisHelper::schreibeVereinsWertungen(DatabaseContext *context, std::vector<VereinsWertung*> wertungen) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Schreibe " + QString::number(wertungen.size()) + " Vereins-Wertungen auf die Datenbank...");
	VereinsWertungController controller = VereinsWertungController();
	for (std::vector<VereinsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		VereinsWertung *wertung = *it;
		controller.insertOrUpdate(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "... Schreiben von " + QString::number(wertungen.size()) + " Vereins-Wertungen beendet.");
}

int ErgebnisHelper::loescheVereinsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loesche alle Vereins-Wertungen...");
	VereinsWertungController controller = VereinsWertungController();
	std::vector<VereinsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	for (std::vector<VereinsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		VereinsWertung *wertung = *it;
		controller.remove(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loeschen von " + QString::number(wertungen.size()) + " Vereins-Wertungen beendet.");
	return wertungen.size();
}

std::vector<VereinsKombinationsWertung*> ErgebnisHelper::berechneVereinsKombinationsWertungen(DatabaseContext *context,
			Wettbewerb *wettbewerb, std::vector<VereinsWertung*> vereinsWertungen, qlonglong jugendFilter) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Berechne VereinsKombinationsWertungen mit jugendFilter " + QString::number(jugendFilter));

	std::vector<VereinsKombinationsWertung*> retVector = std::vector<VereinsKombinationsWertung*>();

	SparteController sparteController = SparteController();
	std::vector<Sparte*> sparten = sparteController.getByWettbewerb(context, wettbewerb);
	std::map<qlonglong, Sparte*> sparteIdToSparte = std::map<qlonglong, Sparte*>();
	for (std::vector<Sparte*>::const_iterator it = sparten.begin(); it != sparten.end(); it++) {
		Sparte *sparte = *it;
		sparteIdToSparte[sparte->getId()] = sparte;
	}

	// Erstmal nach Vereinen sortieren
	std::multimap<qlonglong, VereinsWertung*> vereinToWertungen = std::multimap<qlonglong, VereinsWertung*>();
	std::set<qlonglong> vereinsIds = std::set<qlonglong>();
	for (std::vector<VereinsWertung*>::const_iterator it = vereinsWertungen.begin(); it != vereinsWertungen.end(); it++) {
		VereinsWertung *wertung = *it;
		qlonglong vereinsId = wertung->getVerein();
		vereinToWertungen.insert(std::multimap<qlonglong, VereinsWertung*>::value_type(vereinsId, wertung));
		vereinsIds.insert(vereinsId);
	}

	// über alle Vereine iterieren
	for (std::set<qlonglong>::const_iterator it = vereinsIds.begin(); it != vereinsIds.end(); it++) {
		qlonglong vereinsId = *it;

		context->log("Ergebnisse", "ErgebnisHelper", 4, "  Untersuche Verein mit ID " + QString::number(vereinsId));

		int bewertung = 0;
		int anzahlBewertungen = 0;

		for (std::multimap<qlonglong, VereinsWertung*>::const_iterator it2 = vereinToWertungen.find(vereinsId);
				it2 != vereinToWertungen.end() && it2 != vereinToWertungen.upper_bound(vereinsId); it2++) {
			VereinsWertung *wertung = it2->second;

			if (wertung->hasSparte()) {
				Sparte *sparte = sparteIdToSparte[wertung->getSparte()];
				// wir ignorieren alle Wertungen in Sparten, bei denen das inKombinationVerwendet-Flag
				// nicht gesetzt ist
				if (sparte->hasInKombinationVerwendet() && sparte->getInKombinationVerwendet()) {
					bewertung += (wertung->hasPunkte() ? wertung->getPunkte() : 0);
					anzahlBewertungen++;
				}
			}
		}

		context->log("Ergebnisse", "ErgebnisHelper", 6, "  " + QString::number(bewertung) + " Punkte fuer Verein mit ID " + QString::number(vereinsId) + " registriert.");

		if (anzahlBewertungen > 0) {
			VereinsKombinationsWertung *vereinsKombinationsWertung = new VereinsKombinationsWertung();
			vereinsKombinationsWertung->setWettbewerb(wettbewerb->getId());
			vereinsKombinationsWertung->setVerein(vereinsId);
			vereinsKombinationsWertung->setPunkte(bewertung);
			vereinsKombinationsWertung->setJugend(jugendFilter);
			retVector.push_back(vereinsKombinationsWertung);
		}
	}
	DataHelper::deleteVectorContents(sparten);
	context->log("Ergebnisse", "ErgebnisHelper", 8, QString::number(retVector.size()) + " VereinsKombinationsWertungen mit jugendFilter " + QString::number(jugendFilter) + " berechnet.");
	return retVector;
}

bool ErgebnisHelper::sindVereinsKombinationsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb) {
	VereinsKombinationsWertungController controller = VereinsKombinationsWertungController();
	std::vector<VereinsKombinationsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	return wertungen.size() > 0;
}

void ErgebnisHelper::schreibeVereinsKombinationsWertungen(DatabaseContext *context,
					std::vector<VereinsKombinationsWertung*> wertungen) {

	context->log("Ergebnisse", "ErgebnisHelper", 8, "Schreibe " + QString::number(wertungen.size()) + " VereinsKombinationsWertungen auf die Datenbank...");
	VereinsKombinationsWertungController controller = VereinsKombinationsWertungController();
	for (std::vector<VereinsKombinationsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		VereinsKombinationsWertung *wertung = *it;
		controller.insertOrUpdate(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "... Schreiben von " + QString::number(wertungen.size()) + " VereinsKombinationsWertungen beendet.");
}

int ErgebnisHelper::loescheVereinsKombinationsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb) {
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loesche alle VereinsKombinationsWertungen...");
	VereinsKombinationsWertungController controller = VereinsKombinationsWertungController();
	std::vector<VereinsKombinationsWertung*> wertungen = controller.getByWettbewerb(context, wettbewerb);
	for (std::vector<VereinsKombinationsWertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		VereinsKombinationsWertung *wertung = *it;
		controller.remove(context, wertung);
	}
	context->log("Ergebnisse", "ErgebnisHelper", 8, "Loeschen von " + QString::number(wertungen.size()) + " VereinsKombinationsWertungen beendet.");
	return wertungen.size();
}

bool ErgebnisHelper::istJugend(DatabaseContext *context, Wettbewerb *wettbewerb, TeilnehmerErgebnisJoinModel *joinModel) {
	TeilnehmerErgebnisJoinController controller = TeilnehmerErgebnisJoinController();
	Mitglied *mitglied = controller.extractMitglied(joinModel);
	bool istJugend = WettbewerbsHelper::istMitgliedJugend(context, wettbewerb, mitglied);
	delete mitglied;
	return istJugend;
}

bool ErgebnisHelper::istErwachsener(DatabaseContext *context, Wettbewerb *wettbewerb, TeilnehmerErgebnisJoinModel *joinModel) {
	return !istJugend(context, wettbewerb, joinModel);
}

void ErgebnisHelper::berechneAnnahmen(DatabaseContext *context, Wettbewerb *wettbewerb) {
	BilderNachWettbewerbJoinController joinController = BilderNachWettbewerbJoinController();
	BilderNachWettbewerbJoinSearchModel *searchModel = new BilderNachWettbewerbJoinSearchModel();
	searchModel->setWettbewerbId(wettbewerb->getId());
	std::vector<BilderNachWettbewerbJoinModel*> joinModels = joinController.get(context, searchModel);
	delete searchModel;

	CodeManager *codeManager = context->getCodeManager();
	qlonglong jugendCodeId = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	qlonglong erwCodeId = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();

	SparteController sparteController = SparteController();
	std::vector<Sparte*> sparten = sparteController.getByWettbewerb(context, wettbewerb);
	for (std::vector<Sparte*>::const_iterator it = sparten.begin(); it != sparten.end(); it++) {
		Sparte *sparte = *it;

		berechneAnnahmen(context, joinModels, wettbewerb, sparte, jugendCodeId);
		berechneAnnahmen(context, joinModels, wettbewerb, sparte, erwCodeId);
	}

	DataHelper::deleteVectorContents(joinModels);
	DataHelper::deleteVectorContents(sparten);
}

void ErgebnisHelper::berechneAnnahmen(DatabaseContext *context, std::vector<BilderNachWettbewerbJoinModel*> joinModels, Wettbewerb *wettbewerb, Sparte *sparte, qlonglong jugendCodeId) {
	CodeManager *codeManager = context->getCodeManager();
	qlonglong abrundenCodeId = codeManager->getCodeValueByTypeAndName("AbrundungPunkteBeiAnnahmen", "JA")->getId();

	int anteilAnnahmenInt = (wettbewerb->hasAnteilAnnahmenGewuenscht() ? wettbewerb->getAnteilAnnahmenGewuenscht() : 100);
	double anteilAnnahmen = (double)anteilAnnahmenInt / 100;

	bool abrunden;
	if (wettbewerb->hasAbrundungPunkteBeiAnnahmen()) {
		abrunden = (wettbewerb->getAbrundungPunkteBeiAnnahmen() == abrundenCodeId);
	}

	int maxPunkte = 0;

	for (std::vector<BilderNachWettbewerbJoinModel*>::const_iterator it = joinModels.begin(); it != joinModels.end(); it++) {
		BilderNachWettbewerbJoinModel *joinModel = *it;
		if (joinModel->hasBildBewertung() && joinModel->hasBildSparte() && joinModel->hasMitgliedJugend()) {
			if (joinModel->getBildSparte() == sparte->getId() && joinModel->getMitgliedJugend() == jugendCodeId) {
				int currPunkte = joinModel->getBildBewertung();
				if (currPunkte > maxPunkte) {
					maxPunkte = currPunkte;
				}
			}
		}
	}

	double annahmeGrenzeDouble = (1 - anteilAnnahmen) * (double)maxPunkte;
	// floor und ceil geben double zurueck...
	double annahmeGrenzeRoundedDouble = (abrunden ? floor(annahmeGrenzeDouble) : ceil(annahmeGrenzeDouble));
	// ... daher sicherheitshalber nochmal auf den nächsten Integer runden, um Rundungsproblemen bei einem etwaigen Cast vorzubeugen
	int annahmeGrenze = lrint(annahmeGrenzeRoundedDouble);

	// Speichere Annahme-Grenze in Sparte, um sie später im WettbewerbStatistikWidget auswerten zu können
	qlonglong jugend = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	qlonglong erwachsene = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();
	if (jugendCodeId == jugend) {
		sparte->setAnnahmeGrenzeJugend(annahmeGrenze);
	} else if (jugendCodeId == erwachsene) {
		sparte->setAnnahmeGrenzeErwachsene(annahmeGrenze);
	}
	SparteController sparteController = SparteController();
	sparteController.insertOrUpdate(context, sparte);

	BildController controller = BildController();
	for (std::vector<BilderNachWettbewerbJoinModel*>::const_iterator it = joinModels.begin(); it != joinModels.end(); it++) {
		BilderNachWettbewerbJoinModel *joinModel = *it;
		if (joinModel->hasBildBewertung() && joinModel->hasBildSparte() && joinModel->hasMitgliedJugend()) {
			if (joinModel->getBildSparte() == sparte->getId() && joinModel->getMitgliedJugend() == jugendCodeId) {
				Bild *bild = BilderNachWettbewerbJoinController::extractBild(joinModel);

				int punkte = joinModel->getBildBewertung();
				if (punkte >= annahmeGrenze) {
					bild->setBewertungString(QObject::trUtf8("%1*").arg(QString::number(punkte)));
					bild->setAngenommen(true);
				} else {
					bild->setBewertungString(QObject::trUtf8("%1").arg(QString::number(punkte)));
					bild->setAngenommen(false);
				}

				controller.insertOrUpdate(context, bild);
			}
		}
	}
}
