#ifndef ERGEBNISHELPER_H
#define ERGEBNISHELPER_H

#include "BilderNachWettbewerbJoinModel.h"
#include "DatabaseContext.h"
#include "Durchgang.h"
#include "EinzelwertungsAnzeigeJoinModel.h"
#include "JuriertBeiPersonJoinModel.h"
#include "Sparte.h"
#include "TeilnehmerErgebnisJoinModel.h"
#include "TeilnehmerWertung.h"
#include "TeilnehmerKombinationsWertung.h"
#include "VereinsWertung.h"
#include "VereinsKombinationsWertung.h"
#include "Wertung.h"
#include "Wettbewerb.h"
#include "WettbewerbCache.h"

#include <vector>

class ErgebnisHelper {

public:
	/**  Berechnet die TeilnehmerWertungen.  Diese werden auf
	 *   Basis der in den Bild-Instanzen gespeicherten Punktewerte berechnet.
	 *   Schreibt die berechneten TeilnehmerWertungen aber nicht auf die Datenbank, 
	 *   sondern gibt sie lediglich als Vektor zurück.
	 */
	static std::vector<TeilnehmerWertung*> berechneTeilnehmerWertungen(DatabaseContext *context, Wettbewerb *wettbewerb, qlonglong jugendFilter);

	/**  Gibt zurück, ob sich Teilnehmer-Wertungen für den gegebenen 
	 *   Wettbewerb auf der Datenbank befinden.
	 */
	static bool sindTeilnehmerWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb);

	/**  Schreibt die gegebenen TeilnehmerWertungen auf die Datenbank.  Führt
	 *   keine Duplikatsprüfungen durch.
	 */
	static void schreibeTeilnehmerWertungen(DatabaseContext *context, std::vector<TeilnehmerWertung*> wertungen);

	/**  Löscht alle Teilnehmer-Wertungen des gegebenen Wettbewerbs, und gibt
	 *   die Anzahl der gelöschten Objekte zurück.
	 */
	static int loescheTeilnehmerWertungen(DatabaseContext *context, Wettbewerb *wettbewerb);

	static std::vector<TeilnehmerKombinationsWertung*> berechneTeilnehmerKombinationsWertungen(DatabaseContext *context, 
				Wettbewerb *wettbewerb, std::vector<TeilnehmerWertung*> teilnehmerWertungen, qlonglong jugendFilter);
	static bool sindTeilnehmerKombinationsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb);
	static void schreibeTeilnehmerKombinationsWertungen(DatabaseContext *context, 
				std::vector<TeilnehmerKombinationsWertung*> wertungen);
	static int loescheTeilnehmerKombinationsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb);

	static std::vector<VereinsWertung*> berechneVereinsWertungen(DatabaseContext *context, 
				Wettbewerb *wettbewerb, std::vector<TeilnehmerWertung*> teilnehmerWertungen, qlonglong jugendFilter);
	static bool sindVereinsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb);
	static void schreibeVereinsWertungen(DatabaseContext *context, std::vector<VereinsWertung*> wertungen);
	static int loescheVereinsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb);

	static std::vector<VereinsKombinationsWertung*> berechneVereinsKombinationsWertungen(DatabaseContext *context, 
				Wettbewerb *wettbewerb, std::vector<VereinsWertung*> vereinsWertungen, qlonglong jugendFilter);
	static bool sindVereinsKombinationsWertungenAufDerDatenbank(DatabaseContext *context, Wettbewerb *wettbewerb);
	static void schreibeVereinsKombinationsWertungen(DatabaseContext *context, 
						std::vector<VereinsKombinationsWertung*> wertungen);
	static int loescheVereinsKombinationsWertungen(DatabaseContext *context, Wettbewerb *wettbewerb);

	static void berechneAnnahmen(DatabaseContext *context, Wettbewerb *wettbewerb);
private:

	static bool istJugend(DatabaseContext *context, Wettbewerb *wettbewerb, TeilnehmerErgebnisJoinModel *joinModel);
	static bool istErwachsener(DatabaseContext *context, Wettbewerb *wettbewerb, TeilnehmerErgebnisJoinModel *joinModel);

	static void berechneAnnahmen(DatabaseContext *context, std::vector<BilderNachWettbewerbJoinModel*> joinModels, Wettbewerb *wettbewerb, Sparte *sparte, qlonglong jugendCodeId);


};
#endif
