#include "WertungsHelper.h"

#include <algorithm>
#include <stdlib.h>
#include <vector>

#include "BildController.h"
#include "BilderNachWettbewerbJoinController.h"
#include "BilderNachWettbewerbJoinModel.h"
#include "BilderNachWettbewerbJoinSearchModel.h"
#include "BilderSucheJoinController.h"
#include "BilderSucheJoinModel.h"
#include "BilderSucheJoinSearchModel.h"
#include "DataHelper.h"
#include "DurchgangController.h"
#include "EinzelWertungController.h"
#include "EinzelWertung.h"
#include "EinzelwertungsAnzeigeJoinModel.h"
#include "EinzelwertungsAnzeigeJoinSearchModel.h"
#include "EinzelwertungenExterneJurorenJoinController.h"
#include "EinzelwertungenMitgliederJoinController.h"
#include "JuriertBeiExternJoinController.h"
#include "JuriertBeiMitgliedJoinController.h"
#include "JuriertBeiPersonJoinModel.h"
#include "JuriertBeiPersonJoinSearchModel.h"
#include "SortHelper.h"
#include "Sparte.h"
#include "SparteController.h"
#include "SpartenJurierung.h"
#include "SpartenJurierungController.h"
#include "Wertung.h"
#include "WertungController.h"
#include "Wettbewerb.h"
#include "WettbewerbCache.h"

bool WertungsHelper::istErsterDurchgang(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong ersterCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt", "ERSTER_DURCHGANG")->getId();
	return durchgang->hasArt()
		&& durchgang->getArt() == ersterCodeId;
}

bool WertungsHelper::istZweiterDurchgang(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong zweiterSchlechtCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_SCHLECHTE")->getId();
	qlonglong zweiterMittelCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_MITTLERE")->getId();
	qlonglong zweiterGutCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_BESTE")->getId();
	return durchgang->hasArt()
		&& ( durchgang->getArt() == zweiterSchlechtCodeId
		  || durchgang->getArt() == zweiterMittelCodeId
		  || durchgang->getArt() == zweiterGutCodeId);
}

bool WertungsHelper::istJugendDurchgang(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong jugendJaId = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	return (durchgang->hasJugend() ? durchgang->getJugend() == jugendJaId : false);
}

bool WertungsHelper::istErwachsenenDurchgang(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong jugendJaId = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();
	return (durchgang->hasJugend() ? durchgang->getJugend() == jugendJaId : false);
}

Durchgang* WertungsHelper::getErsterDurchgang(DatabaseContext *context, Sparte *sparte, bool jugend) {
	CodeManager *codeManager = context->getCodeManager();
	qlonglong erwUndJugendId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "ERWACHSENE_UND_JUGEND")->getId();
	qlonglong jugendAlsErwId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "JUGEND_ALS_ERWACHSENE")->getId();
	qlonglong durchgangsModus = (sparte->hasDurchgangsModus() ? sparte->getDurchgangsModus() : erwUndJugendId);

	bool sucheJugend = false;
	bool sucheErwachsene = false;

	// Erstmal rausfinden ob wir einen Jugend-Durchgang oder einen Erwachsenen-Durchgang suchen
	if ((jugend && durchgangsModus == jugendAlsErwId) || !jugend) {
		sucheErwachsene = true;
	} else {
		sucheJugend = true;
	}

	DurchgangController controller = DurchgangController();
	std::vector<Durchgang*> durchgaenge = controller.getBySparte(context, sparte);
	for (std::vector<Durchgang*>::const_iterator it = durchgaenge.begin(); it != durchgaenge.end(); it++) {
		Durchgang *durchgang = *it;
		if (istErsterDurchgang(context, durchgang) && ((sucheJugend && istJugendDurchgang(context, durchgang)) || (!sucheJugend && istErwachsenenDurchgang(context, durchgang)))) {
			return durchgang;
		}
	}
	return NULL;
}

bool WertungsHelper::kannWertungStarten(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong ersterCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt", "ERSTER_DURCHGANG")->getId();
	qlonglong zweiterSchlechtCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_SCHLECHTE")->getId();
	qlonglong zweiterMittelCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_MITTLERE")->getId();
	qlonglong zweiterGutCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt",
																			 "ZWEITER_DURCHGANG_BESTE")->getId();
	return durchgang->hasArt()
		&& ( durchgang->getArt() == ersterCodeId
		  || durchgang->getArt() == zweiterSchlechtCodeId
		  || durchgang->getArt() == zweiterMittelCodeId
		  || durchgang->getArt() == zweiterGutCodeId);

}

bool WertungsHelper::istAbgeschlosseneWertung(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong abgeschlossenCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsStatus", "BEENDET")->getId();
	return durchgang->hasStatus() && durchgang->getStatus() == abgeschlossenCodeId;
}

bool WertungsHelper::istNeueWertung(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong angelegtCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsStatus", "ANGELEGT")->getId();
	return durchgang->hasStatus() && durchgang->getStatus() == angelegtCodeId;
}

bool WertungsHelper::istHoechsterDurchgang(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong codeId = codeManager->getCodeValueByTypeAndName("DurchgangsArt", "ZWEITER_DURCHGANG_BESTE")->getId();
	return durchgang->hasArt() && durchgang->getArt() == codeId;
}

bool WertungsHelper::istBasisDurchgangAbgeschlossen(DatabaseContext *databaseContext, Durchgang *durchgang) {
	if (durchgang->hasReferenzDurchgang()) {
		qlonglong referenzId = durchgang->getReferenzDurchgang();
		DurchgangController durchgangController = DurchgangController();
		Durchgang *referenzDurchgang = durchgangController.getByPrimaryKey(databaseContext, referenzId);
		return istAbgeschlosseneWertung(databaseContext, referenzDurchgang);
	} else {
		// Ein nicht vorhandener Referenzdurchgang kann nicht offen sein.
		return true;
	}
}

void WertungsHelper::setzeWertungenZurueck(DatabaseContext *databaseContext, Durchgang *durchgang) {
	WertungController wertungController = WertungController();
	EinzelWertungController einzelWertungController = EinzelWertungController();
	std::vector<Wertung*> wertungen = wertungController.getByDurchgang(databaseContext, durchgang);
	for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		Wertung *wertung = *it;
		std::vector<EinzelWertung*> einzelWertungen = einzelWertungController.getByWertung(databaseContext, wertung);
		for (std::vector<EinzelWertung*>::const_iterator it2 = einzelWertungen.begin(); it2 != einzelWertungen.end(); it2++) {
			EinzelWertung *einzelWertung = *it2;
			einzelWertung->deleteBewertung();
			einzelWertungController.insertOrUpdate(databaseContext, einzelWertung);
		}
		DataHelper::deleteVectorContents(einzelWertungen);

		wertung->deleteBewertung();
		wertungController.insertOrUpdate(databaseContext, wertung);
	}

	DataHelper::deleteVectorContents(wertungen);
}

void WertungsHelper::setzeDurchgangAufGestartet(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong wirdJuriertCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsStatus", "WIRD_JURIERT")
			->getId();
	durchgang->setStatus(wirdJuriertCodeId);
	DurchgangController durchgangController = DurchgangController();
	durchgangController.insertOrUpdate(databaseContext, durchgang);
}

void WertungsHelper::setzeDurchgangAufAbgeschlossen(DatabaseContext *databaseContext, Durchgang *durchgang) {
	CodeManager *codeManager = databaseContext->getCodeManager();
	qlonglong wirdJuriertCodeId = codeManager->getCodeValueByTypeAndName("DurchgangsStatus", "BEENDET")
			->getId();
	durchgang->setStatus(wirdJuriertCodeId);
	DurchgangController durchgangController = DurchgangController();
	durchgangController.insertOrUpdate(databaseContext, durchgang);
}

void WertungsHelper::loescheWertungen(DatabaseContext *context, Durchgang *durchgang) {
	WertungController wertungController = WertungController();
	EinzelWertungController einzelWertungController = EinzelWertungController();
	std::vector<Wertung*> wertungen = wertungController.getByDurchgang(context, durchgang);
	for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		Wertung *wertung = *it;
		std::vector<EinzelWertung*> einzelWertungen = einzelWertungController.getByWertung(context, wertung);
		einzelWertungController.remove(context, einzelWertungen);
		DataHelper::deleteVectorContents(einzelWertungen);
	}
	wertungController.remove(context, wertungen);
	DataHelper::deleteVectorContents(wertungen);
}

bool WertungsHelper::filtereNachJugend(DatabaseContext *context, Wettbewerb *wettbewerb, Sparte *sparte, Durchgang *durchgang, BilderNachWettbewerbJoinModel *joinModel) {
	// Siehe auch WettbewerbsHelper.istMitgliedJugend bei Aenderungen

	// Ein hinreichend fruehes Datum um alles als Jugend zuzulassen, falls das Feld nicht ausgefuellt ist
	QDate jugendAb = (wettbewerb->hasJugendAb() ? wettbewerb->getJugendAb() : QDate::fromString("02.01.1000", "dd.MM.yyyy"));

	CodeManager *codeManager = context->getCodeManager();

	qlonglong erwUndJugendId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "ERWACHSENE_UND_JUGEND")->getId();
//	qlonglong nurErwId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_ERWACHSENE")->getId();
//	qlonglong nurJugendId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "NUR_JUGEND")->getId();
	qlonglong jugendAlsErwId = codeManager->getCodeValueByTypeAndName("DurchgangsModus", "JUGEND_ALS_ERWACHSENE")->getId();

	qlonglong durchgangsModus = (sparte->hasDurchgangsModus() ? sparte->getDurchgangsModus() : erwUndJugendId);
	if (durchgangsModus == jugendAlsErwId) {
		// Wenn Jugendliche bei den Erwachsenen gewertet werden, wird sowieso alles zusammengeworfen, und wir brauchen hier nicht filtern.
		return true;
	}

	qlonglong jugendJaId = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	qlonglong jugendNeinId = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();

	QDate geburtsdatum = (joinModel->hasMitgliedDatumGeburt() ? joinModel->getMitgliedDatumGeburt() : QDate::fromString("01.01.1000", "dd.MM.yyyy"));
	qlonglong jugend = (joinModel->hasMitgliedJugend() ? joinModel->getMitgliedJugend() : jugendNeinId);
	bool istJugend = (geburtsdatum >= jugendAb && jugend == jugendJaId);
	bool jugendDurchgang = (durchgang->hasJugend() ? durchgang->getJugend() == jugendJaId : false);

//	std::cout << "Teste Bild " << qPrintable(joinModel->hasBildTitel() ? joinModel->getBildTitel() : "(ohne Titel)") << std::endl;
//	std::cout << "Fuer Durchgang mit Jugend = " << jugendDurchgang << std::endl;
//	std::cout << "Geburtsdatum ist " << qPrintable(geburtsdatum.toString()) << ", jugendAb = " << qPrintable(jugendAb.toString()) << std::endl;
//	std::cout << "Somit istJugend = " << istJugend << std::endl;

	bool result = (jugendDurchgang && istJugend) || (!jugendDurchgang && !istJugend);
//	std::cout << "Ergebnis ist also " << result << std::endl;
	return result;
}

void WertungsHelper::generiereWertungen(DatabaseContext *context, Wettbewerb *wettbewerb, Durchgang *durchgang) {
	CodeManager *codeManager = context->getCodeManager();
	QString durchgangsArtString = (durchgang->hasArt() ? codeManager->getCodeValueById(durchgang->getArt())->getName() : "(ohne Durchgangs-Art)");
	QString wettbewerbName = (wettbewerb->hasName() ? wettbewerb->getName() : "(ohne Name)");
	QString wettbewerbNummer = (wettbewerb->hasNummer() ? wettbewerb->getNummer() : "(ohne Nummer)");
	context->log("Wertung", "WertungsHelper", 8, "Beginne Wertung fuer Durchgang mit Namen " + durchgangsArtString + " des Wettbewerbs "
				  + wettbewerbName + " mit Nummer " + wettbewerbNummer);

	if (!istNeueWertung(context, durchgang)) {
		// maybe TODO error handling
	} else {
		context->log("Wertung", "WertungsHelper", 4, "  Durchgang ist im Status Neu");
		std::map<qlonglong, BilderNachWettbewerbJoinModel*> bildIdToBild
				= std::map<qlonglong, BilderNachWettbewerbJoinModel*>();

		qlonglong sparteId = durchgang->getSparte();
		SparteController sparteController = SparteController();
		Sparte *sparte = sparteController.getByPrimaryKey(context, sparteId);

		if (durchgang->hasReferenzDurchgang()) {
			context->log("Wertung", "WertungsHelper", 4, "  Durchgang hat einen Referenz-Durchgang");

			// Bilder des Referenzdurchgangs gemäß Filterkriterien auswählen
			// Der Referenzdurchgang ist grundsätzlich für dieselbe Sparte und
			// diesselbe Jugend-Einstellung.  Das heißt, um das Filtern von
			// Sparte bzw. Jugend müssen wir uns hier nicht mehr kümmern
			BilderSucheJoinSearchModel *searchModel = new BilderSucheJoinSearchModel();
			searchModel->setDurchgangId(durchgang->getReferenzDurchgang());
			BilderSucheJoinController controller = BilderSucheJoinController();
			std::vector<BilderSucheJoinModel*> results = controller.get(context, searchModel);
			delete searchModel;

			context->log("Wertung", "WertungsHelper", 4, "  Referenzdurchgang hat " + QString::number(results.size()) + " Bilder");

			std::map<qlonglong, BilderSucheJoinModel*> alleBildIdToBild = std::map<qlonglong, BilderSucheJoinModel*>();

			// dazu erstmal alle Bilder des Referenzdurchgangs holen
			for (std::vector<BilderSucheJoinModel*>::const_iterator it = results.begin(); it != results.end(); it++) {
				BilderSucheJoinModel *joinModel = *it;
				alleBildIdToBild[joinModel->getBildId()] = joinModel;
			}
			// dann die entsprechenden Wertungen extrahieren
			std::vector<Wertung*> wertungen = std::vector<Wertung*>();
			for (std::vector<BilderSucheJoinModel*>::const_iterator it = results.begin(); it != results.end(); it++) {
				BilderSucheJoinModel *joinModel = *it;
				wertungen.push_back(BilderSucheJoinController::extractWertung(joinModel));
			}
			// auf Basis der Wertungen auswählen, welche für diesen Durchgang relevant sind
			// dabei die Wertungs-Instanzen löschen, die nicht genommen werden
			std::vector<Wertung*> relevanteWertungen = filterWertungen(context, wertungen, durchgang);
			// und Schlussendlich die entsprechenden Bilder sichern

			context->log("Wertung", "WertungsHelper", 4, "  " + QString::number(relevanteWertungen.size()) + " Wertungen des Referenzdurchgangs sind relevant fuer diesen Durchgang");
			for (std::vector<Wertung*>::const_iterator it = relevanteWertungen.begin();
				  it != relevanteWertungen.end(); it++) {
				Wertung *wertung = *it;
				BilderSucheJoinModel *bilderSuche = alleBildIdToBild[wertung->getBild()];
				BilderNachWettbewerbJoinModel *bilderNachWettbewerb = new BilderNachWettbewerbJoinModel();
				WettbewerbsTeilnahme *teilnahme = BilderSucheJoinController::extractWettbewerbsTeilnahme(bilderSuche);
				Wettbewerb *wettbewerb = BilderSucheJoinController::extractWettbewerb(bilderSuche);
				Mitglied *mitglied = BilderSucheJoinController::extractMitglied(bilderSuche);
				Bild *bild = BilderSucheJoinController::extractBild(bilderSuche);

				BilderNachWettbewerbJoinController::copyWettbewerbsTeilnahmeInto(bilderNachWettbewerb, teilnahme);
				BilderNachWettbewerbJoinController::copyWettbewerbInto(bilderNachWettbewerb, wettbewerb);
				BilderNachWettbewerbJoinController::copyMitgliedInto(bilderNachWettbewerb, mitglied);
				BilderNachWettbewerbJoinController::copyBildInto(bilderNachWettbewerb, bild);

				delete teilnahme;
				delete wettbewerb;
				delete mitglied;
				delete bild;

				bildIdToBild[wertung->getBild()] = bilderNachWettbewerb;
				delete wertung;
			}

			context->log("Wertung", "WertungsHelper", 4, "  Joinergebnisse kopiert.");

			// Speicher aufräumen - Nicht genommene BilderSucheJoinModels löschen
			for (std::map<qlonglong, BilderSucheJoinModel*>::iterator it = alleBildIdToBild.begin();
				  it != alleBildIdToBild.end(); it++) {
				delete it->second;
			}
		} else {
			context->log("Wertung", "WertungsHelper", 4, "  Durchgang hat keinen Referenzdurchgang");

			// Alle Bilder des Wettbewerbs nehmen, die der Sparte und der
			// Jugend-Einstellung des Durchgangs entsprechen
			qlonglong wettbewerbsId = wettbewerb->getId();
			BilderNachWettbewerbJoinSearchModel *searchModel = new BilderNachWettbewerbJoinSearchModel();
			searchModel->setWettbewerbId(wettbewerbsId);
			searchModel->setBildSparte(durchgang->getSparte());
			searchModel->setBildGueltig(true);

			BilderNachWettbewerbJoinController controller = BilderNachWettbewerbJoinController();
			std::vector<BilderNachWettbewerbJoinModel*> results = controller.get(context, searchModel);
			delete searchModel;

			context->log("Wertung", "WertungsHelper", 4, "  " + QString::number(results.size()) + " Bilder haben die richtige Sparte und die richtige Jugend-Einstellung.");

			for (std::vector<BilderNachWettbewerbJoinModel*>::const_iterator it = results.begin(); 
						  it != results.end(); it++) {
				BilderNachWettbewerbJoinModel *joinModel = *it;
				if (filtereNachJugend(context, wettbewerb, sparte, durchgang, joinModel)) {
					bildIdToBild[joinModel->getBildId()] = joinModel;
				} else {
					delete joinModel;
				}
			}
		}

		std::vector<QAbstractListModel*> modelVector = std::vector<QAbstractListModel*>();
		std::vector<BilderNachWettbewerbJoinModel*> tmpBilder = std::vector<BilderNachWettbewerbJoinModel*>();
		for (std::map<qlonglong, BilderNachWettbewerbJoinModel*>::const_iterator it = bildIdToBild.begin(); it != bildIdToBild.end(); it++) {
			modelVector.push_back(it->second);
			tmpBilder.push_back(it->second);
		}

		context->log("Wertung", "WertungsHelper", 4, "  Beginne mit Sortierung der Bilder gemaess Einstellung...");

		std::vector<BilderNachWettbewerbJoinModel*> sortierteBilder = std::vector<BilderNachWettbewerbJoinModel*>();

		CodeManager *codeManager = context->getCodeManager();
		if (!durchgang->hasSortierModus() || codeManager->isCodeValue(durchgang->getSortierModus(), "SpartenSortierModus", "ZUFAELLIG")) {
			context->log("Wertung", "WertungsHelper", 4, "    Sortiere zufaellig.");
			while(tmpBilder.size() > 0) {
				int currPosition = WertungsHelper::getRandomNumber(int(tmpBilder.size()));
				sortierteBilder.push_back(tmpBilder[currPosition]);
				tmpBilder.erase(tmpBilder.begin() + currPosition);
			}
		} else if (codeManager->isCodeValue(durchgang->getSortierModus(), "SpartenSortierModus", "VEREIN_MITGLIED")) {
			context->log("Wertung", "WertungsHelper", 4, "    Sortiere nach Mitgliedsnummer aufsteigend.");
			SortHelper::sort(modelVector, BilderNachWettbewerbJoinModel::MITGLIEDMITGLIEDSNUMMERKOMPLETT, SortHelper::STANDARD_STRING_SORT, Qt::AscendingOrder);

			for (std::vector<QAbstractListModel*>::const_iterator it = modelVector.begin(); it != modelVector.end(); it++) {
				BilderNachWettbewerbJoinModel *joinModel = qobject_cast<BilderNachWettbewerbJoinModel*>(*it);
				std::cout << "Ergaenze Mitgliedsnr " << qPrintable(joinModel->getMitgliedMitgliedsNummerKomplett()) << std::endl;
				sortierteBilder.push_back(joinModel);
			}
//		} else if (codeManager->isCodeValue(durchgang->getSortierModus(), "SpartenSortierModus", "BILD")) {
//			context->log("Wertung", "WertungsHelper", 4, "    Sortiere nach Bildnummer aufsteigend.");
//			SortHelper::sort(modelVector, BilderNachWettbewerbJoinModel::BILD, SortHelper::STANDARD_STRING_SORT, Qt::AscendingOrder);
		} else {
			// TODO Error handling
		}

		context->log("Wertung", "WertungsHelper", 4, "  Sortierung beendet, lege nun Wertungen und Einzelwertungen an...");

		int aktuellePosition = 1;
		for (std::vector<BilderNachWettbewerbJoinModel*>::const_iterator it = sortierteBilder.begin();
			it != sortierteBilder.end(); it++) {
			BilderNachWettbewerbJoinModel* joinModel = *it;

			Wertung *wertung = new Wertung();
			wertung->setDurchgang(durchgang->getId());
			wertung->setBild(joinModel->getBildId());
			wertung->setPositionImDurchgang(aktuellePosition);
			QString bildTitel = (joinModel->hasBildTitel() ? joinModel->getBildTitel() : QObject::trUtf8("(kein Titel)"));
			std::cout << "Setze Bild " << qPrintable(bildTitel)
					  << " an Position " << aktuellePosition << std::endl;

			WertungController wertungController = WertungController();
			wertungController.insertOrUpdate(context, wertung);

			aktuellePosition++;
		}

		context->log("Wertung", "WertungsHelper", 4, "  ... Anlegen beendet.");

		DataHelper::deleteVectorContents(sortierteBilder);

		DurchgangController durchgangController = DurchgangController();
		durchgang->setReihenfolgeGeneriert(true);
		durchgangController.insertOrUpdate(context, durchgang);
	}
}

void WertungsHelper::beginneWertung(DatabaseContext *context, Wettbewerb *wettbewerb,
									Durchgang *durchgang) {

	if (!istNeueWertung(context, durchgang)) {
		// maybe TODO error handling
	} else {
		qlonglong sparteId = durchgang->getSparte();
		SparteController sparteController = SparteController();
		Sparte *sparte = sparteController.getByPrimaryKey(context, sparteId);

		WertungController wertungController = WertungController();
		std::vector<Wertung*> wertungen = wertungController.getByDurchgang(context, durchgang);
		for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
			Wertung *wertung = *it;
			legeEinzelwertungenAn(context, wertung, wettbewerb, sparte);
			delete wertung;
		}

		delete sparte;

		setzeDurchgangAufGestartet(context, durchgang);

		context->log("Wertung", "WertungsHelper", 8, "Durchgang wurde nun auf Gestartet gesetzt.");
	}
}

std::vector<Wertung*> WertungsHelper::filterWertungen(DatabaseContext *context,
													  std::vector<Wertung*> wertungen, Durchgang *durchgang) {
	context->log("Wertung", "WertungsHelper", 4, "  Filtere Wertungen gemaess Durchgangseinstellungen...");

	std::vector<Wertung*> retWertungen = std::vector<Wertung*>();
	// Es werden nur Bilder ausgewählt, für die es im Durchgang einen Anteil von mindestens minRang schlechterer
	// Bilder gibt.  Dazu auch Umrechnung von Prozent auf Anteil.
	double minRang = (durchgang->hasMinRang() ? durchgang->getMinRang() / 100 : 0);
	double maxRang = (durchgang->hasMaxRang() ? durchgang->getMaxRang() / 100 : 1);

	context->log("Wertung", "WertungsHelper", 4, "  MinRang = " + QString::number(minRang) + "; MaxRang = " + QString::number(maxRang));

	std::vector<double> bewertungen = std::vector<double>();
	for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		Wertung *wertung = *it;
		if (wertung->hasBewertung()) {
			context->log("Wertung", "WertungsHelper", 2, "    Registiere Bewertung " + QString::number(wertung->getBewertung()));
			bewertungen.push_back(wertung->getBewertung());
		}
	}

	sort(bewertungen.begin(), bewertungen.end());
	context->log("Wertung", "WertungsHelper", 1, "    Nach Sortieren liegen " + QString::number(bewertungen.size()) + " Bewertungen vor.");

	for (std::vector<double>::const_iterator it = bewertungen.begin(); it != bewertungen.end(); it++) {
		context->log("Wertung", "WertungsHelper", 1, "      Bewertung " + QString::number(*it));
	}

	if (bewertungen.size() == 0) {
		context->log("Wertung", "WertungsHelper", 4, "    Keine Bewertung vorhanden, also leerer Rueckgabevektor.");
		return std::vector<Wertung*>();
	} else {
		int minIndex = int(minRang * double(bewertungen.size() - 1) + 0.5);
		int maxIndex = int(maxRang * double(bewertungen.size() - 1) + 0.5);
		context->log("Wertung", "WertungsHelper", 4, "    MinIndex = " + QString::number(minIndex) + "; MaxIndex = " + QString::number(maxIndex));

		double minAbsoluteRang = bewertungen[minIndex];
		double maxAbsoluteRang = bewertungen[maxIndex];

		context->log("Wertung", "WertungsHelper", 4, "    MinAbsoluteRang = " + QString::number(minAbsoluteRang) + "; MaxAbsoluteRang = " + QString::number(maxAbsoluteRang));

		bool hoechster = istHoechsterDurchgang(context, durchgang);
		for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
			Wertung *wertung = *it;
			if (wertung->hasBewertung() && wertung->getBewertung() >= minAbsoluteRang
					&& ((hoechster && wertung->getBewertung() <= maxAbsoluteRang)
							|| (!hoechster && wertung->getBewertung() < maxAbsoluteRang))) {

				context->log("Wertung", "WertungsHelper", 4, "    Waehle Bewertung " + QString::number(wertung->getBewertung()) + " aus.");

				retWertungen.push_back(wertung);
			} else {
				delete wertung;
			}
		}


		context->log("Wertung", "WertungsHelper", 4, "    Ausgewaehlte Bewerungen:");

		std::cout << "Gewaehlte Bewertungen: ";
		for (std::vector<Wertung*>::const_iterator it = retWertungen.begin(); it != retWertungen.end(); it++) {
			Wertung *wertung = *it;
			if (wertung->hasBewertung()) {
				context->log("Wertung", "WertungsHelper", 4, "      Bewertung " + QString::number(wertung->getBewertung()));
			} else {
				context->log("Wertung", "WertungsHelper", 9, "Wertung ohne Bewertung, id " + (wertung->hasId() ? QString::number(wertung->getId()) : "(ohne Id)"));
			}
		}

		context->log("Wertung", "WertungsHelper", 4, "    filtereWertungen gibt " + QString::number(retWertungen.size()) + " Wertungen zurueck.");
		return retWertungen;
	}
}

void WertungsHelper::ladeWertungen(DatabaseContext *context, WettbewerbCache *wettbewerbCache,
								   Durchgang *durchgang) {
	context->log("Wertung", "WertungsHelper", 8, "Lade Wertungen...");

	// get rid of any old stuff
	wettbewerbCache->forgetWertungen(durchgang);

	WertungController wertungController = WertungController();
	std::vector<Wertung*> wertungen = wertungController.getByDurchgang(context, durchgang);
	for (std::vector<Wertung*>::const_iterator it = wertungen.begin(); it != wertungen.end(); it++) {
		wettbewerbCache->addWertung(*it);
	}

	context->log("Wertung", "WertungsHelper", 8, "Laden von " + QString::number(wertungen.size()) + " Wertungen beendet.");
}

bool WertungsHelper::setzeNaechsteOffeneWertung(DatabaseContext *context, WettbewerbCache *wettbewerbCache,
												Durchgang *durchgang) {
	context->log("Wertung", "WertungsHelper", 7, "Setze naechste noch offene Wertung...");

	WertungController wertungController = WertungController();

	bool allesJuriert = false;
	if (durchgang->hasAktuelleWertung()) {
		context->log("Wertung", "WertungsHelper", 4, "  Durchgang hat schon eine aktuelle Wertung");
		qlonglong alteWertungsId = durchgang->getAktuelleWertung();
		Wertung *alteWertung = wettbewerbCache->getWertung(alteWertungsId);
		if (alteWertung == NULL) {
			WertungController wertungController = WertungController();
			alteWertung = wertungController.getByPrimaryKey(context, alteWertungsId);
		}
		context->log("Wertung", "WertungsHelper", 4, "  Aktuelle Wertung hat die ID " + (alteWertung->hasId() ? QString::number(alteWertung->getId()) : "(ohne ID)"));

		if (alteWertung->hasBewertung()) {
			context->log("Wertung", "WertungsHelper", 4, "    Aktuelle Wertung hat schon eine Bewertung");

			// diese Wertung ist schon durch, wir brauchen die nächste
			int altePosition = alteWertung->getPositionImDurchgang();
			int neuePosition = altePosition + 1;
			Wertung *neueWertung;
			do {
				// nichts aus Versehen zweimal jurieren
				neueWertung = wettbewerbCache->getNaechsteWertung(durchgang, neuePosition);
				neuePosition++;
			} while (neueWertung != NULL && neueWertung->hasBewertung());
			if (neueWertung == NULL) {
				// alles juriert
				durchgang->deleteAktuelleWertung();
				allesJuriert = true;
				context->log("Wertung", "WertungsHelper", 7, "  Alle Wertungen mit Bewertung versehen.");
			} else {
				// nächste Wertung gefunden
				durchgang->setAktuelleWertung(neueWertung->getId());
				allesJuriert = false;
				context->log("Wertung", "WertungsHelper", 4, "  Waehle Wertung " + QString::number(neueWertung->getId()));
			}
		} else {
			// Nichts zu tun, die Wertung für das aktuelle Bild wurde noch nicht durchgeführt
			allesJuriert = false;
			context->log("Wertung", "WertungsHelper", 7, "  Alle Wertungen mit Bewertung versehen.");
		}
	} else {
		context->log("Wertung", "WertungsHelper", 4, "  Durchgang hat noch keine aktuelle Wertung, suche sie...");

		// wir brauchen die erste Wertung des gegebenen Durchgangs
		Wertung *neueWertung = wettbewerbCache->getNaechsteWertung(durchgang, 0);
		if (neueWertung == NULL || neueWertung->hasBewertung()) {
			// Keine Wertung gefunden / ausstehend
			durchgang->deleteAktuelleWertung();
			allesJuriert = true;
			context->log("Wertung", "WertungsHelper", 7, "  Keine Wertung gefunden.");
		} else {
			durchgang->setAktuelleWertung(neueWertung->getId());

			allesJuriert = false;
			context->log("Wertung", "WertungsHelper", 7, "  Wertung " + QString::number(neueWertung->getId()) + " gewaehlt.");;
		}
	}

	DurchgangController durchgangController = DurchgangController();
	durchgangController.insertOrUpdate(context, durchgang);

	QString allesJuriertString = (allesJuriert ? "" : "nicht");
	context->log("Wertung", "WertungsHelper", 7, "Naechste Wertung ausgewaehlt, es wurde damit " + allesJuriertString + " alles juriert.");
	return allesJuriert;
}

std::vector<JuriertBeiPersonJoinModel*> WertungsHelper::ladeJuroren(DatabaseContext *context,
																	Wettbewerb *wettbewerb, Sparte *sparte) {
	context->log("Wertung", "WertungsHelper", 7, "Lade Juroren...");
	JuriertBeiPersonJoinSearchModel *searchModel = new JuriertBeiPersonJoinSearchModel();
	searchModel->setJuriertBeiWettbewerb(wettbewerb->getId());

	JuriertBeiMitgliedJoinController mitgliedController = JuriertBeiMitgliedJoinController();
	std::vector<JuriertBeiPersonJoinModel*> mitglieder = mitgliedController.get(context, searchModel);

	JuriertBeiExternJoinController externController = JuriertBeiExternJoinController();
	std::vector<JuriertBeiPersonJoinModel*> externeJuroren = externController.get(context, searchModel);

	SpartenJurierungController spartenJurierungController = SpartenJurierungController();

	std::vector<JuriertBeiPersonJoinModel*> juroren = std::vector<JuriertBeiPersonJoinModel*>();
	for (std::vector<JuriertBeiPersonJoinModel*>::const_iterator it = mitglieder.begin();
																 it != mitglieder.end(); it++) {
		JuriertBeiPersonJoinModel *juror = *it;
		if (sparte != NULL) {
			JuriertBei *juriertBei = JuriertBeiMitgliedJoinController::extractJuriertBei(juror);
			if (juriertBei->hasAlleSparten() && juriertBei->getAlleSparten()) {
				juroren.push_back(juror);
			} else {
				std::vector<SpartenJurierung*> spartenJurierungen = spartenJurierungController.getByJuriertBei(context, juriertBei);
				for (std::vector<SpartenJurierung*>::const_iterator it2 = spartenJurierungen.begin(); it2 != spartenJurierungen.end(); it2++) {
					SpartenJurierung *spartenJurierung = *it2;
					if (spartenJurierung->hasSparte() && spartenJurierung->getSparte() == sparte->getId()) {
						juroren.push_back(juror);
						break;
					}
				}
			}
			delete juriertBei;
		} else {
			juroren.push_back(juror);
		}
	}
	for (std::vector<JuriertBeiPersonJoinModel*>::const_iterator it = externeJuroren.begin(); 
																 it != externeJuroren.end(); it++) {
		JuriertBeiPersonJoinModel *juror = *it;
		if (sparte != NULL) {
			JuriertBei *juriertBei = JuriertBeiMitgliedJoinController::extractJuriertBei(juror);
			if (juriertBei->hasAlleSparten() && juriertBei->getAlleSparten()) {
				juroren.push_back(juror);
			} else {
				std::vector<SpartenJurierung*> spartenJurierungen = spartenJurierungController.getByJuriertBei(context, juriertBei);
				for (std::vector<SpartenJurierung*>::const_iterator it2 = spartenJurierungen.begin(); it2 != spartenJurierungen.end(); it2++) {
					SpartenJurierung *spartenJurierung = *it2;
					if (spartenJurierung->hasSparte() && spartenJurierung->getSparte() == sparte->getId()) {
						juroren.push_back(juror);
						break;
					}
				}
			}
			delete juriertBei;
		} else {
			juroren.push_back(juror);
		}
	}

	delete searchModel;

	context->log("Wertung", "WertungsHelper", 7, QString::number(juroren.size()) + " Juroren geladen.");
	return juroren;
}

std::vector<EinzelwertungsAnzeigeJoinModel*> WertungsHelper::ladeEinzelwertungen(DatabaseContext *context, 
																				 Wertung *wertung) {

	context->log("Wertung", "WertungsHelper", 6, "Lade EinzelWertungen zu Wertung " + QString::number(wertung->getId()));

	EinzelwertungsAnzeigeJoinSearchModel *searchModel = new EinzelwertungsAnzeigeJoinSearchModel();
	qlonglong wertungsId = wertung->getId();
	searchModel->setEinzelWertungWertung(wertungsId);

	EinzelwertungenMitgliederJoinController mitgliedController = EinzelwertungenMitgliederJoinController();
	std::vector<EinzelwertungsAnzeigeJoinModel*> mitglieder = mitgliedController.get(context, searchModel);

	EinzelwertungenExterneJurorenJoinController externController = EinzelwertungenExterneJurorenJoinController();
	std::vector<EinzelwertungsAnzeigeJoinModel*> externeJuroren = externController.get(context, searchModel);

	std::vector<EinzelwertungsAnzeigeJoinModel*> einzelwertungen = std::vector<EinzelwertungsAnzeigeJoinModel*>();
	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = mitglieder.begin(); 
																 it != mitglieder.end(); it++) {
		einzelwertungen.push_back(*it);
	}
	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = externeJuroren.begin(); 
																 it != externeJuroren.end(); it++) {
		einzelwertungen.push_back(*it);
	}

	delete searchModel;

	context->log("Wertung", "WertungsHelper", 6, QString::number(einzelwertungen.size()) + " EinzelWertungen geladen.");
	return einzelwertungen;

}

void WertungsHelper::legeEinzelwertungenAn(DatabaseContext *context, Wertung *wertung, 
										   Wettbewerb *wettbewerb, Sparte *sparte) {
	context->log("Wertung", "WertungsHelper", 6, "Lege EinzelWertungen fuer Wertung mit ID " + (wertung->hasId() ? QString::number(wertung->getId()) : "(ohne ID)") + " an.");;
	std::vector<JuriertBeiPersonJoinModel*> juroren = ladeJuroren(context, wettbewerb, sparte);
	for (std::vector<JuriertBeiPersonJoinModel*>::const_iterator it = juroren.begin(); 
		it != juroren.end(); it++) {
		JuriertBeiPersonJoinModel* juror = *it;

		EinzelWertungController controller = EinzelWertungController();

		// Testen, ob die EinzelWertung schon existiert.
		EinzelWertungsAuswahlSearchModel *searchModel = new EinzelWertungsAuswahlSearchModel();
		searchModel->setWertung(wertung->getId());
		searchModel->setJuror(juror->getJuriertBeiId());
		std::vector<EinzelWertung*> einzelWertungen = controller.get(context, searchModel);
		if (einzelWertungen.size() == 0) {
			EinzelWertung *einzelWertung = new EinzelWertung();
			einzelWertung->setWertung(wertung->getId());
			einzelWertung->setJuror(juror->getJuriertBeiId());
			controller.insertOrUpdate(context, einzelWertung);

			delete einzelWertung;
		} else {
			DataHelper::deleteVectorContents(einzelWertungen);
		}

		delete searchModel;
	}
	context->log("Wertung", "WertungsHelper", 6, "EinzelWertungen angelegt.");
}

bool WertungsHelper::istWertungKomplett(std::vector<EinzelwertungsAnzeigeJoinModel*> einzelwertungen) {
	bool komplett = true;
	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelwertungen.begin();
		it != einzelwertungen.end(); it++) {
		EinzelwertungsAnzeigeJoinModel* einzelwertung = *it;
		if (!einzelwertung->hasEinzelWertungBewertung()) {
			komplett = false;
		}
	}
	return komplett;
}

void WertungsHelper::aktualisiereGesamtwertung(DatabaseContext *context, Wertung *wertung, Bild *bild,
										  std::vector<EinzelwertungsAnzeigeJoinModel*> einzelwertungen) {
	int summe = 0;
	context->log("Wertung", "WertungsHelper", 6, "Aktualisiere Gesamtwertung von Bild " + QString::number(bild->getId()));

	for (std::vector<EinzelwertungsAnzeigeJoinModel*>::const_iterator it = einzelwertungen.begin();
		it != einzelwertungen.end(); it++) {
		EinzelwertungsAnzeigeJoinModel* einzelwertung = *it;
		if (einzelwertung->hasEinzelWertungBewertung()) {
			summe += einzelwertung->getEinzelWertungBewertung();
		}
	}

	// Die Wertungs-Instanz ist einem Durchgang zugeordnet und bleibt somit
	// dauerhaft erhalten.  Die Bewertung in der Bild-Instanz dagegen wird 
	// beim nächsten Durchgang überschrieben.
	wertung->setBewertung(summe);
	bild->setBewertung(summe);

	WertungController wertungController = WertungController();
	wertungController.insertOrUpdate(context, wertung);

	BildController bildController = BildController();
	bildController.insertOrUpdate(context, bild);

	context->log("Wertung", "WertungsHelper", 6, "Gesamtwertung von Bild " + QString::number(bild->getId()) + " aktualisiert, neue Summe ist " + QString::number(summe));
}

//static int getRandomNumber(int maxValue);
int WertungsHelper::getRandomNumber(int maxValue) {
	return int((double(rand()) / RAND_MAX) * maxValue);
}
