#include "DummyRSAManager.h"

#include <QByteArray>
#include <QDateTime>
#include <QTime>
#include <QString>

#include <iostream>
#include <stdlib.h>

#include "DataHelper.h"
#include "PrivateRSAKey.h"
#include "PublicRSAKey.h"

RSAKeyPair* DummyRSAManager::generateKeyPair() {
	QString privateKey = "";
	QString publicKey = "";
	initRandomGenerator();

	for (int n = 0; n < 7; n++) {
		privateKey += intToString(rand() % 36);
		publicKey += intToString(rand() % 36);
	}

	for (int n = 0; n < 16; n++) {
		int r = rand() % 36;
		privateKey += intToString(r);

		int rComp = 36 - r;
		publicKey += intToString(rComp);

		int gap = getKeyGap(r);
		for (int z = 0; z < gap; z++) {
			int r2 = rand() % 36;
			privateKey += intToString(r2);

			int r2Comp = rand() % 36;
			publicKey += intToString(r2Comp);
		}
	}

	// std::cout << "Generated private " << qPrintable(privateKey) << std::endl;
	// std::cout << "Generated public " << qPrintable(publicKey) << std::endl;


	PrivateRSAKey *privateRSAKey = new PrivateRSAKey(privateKey);
	PublicRSAKey *publicRSAKey = new PublicRSAKey(publicKey);

	return new RSAKeyPair(privateRSAKey, publicRSAKey);
}

PrivateRSAKey* DummyRSAManager::loadPrivateKey(QString representation) {
	return new PrivateRSAKey(representation);
}

PublicRSAKey* DummyRSAManager::loadPublicKey(QString representation) {
	return new PublicRSAKey(representation);
}

QString DummyRSAManager::encryptToHex(PrivateRSAKey *privateKey, QString data) {
	// std::cout << "Data is " << qPrintable(data) << std::endl;

	QByteArray dataAsByteArray = data.toAscii();
	QString hexData = DataHelper::toHexRepresentation(dataAsByteArray).toUpper();

	// std::cout << "Hex representation is " << qPrintable(hexData) << std::endl;

	QString privateKeyString = privateKey->getKey();
	std::vector<int> offsets = std::vector<int>();
	for (int n = 0; n < 16; n++) {
		QString p = privateKeyString.mid(7 + 5 * n, 1);
		offsets.push_back(stringToInt(p));
	}

	// std::cout << "Offsets are: ";
	for (int n = 0; n < 16; n++) {
		// std::cout << offsets[n] << " ";
	}
	// std::cout << std::endl;

	QString enc = "";
	int o = 5;
	for (int n = 0; n < hexData.size(); n++) {
		QString s = hexData.mid(n, 1);
		int sNumber = stringToInt(s);
		int sEncNumber = (sNumber + offsets[o]) % 36;
		o = (o + 1) % 16;
		QString sEnc = intToString(sEncNumber);
		enc += sEnc;

		// std::cout << "Converting hexData[" << n << "] = " << qPrintable(s) << " using offsets[" << o << "] = " << offsets[o] << " to " << qPrintable(sEnc) 
		//		  << " (sNumber = " << sNumber << ", sEncNumber = " << sEncNumber << ")" << std::endl;

		// std::cout << "Adding rubbish... ";
		for (int z = 0; z < sEncNumber / 3; z++) {
			int r = (rand() % 36);
			QString rString = intToString(r);
			// std::cout << qPrintable(rString);
			enc += rString;
		}
		// std::cout << std::endl;
	}

	// std::cout << "Finished. Result is " << qPrintable(enc) << std::endl;
	return enc;
}

QString DummyRSAManager::decryptFromHex(PublicRSAKey *publicKey, QString data) {
	// std::cout << "Decrypt: " << qPrintable(data) << std::endl;

	QString publicKeyString = publicKey->getKey();
	std::vector<int> offsets = std::vector<int>();
	for (int n = 0; n < 16; n++) {
		QString p = publicKeyString.mid(7 + 5 * n, 1);
		offsets.push_back(stringToInt(p));
	}

	// std::cout << "Offsets are: ";
	for (int n = 0; n < 16; n++) {
		// std::cout << offsets[n] << " ";
	}
	// std::cout << std::endl;

	QString dec = "";
	int o = 5;
	int r = 0;
	int lastNumber = 0;
	for (int n = 0; n < data.size(); n++) {
		if (r == 0) {
			QString s = data.mid(n, 1);
			int sNumber = stringToInt(s);
			int sDecNumber = (sNumber + offsets[o]) % 36;
			lastNumber = sNumber;
			o = (o + 1) % 16;
			QString sDec = intToString(sDecNumber);
			dec += sDec;

			r = lastNumber / 3;

			// std::cout << "Inspecting " << qPrintable(s) << ", sNumber = " << sNumber << ", offsets[" << o << "] = " << offsets[o] << ", thus sDecNumber = " << sDecNumber
			//		  << ", sDec = " << qPrintable(sDec) << std::endl;
		} else {
			r--;
			// std::cout << "Skipping rubbish " << qPrintable(data.mid(n, 1)) << std::endl;
		}

	}

	// std::cout << "Dec: " << qPrintable(dec) << std::endl;


	QByteArray dataAsByteArray = DataHelper::fromHexRepresentation(dec.toLower());
	QString retString = QString(dataAsByteArray);

	// std::cout << "Ret: " << qPrintable(retString) << std::endl;
	return retString;
}

void DummyRSAManager::initRandomGenerator() {
	QDateTime currentDateTime = QDateTime::currentDateTime();
	QTime currentTime = currentDateTime.time();
	unsigned int seed = (currentTime.second() * currentTime.msec() + 53) * 17;
	srand(seed);
}

int DummyRSAManager::stringToInt(QString s) {
	if (s == "0") {
		return 0;
	} else if (s == "1") {
		return 1;
	} else if (s == "2") {
		return 2;
	} else if (s == "3") {
		return 3;
	} else if (s == "4") {
		return 4;
	} else if (s == "5") {
		return 5;
	} else if (s == "6") {
		return 6;
	} else if (s == "7") {
		return 7;
	} else if (s == "8") {
		return 8;
	} else if (s == "9") {
		return 9;
	} else if (s == "A") {
		return 10;
	} else if (s == "B") {
		return 11;
	} else if (s == "C") {
		return 12;
	} else if (s == "D") {
		return 13;
	} else if (s == "E") {
		return 14;
	} else if (s == "F") {
		return 15;
	} else if (s == "G") {
		return 16;
	} else if (s == "H") {
		return 17;
	} else if (s == "I") {
		return 18;
	} else if (s == "J") {
		return 19;
	} else if (s == "K") {
		return 20;
	} else if (s == "L") {
		return 21;
	} else if (s == "M") {
		return 22;
	} else if (s == "N") {
		return 23;
	} else if (s == "O") {
		return 24;
	} else if (s == "P") {
		return 25;
	} else if (s == "Q") {
		return 26;
	} else if (s == "R") {
		return 27;
	} else if (s == "S") {
		return 28;
	} else if (s == "T") {
		return 29;
	} else if (s == "U") {
		return 30;
	} else if (s == "V") {
		return 31;
	} else if (s == "W") {
		return 32;
	} else if (s == "X") {
		return 33;
	} else if (s == "Y") {
		return 34;
	} else if (s == "Z") {
		return 35;
	}
   std::cout << "Error unrecognized string: " << qPrintable(s) << std::endl;
   return 0;
}

QString DummyRSAManager::intToString(int i) {
	switch (i) {
		case 0: return "0";
		case 1: return "1";
		case 2: return "2";
		case 3: return "3";
		case 4: return "4";
		case 5: return "5";
		case 6: return "6";
		case 7: return "7";
		case 8: return "8";
		case 9: return "9";
		case 10: return "A";
		case 11: return "B";
		case 12: return "C";
		case 13: return "D";
		case 14: return "E";
		case 15: return "F";
		case 16: return "G";
		case 17: return "H";
		case 18: return "I";
		case 19: return "J";
		case 20: return "K";
		case 21: return "L";
		case 22: return "M";
		case 23: return "N";
		case 24: return "O";
		case 25: return "P";
		case 26: return "Q";
		case 27: return "R";
		case 28: return "S";
		case 29: return "T";
		case 30: return "U";
		case 31: return "V";
		case 32: return "W";
		case 33: return "X";
		case 34: return "Y";
		case 35: return "Z";
		case 36: return "0";
   }
   std::cout << "Error unrecognized int: " << i << std::endl;
   return QString();
}

int DummyRSAManager::getKeyGap(int n) {
	return 4;;
}
