#ifndef QUERYLOGGER_H
#define QUERYLOGGER_H

#include <QString>

#include <map>
#include <vector>

#include "CodeManager.h"
#include "CodeValue.h"
#include "QueryLogSettings.h"

class DatabaseContext;

class QueryLogger {

public:
	QueryLogger();
	~QueryLogger();

	void disable();
	void initialize(DatabaseContext *context);

	/** Logs information about a query according to the given information.
	 *  @param location free-formed string giving a location in Code
	 *  @param numberOfAffectedRows number of affected rows as given by
	 *                              the result of the mysqlpp-call, -1 if not applicable/known
	 *  @param queryType type of the Query according to the QueryType CodeValue constants
	 *  @param info some information about the query
	 *  @param queryText text of the query - maybe without long parameter lists etc.
     *  @param entities comma separated list of all affected Entities (by code name)
	 */
	void log(QString location, int numberOfAffectedRows, QString queryType,  
			 QString info, QString queryText, QString entities, bool dbLoggingEnabled);

	void log(QString location, int numberOfAffectedRows, CodeValue* queryType,  
			 QString info, QString queryText, QString entities, bool dbLoggingEnabled);

private:
	DatabaseContext *context;

	std::map<qlonglong, QueryLogSettings*> settingsIdToSettings;
	std::multimap<QString, QueryLogSettings*> entityCodeNameToSettings;
	std::vector<QueryLogSettings*> settingsWithoutEntity;

	void deleteConfigObjects();

	/** Returns wether the given QueryLogSettings matches the given query type.
	 */
	bool queryTypeMatches(CodeManager *codeManager, QueryLogSettings *settings, CodeValue *queryType);
//	bool output(EventLogSettings* settings, EventLogCategory *category, QString location, int level);

	bool outputToDB(QueryLogSettings* settings);
	bool outputToStdOut(QueryLogSettings *settings);

	bool enabled;
};
#endif
