#include "WettbewerbsHelper.h"

#include "DataHelper.h"

#include "Bild.h"
#include "BildController.h"
#include "BildDaten.h"
#include "BildDatenController.h"
#include "Durchgang.h"
#include "DurchgangController.h"
#include "EinzelWertung.h"
#include "EinzelWertungController.h"
#include "JuriertBei.h"
#include "JuriertBeiController.h"
#include "Jurierung.h"
#include "JurierungController.h"
#include "Sparte.h"
#include "SparteController.h"
#include "TeilnehmerWertung.h"
#include "TeilnehmerWertungController.h"
#include "TeilnehmerKombinationsWertung.h"
#include "TeilnehmerKombinationsWertungController.h"
#include "VereinsWertung.h"
#include "VereinsWertungController.h"
#include "VereinsKombinationsWertung.h"
#include "VereinsKombinationsWertungController.h"
#include "Wertung.h"
#include "WertungController.h"
#include "Wettbewerb.h"
#include "WettbewerbController.h"
#include "WettbewerbsTeilnahme.h"
#include "WettbewerbsTeilnahmeController.h"

#include <vector>

void WettbewerbsHelper::loescheWettbewerb(DatabaseContext *context, Wettbewerb *wettbewerb) {
	QString name = (wettbewerb->hasName() ? wettbewerb->getName() : "(ohne Name)");
	QString nummer = (wettbewerb->hasNummer() ? wettbewerb->getNummer() : "(ohne Nummer)");
	context->log("Wettbewerb", "WettbewerbHelper", 8, "Loesche Wettbewerb " + name + " mit Nummer " + nummer);

	BildController bildController = BildController();
	BildDatenController bildDatenController = BildDatenController();
	DurchgangController durchgangController = DurchgangController();
	EinzelWertungController einzelWertungController = EinzelWertungController();
	JuriertBeiController juriertBeiController = JuriertBeiController();
	JurierungController jurierungController = JurierungController();
	SparteController sparteController = SparteController();
	TeilnehmerWertungController teilnehmerWertungController = TeilnehmerWertungController();
	TeilnehmerKombinationsWertungController teilnehmerKombinationsWertungController = TeilnehmerKombinationsWertungController();
	VereinsWertungController vereinsWertungController = VereinsWertungController();
	VereinsKombinationsWertungController vereinsKombinationsWertungController = VereinsKombinationsWertungController();
	WertungController wertungController = WertungController();
	WettbewerbController wettbewerbController = WettbewerbController();
	WettbewerbsTeilnahmeController wettbewerbsTeilnahmeController = WettbewerbsTeilnahmeController();

	std::vector<JuriertBei*> juriertBeiVector = juriertBeiController.getByWettbewerb(context, wettbewerb);
	juriertBeiController.remove(context, juriertBeiVector);

	std::vector<Sparte*> sparteVector = sparteController.getByWettbewerb(context, wettbewerb);
	for (std::vector<Sparte*>::const_iterator sparteIt = sparteVector.begin(); sparteIt != sparteVector.end(); sparteIt++) {
		Sparte* sparte = *sparteIt;
		std::vector<Bild*> bildVector = bildController.getBySparte(context, sparte);
		for (std::vector<Bild*>::const_iterator bildIt = bildVector.begin(); bildIt != bildVector.end(); bildIt++) {
			Bild *bild = *bildIt;
			if (bild->hasBildDaten()) {
				qlonglong bildDatenId = bild->getBildDaten();
				BildDaten *bildDaten = bildDatenController.getByPrimaryKey(context, bildDatenId);
				bildDatenController.remove(context, bildDaten);
				delete bildDaten;
			}
		}
		bildController.remove(context, bildVector);
		DataHelper::deleteVectorContents(bildVector);
	}
	sparteController.remove(context, sparteVector);
	DataHelper::deleteVectorContents(sparteVector);

	std::vector<WettbewerbsTeilnahme*> wettbewerbsTeilnahmeVector = wettbewerbsTeilnahmeController.getByWettbewerb(context, wettbewerb);
	wettbewerbsTeilnahmeController.remove(context, wettbewerbsTeilnahmeVector);
	DataHelper::deleteVectorContents(wettbewerbsTeilnahmeVector);

	std::vector<Jurierung*> jurierungVector = jurierungController.getByWettbewerb(context, wettbewerb);
	for (std::vector<Jurierung*>::const_iterator jurierungIt = jurierungVector.begin(); jurierungIt != jurierungVector.end(); jurierungIt++) {
		Jurierung *jurierung = *jurierungIt;
		std::vector<Durchgang*> durchgangVector = durchgangController.getByJurierung(context, jurierung);
		for (std::vector<Durchgang*>::const_iterator durchgangIt = durchgangVector.begin(); durchgangIt != durchgangVector.end(); durchgangIt++) {
			Durchgang *durchgang = *durchgangIt;
			std::vector<Wertung*> wertungVector = wertungController.getByDurchgang(context, durchgang);
			for (std::vector<Wertung*>::const_iterator wertungIt = wertungVector.begin(); wertungIt != wertungVector.end(); wertungIt++) {
				Wertung* wertung = *wertungIt;
				std::vector<EinzelWertung*> einzelWertungVector = einzelWertungController.getByWertung(context, wertung);
				einzelWertungController.remove(context, einzelWertungVector);
				DataHelper::deleteVectorContents(einzelWertungVector);
			}
			wertungController.remove(context, wertungVector);
			DataHelper::deleteVectorContents(wertungVector);
		}
		durchgangController.remove(context, durchgangVector);
		DataHelper::deleteVectorContents(durchgangVector);
	}
	jurierungController.remove(context, jurierungVector);
	DataHelper::deleteVectorContents(jurierungVector);

	std::vector<TeilnehmerWertung*> teilnehmerWertungVector = teilnehmerWertungController.getByWettbewerb(context, wettbewerb);
	teilnehmerWertungController.remove(context, teilnehmerWertungVector);
	DataHelper::deleteVectorContents(teilnehmerWertungVector);

	std::vector<TeilnehmerKombinationsWertung*> teilnehmerKombinationsWertungVector = teilnehmerKombinationsWertungController.getByWettbewerb(context, wettbewerb);
	teilnehmerKombinationsWertungController.remove(context, teilnehmerKombinationsWertungVector);
	DataHelper::deleteVectorContents(teilnehmerKombinationsWertungVector);

	std::vector<VereinsWertung*> vereinsWertungVector = vereinsWertungController.getByWettbewerb(context, wettbewerb);
	vereinsWertungController.remove(context, vereinsWertungVector);
	DataHelper::deleteVectorContents(vereinsWertungVector);

	std::vector<VereinsKombinationsWertung*> vereinsKombinationsWertungVector = vereinsKombinationsWertungController.getByWettbewerb(context, wettbewerb);
	vereinsKombinationsWertungController.remove(context, vereinsKombinationsWertungVector);
	DataHelper::deleteVectorContents(vereinsKombinationsWertungVector);

	wettbewerbController.remove(context, wettbewerb);

	context->log("Wettbewerb", "WettbewerbHelper", 8, "Loeschen des Wettbewerb " + name + " mit der Nummer " + nummer + " beendet.");
	delete wettbewerb;
}

bool WettbewerbsHelper::istMitgliedJugend(DatabaseContext *context, Wettbewerb *wettbewerb, Mitglied *mitglied) {
	// Siehe auch WertungsHelper.filtereNachJugend bei Aenderungen

	// Ein hinreichend fruehes Datum um alles als Jugend zuzulassen, falls das Feld nicht ausgefuellt ist
	QDate jugendAb = (wettbewerb->hasJugendAb() ? wettbewerb->getJugendAb() : QDate::fromString("02.01.1000", "dd.MM.yyyy"));

	CodeManager *codeManager = context->getCodeManager();
	qlonglong jugendJaId = codeManager->getCodeValueByTypeAndName("Jugend", "Ja")->getId();
	qlonglong jugendNeinId = codeManager->getCodeValueByTypeAndName("Jugend", "Nein")->getId();

	QDate geburtsdatum = (mitglied->hasDatumGeburt() ? mitglied->getDatumGeburt() : QDate::fromString("01.01.1000", "dd.MM.yyyy"));
	qlonglong jugend = (mitglied->hasJugend() ? mitglied->getJugend() : jugendNeinId);
	bool istJugend = (geburtsdatum >= jugendAb && jugend == jugendJaId);
	return istJugend;
}
